package com.Goldenstreet.App.GoldenStreet;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.List;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.protocol.HTTP;
import org.json.JSONException;
import org.json.JSONObject;

import com.Goldenstreet.App.R;
import com.LibreGeoSocial.Api.LibreGeoSocial;

import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.util.Log;
import android.widget.Toast;

/**
 * Clase auxiliar encargada de realizar la compra de una propiedad (calle) para asocirsela al equipo.
 * 
 * @author Rubn Quero Castrillo <rubenquero@gmail.com>
 * 
 */

public class AccionMenuComprar{
	private static String TAG = "AccionMenuComprar";
	
	private static Context context;
	private static Calle calle;	
	private static ProgressDialog progressDialog = null;
	
	public static void comprarCalleMapa(Context context, Calle calle){
		AccionMenuComprar.context = context;
		AccionMenuComprar.calle = calle;
		if (calle != null)
			comprarCalle();
	}		
	
	private static void comprarCalle(){
		AlertDialog.Builder builder = new AlertDialog.Builder(context);
		builder.setMessage(String.format(context.getString(R.string.infoComprar), calle.getNombre(), calle.getPrecio()))
		       .setCancelable(false)
    	       .setPositiveButton(context.getString(R.string.butSi), new DialogInterface.OnClickListener() {
		           public void onClick(DialogInterface dialog, int id) {
		        	   dialog.cancel();
		        	   String title = context.getString(R.string.espere);
	        	       String text = context.getString(R.string.comprarCalle);
	        			
	        	       AccionMenuComprar.progressDialog = ProgressDialog.show(context, title, text, true, false);
	        	       AccionMenuComprar.progressDialog.setIcon(R.drawable.ic_dialog_conexion);

			   	       Thread thread = new Thread(comprarCalle);
			   	       thread.start();
		           }
		       }).setNegativeButton(context.getString(R.string.butNo), new DialogInterface.OnClickListener() {
    	           public void onClick(DialogInterface dialog, int id) {
    	        	   dialog.cancel();
    	           }
    	       });
    	AlertDialog alert = builder.create();
    	alert.setTitle(context.getString(R.string.comprar));
    	alert.setIcon(R.drawable.ic_dialog_informacion);
    	alert.show();
	}
		
	private static Runnable comprarCalle = new Runnable(){
		@Override
		public void run(){
			Looper.prepare();
			conexionComprarCalle();
			Looper.myLooper().quit();
			Looper.loop();
		}
		
		private void showToastConexion(){			
			Toast toast = Toast.makeText(context,
					context.getString(R.string.errConexion),
					Toast.LENGTH_SHORT);
			toast.show();
		}
		
		private void showToastDatos(){
			Toast toast = Toast.makeText(context,
					context.getString(R.string.errDatos),
					Toast.LENGTH_SHORT);
			toast.show();
		}
	
		private void showToastAceptada(){
			Toast toast = Toast.makeText(context,
					String.format(context.getString(R.string.calleComprarAceptada), calle.getNombre()),
					Toast.LENGTH_LONG);
			toast.show();
		}
		
		private void showToastSinDinero(){
			Toast toast = Toast.makeText(context,
					String.format(context.getString(R.string.calleComprarSinDinero), calle.getNombre()),
					Toast.LENGTH_LONG);
			toast.show();
		}
		
		private void showToastRechazada(){
			Toast toast = Toast.makeText(context,
					String.format(context.getString(R.string.calleComprarRechazada), calle.getNombre()),
					Toast.LENGTH_LONG);
			toast.show();
		}
		
		public void conexionComprarCalle(){
			int resultado;
			try{
				DefaultHttpClient httpclient = new DefaultHttpClient();
				int idJuego = JuegoListar.getJuego().getIdentificador();
				int idEquipo = JuegoDatosListar.getEquipo().getIdentificador();
				int idCalle = AccionMenuComprar.calle.getId();
				URI uri = new URI(GoldenStreets.urlServer+"juego/"+idJuego+"/equipo/"+idEquipo+"/calle/"+idCalle+"/comprar/?format=json");

				HttpPost httpost = new HttpPost(uri);
				LibreGeoSocial LGS = LibreGeoSocial.getInstance();
				httpclient.setCookieStore(LGS.getCookieStore());
				
				List <NameValuePair> datos = new ArrayList <NameValuePair>();
	            datos.add(new BasicNameValuePair("callePrecio", Integer.toString(calle.getPrecio())));
	            httpost.setEntity(new UrlEncodedFormEntity(datos, HTTP.UTF_8));
	            
	            HttpResponse response = httpclient.execute(httpost);
	            HttpEntity entity = response.getEntity();
	            String str = Utils.deStreamAString(entity.getContent());
				resultado = procesarRespuestaCalleComprar(str);			

				handler.sendEmptyMessage(resultado);

			}catch(URISyntaxException urisex){
				Log.e(TAG, urisex.getMessage());
				handler.sendEmptyMessage(-1);
		    }catch (IOException e) {
		    	Log.e(TAG,e.getMessage());
		    	handler.sendEmptyMessage(-1);
		    }
		}
		
		private int procesarRespuestaCalleComprar(String str) {
			int ultCode;
	    	String ultMensaje;
	        try {
	        	JSONObject json;
				json = new JSONObject(str);
				
				if (json.has("code"))
					ultCode = json.getInt("code");
				else
					ultCode = -1;
				
				if (json.has("descripcion"))
					ultMensaje = json.getString("descripcion");
				else
					ultMensaje = null;
				
				if (ultCode != 200){
					if (ultCode == 400 && ultMensaje.toLowerCase().compareTo("sin dinero") == 0)
						return -3;
					if (ultCode == 400 && ultMensaje.toLowerCase().compareTo("calle con modificaciones") == 0)
						return -4;
					return -2;
				}
			} catch (JSONException e) {	
				ultCode = 500;
				ultMensaje = "Error in json parsed: " + e.getMessage();
				Log.e(TAG, ultMensaje);
				return -2;
			}
			return 0;
		}
		
	    private Handler handler = new Handler()
	    {
	    	@Override
	        public void handleMessage(Message msg)
	        {
	        	int result = msg.what;
	        	AccionMenuComprar.progressDialog.cancel();
	        	if(result == -4){
	        		showToastRechazada();
	        	}else if(result == -3){
	        		showToastSinDinero();
	        	}else if(result == -2){
	        		showToastDatos();
	        	}else if(result == -1){
	        		showToastConexion();
	        	}else if(result == 0){
	        		showToastAceptada();
	        	}
	        }
	    };
	};
}