package com.Goldenstreet.App.GoldenStreet;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.DefaultHttpClient;
import org.json.JSONException;
import org.json.JSONObject;

import com.Goldenstreet.App.R;
import com.LibreGeoSocial.Api.LibreGeoSocial;

import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.util.Log;
import android.widget.Toast;

/**
 * Clase auxiliar encargada de realizar el sabotaje a una propiedad (calle) de otro equipo participante.
 * 
 * @author Rubn Quero Castrillo <rubenquero@gmail.com>
 * 
 */

public class AccionMenuSabotear{
	private static String TAG = "AccionMenuSabotear";
	
	private static Context context;
	private static Calle calle;	
	private static Carta carta;
	private static ProgressDialog progressDialog = null;
	
	public static void sabotearCalleMapa(Context context, Calle calle){
		AccionMenuSabotear.context = context;
		AccionMenuSabotear.calle = calle;
		if (calle != null)
			sabotearCalle();
	}		
	
	private static void sabotearCalle(){
		carta = Carta.encontrarPorTipoSinUsar(CartaServicio.getCartasEquipo() ,Carta.SABOTEAR);
		if (carta != null){
			AlertDialog.Builder builder = new AlertDialog.Builder(context);
			builder.setMessage(String.format(context.getString(R.string.infoSabotear), calle.getNombre()))
			       .setCancelable(false)
	    	       .setPositiveButton(context.getString(R.string.butSi), new DialogInterface.OnClickListener() {
			           public void onClick(DialogInterface dialog, int id) {
			        	   dialog.cancel();
			        	   String title = context.getString(R.string.espere);
		        	       String text = context.getString(R.string.sabotearCalle);
		        			
		        	       AccionMenuSabotear.progressDialog = ProgressDialog.show(context, title, text, true, false);
		        	       AccionMenuSabotear.progressDialog.setIcon(R.drawable.ic_dialog_conexion);
	
				   	       Thread thread = new Thread(sabotearCalle);
				   	       thread.start();
			           }
			       }).setNegativeButton(context.getString(R.string.butNo), new DialogInterface.OnClickListener() {
	    	           public void onClick(DialogInterface dialog, int id) {
	    	        	   dialog.cancel();
	    	           }
	    	       });
	    	AlertDialog alert = builder.create();
	    	alert.setTitle(context.getString(R.string.sabotear));
	    	alert.setIcon(R.drawable.ic_dialog_informacion);
	    	alert.show();
		}else{
			Toast toast = Toast.makeText(context,
					context.getString(R.string.calleSabotearSinCarta),
					Toast.LENGTH_LONG);
			toast.show();
		}
	}
	
	private static Runnable sabotearCalle = new Runnable(){
		@Override
		public void run(){
			Looper.prepare();
			conexionSabotearCalle();
			Looper.myLooper().quit();
			Looper.loop();
		}
		
		private void showToastConexion(){			
			Toast toast = Toast.makeText(context,
					context.getString(R.string.errConexion),
					Toast.LENGTH_SHORT);
			toast.show();
		}
		
		private void showToastDatos(){
			Toast toast = Toast.makeText(context,
					context.getString(R.string.errDatos),
					Toast.LENGTH_SHORT);
			toast.show();
		}
		
		private void showToastAceptada(){
			Toast toast = Toast.makeText(context,
					String.format(context.getString(R.string.calleSabotearAceptada), calle.getNombre()),
					Toast.LENGTH_LONG);
			toast.show();
		}
		
		private void showToastRechazada(){
			Toast toast = Toast.makeText(context,
					String.format(context.getString(R.string.calleSabotearRechazada), calle.getNombre()),
					Toast.LENGTH_LONG);
			toast.show();
		}
		
		public void conexionSabotearCalle(){
			int resultado;
			try{
				DefaultHttpClient httpclient = new DefaultHttpClient();
				int idJuego = JuegoListar.getJuego().getIdentificador();
				int idEquipo = JuegoDatosListar.getEquipo().getIdentificador();
				int idCalle = AccionMenuSabotear.calle.getId();
				URI uri = new URI(GoldenStreets.urlServer+"juego/"+idJuego+"/equipo/"+idEquipo+"/calle/"+idCalle+"/sabotear/?format=json");

				HttpPost httpost = new HttpPost(uri);
				LibreGeoSocial LGS = LibreGeoSocial.getInstance();
				httpclient.setCookieStore(LGS.getCookieStore());
					            
	            HttpResponse response = httpclient.execute(httpost);
	            HttpEntity entity = response.getEntity();
	            String str = Utils.deStreamAString(entity.getContent());
				resultado = procesarRespuestaCalleSabotear(str);			

				handler.sendEmptyMessage(resultado);

			}catch(URISyntaxException urisex){
				Log.e(TAG, urisex.getMessage());
				handler.sendEmptyMessage(-1);
		    }catch (IOException e) {
		    	Log.e(TAG,e.getMessage());
		    	handler.sendEmptyMessage(-1);
		    }
		}
		
		private int procesarRespuestaCalleSabotear(String str) {
			int ultCode;
	    	String ultMensaje;
	        try {
	        	JSONObject json;
				json = new JSONObject(str);
				
				if (json.has("code"))
					ultCode = json.getInt("code");
				else
					ultCode = -1;
				
				if (json.has("descripcion"))
					ultMensaje = json.getString("descripcion");
				else
					ultMensaje = null;
				
				if (ultCode != 200){
					if (ultCode == 400 && ultMensaje.toLowerCase().compareTo("calle con modificaciones") == 0)
						return -3;
					return -2;
				}
			} catch (JSONException e) {	
				ultCode = 500;
				ultMensaje = "Error in json parsed: " + e.getMessage();
				Log.e(TAG, ultMensaje);
				return -2;
			}
			return 0;
		}
		
	    private Handler handler = new Handler()
	    {
	    	@Override
	        public void handleMessage(Message msg)
	        {
	        	int result = msg.what;
	        	AccionMenuSabotear.progressDialog.cancel();
	        	if(result == -3){
	        		showToastRechazada();
	        	}else if(result == -2){
	        		showToastDatos();
	        	}else if(result == -1){
	        		showToastConexion();
	        	}else if(result == 0){
	        		carta.setUsada(true);
	        		showToastAceptada();
	        	}
	        }
	    };
	};
}