package com.Goldenstreet.App.GoldenStreet;

import java.util.ArrayList;
import java.util.List;

import android.app.AlertDialog;
import android.app.ListActivity;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.CheckBox;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;

import com.Goldenstreet.App.R;

/**
 * Muestra la lista de cartas que va recibiendo peridicamente el equipo a lo largo de la partida.
 * Si el equipo selecciona una carta se pasar a una nueva activity en la que se mostrar de forma
 * detallada informacin sobre la carta seleccionada (tipo, nombre, descripcin, etc.).
 * 
 * @author Rubn Quero Castrillo <rubenquero@gmail.com>
 * 
 */

public class CartaListar extends ListActivity { 
	//static private String TAG = "CartaListar";
	
	private static final int MENU_INFORMAR = Menu.FIRST;
	
	private static List<Carta> cartas; 
	private static Carta carta = null;
	EventsAdapter mAdapter;

	public static Carta getCarta() {
		return carta;
	}
	public static void setCarta(Carta carta) {
		CartaListar.carta = carta;
	}

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
	}
	
	@Override
	protected void onResume(){
		super.onResume();
		registerReceiver(receiver, new IntentFilter("CartaServicio.NUEVA"));
    	registerReceiver(receiver, new IntentFilter("CartaServicio.SIN"));
		cartas = new ArrayList<Carta>(CartaServicio.getCartasEquipo());
		cartaListar();
	}
    
    @Override
    protected void onPause(){
        super.onPause();       
        unregisterReceiver(receiver);
    }
	
    @Override
	protected void onListItemClick(ListView l, View v, int position, long id) {
		carta = CartaListar.cartas.get(position);
		Intent i = new Intent(CartaListar.this, CartaInfo.class);
        this.startActivity(i);
	}
    
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        super.onCreateOptionsMenu(menu);
        menu.add(0, MENU_INFORMAR, 0 , getString(R.string.informacion))
        .setIcon(R.drawable.ic_menu_informacion);
        return true;
    }
    
    @Override
    public boolean onOptionsItemSelected (MenuItem item) {
    	switch (item.getItemId()) {
    		case MENU_INFORMAR:
    			showAlertInformacion();
    			break;
    	}
    	return super.onOptionsItemSelected(item);
    }   
	
	public boolean onKeyDown(int keyCode, KeyEvent event){
		if (keyCode == KeyEvent.KEYCODE_BACK) {
    		AlertDialog.Builder builder = new AlertDialog.Builder(this);
    		builder.setMessage(getString(R.string.salirJuego))
		           .setCancelable(false)
		           .setPositiveButton(getString(R.string.butSi), new DialogInterface.OnClickListener() {
		               public void onClick(DialogInterface dialog, int id) {
		            	   Utils.cerrarPartida(CartaListar.this);
		               }
		           })
		           .setNegativeButton(getString(R.string.butNo), new DialogInterface.OnClickListener() {
		               public void onClick(DialogInterface dialog, int id) {
		                    dialog.cancel();
		               }
		           });
		    AlertDialog alert = builder.create();
		    alert.setTitle(getString(R.string.aviso));
		    alert.setIcon(R.drawable.ic_dialog_advertencia);
		    alert.show();
	    	return true;
	    }else{
	    	return false;
	    }
	}
	
	private void showAlertListaVacia(){
		AlertDialog.Builder builder = new AlertDialog.Builder(CartaListar.this);
		builder.setMessage(R.string.noCartas)
	           .setCancelable(false)
	           .setPositiveButton(R.string.butContinuar, new DialogInterface.OnClickListener() {
	               public void onClick(DialogInterface dialog, int id) {
	            	   dialog.cancel();
	               }
	           });
	    AlertDialog alert = builder.create();
	    alert.setTitle(R.string.titLisCartas);
	    alert.setIcon(R.drawable.ic_dialog_informacion);
		alert.show();
	}
	
    private void showAlertInformacion(){
    	AlertDialog.Builder builder = new AlertDialog.Builder(this);
    	builder.setMessage(getString(R.string.infoCartaListar))
    	       .setCancelable(false)
    	       .setPositiveButton(getString(R.string.butContinuar), new DialogInterface.OnClickListener() {
    	           public void onClick(DialogInterface dialog, int id) {
    	        	   dialog.cancel();
    	           }
    	       });
    	AlertDialog alert = builder.create();
    	alert.setTitle(getString(R.string.informacion));
    	alert.setIcon(R.drawable.ic_dialog_informacion);
    	alert.show();
    }
	
	private void cartaListar(){
		if (CartaListar.cartas.size() > 0){
    		mAdapter = new EventsAdapter(CartaListar.this);
    		setListAdapter(mAdapter);
    	}else{
    		showAlertListaVacia();    		
    	}
	}
	
	public static class EventsAdapter extends BaseAdapter {

		private Context mContext;
		
		public EventsAdapter (Context c) {
            mContext = c;
        }

		public int getCount() {
			// TODO Auto-generated method stub
			return CartaListar.cartas.size();
		}

		public Object getItem(int position) {
			// TODO Auto-generated method stub
			return position;
		}

		public long getItemId(int position) {
			// TODO Auto-generated method stub
			return position;
		}

		public View getView(int position, View convertView, ViewGroup parent) {
			// TODO Auto-generated method stub
			View view;
			
	        if (convertView == null) {
	            LayoutInflater inflater = (LayoutInflater) mContext
	                    .getSystemService(Context.LAYOUT_INFLATER_SERVICE);
	            view = inflater.inflate(R.layout.cartalistar, null);
	        } else {
	            view = convertView;
	        }	        
	        
	        Carta carta = CartaListar.cartas.get(position);
	        
	        ImageView tipo = (ImageView) view.findViewById(R.id.imageTipo);
			switch(carta.getTipoAccion()){
			case Carta.INGRESAR:
				tipo.setImageResource(R.drawable.ic_list_ingresar);
				break;
			case Carta.RETIRAR:
				tipo.setImageResource(R.drawable.ic_list_retirar);
				break;
			case Carta.SABOTEAR:
				tipo.setImageResource(R.drawable.ic_list_sabotear);
				break;
			case Carta.CONSTRUIR:
				tipo.setImageResource(R.drawable.ic_list_construir);
				break;
			case Carta.DEMOLER:
				tipo.setImageResource(R.drawable.ic_list_demoler);
				break;
			}
	        
	        TextView textNombre = (TextView) view.findViewById(R.id.textNombre);
	        textNombre.setText((String)carta.getNombre());
			
			TextView textDescripcion = (TextView) view.findViewById(R.id.textDescripcion);
			textDescripcion.setText((String)carta.getDescripcion());
			
			CheckBox checkUsada = (CheckBox) view.findViewById(R.id.checkUsada);
			if (carta.getUsada())
				checkUsada.setChecked(true);
			else
				checkUsada.setChecked(false);
			checkUsada.setEnabled(false);
			
			return view;       
		}
	}
	
    private BroadcastReceiver receiver = new BroadcastReceiver() {        
        @Override
        public void onReceive(Context context, Intent intent) {
        	String accion = intent.getAction();
        	if (accion.equals("CartaServicio.NUEVA")) {
        		AlertDialog alert = AccionesRecibidasBroadcast.accionCartaServicioNueva(context, intent);
		    	alert.show();
        	} else if(accion.equals("CartaServicio.SIN")) {
        		AlertDialog alert = AccionesRecibidasBroadcast.accionCartaServicioSin(context, intent);
		    	alert.show();
        	}
        }
    };	
}