package com.Goldenstreet.App.GoldenStreet;

import java.io.IOException;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.protocol.HTTP;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import com.LibreGeoSocial.Api.LibreGeoSocial;

import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.util.Log;

/**
 * Servicio que peridicamente proporcionar de una nueva carta al equipo, segn el tipo de la carta
 * proporcionada esta puede ser aplicada por el servicio instantneamente. 
 * 
 * @author Rubn Quero Castrillo <rubenquero@gmail.com>
 * 
 */

public class CartaServicio extends Service{
	static private String TAG = "CartaServicio";
	
	private static final int RETARDO = 5*1000; 
	private static final int PERIODO = 90*1000;
	
	private static List<Carta> cartasJuego = new ArrayList<Carta>();
	private static List<Carta> cartasEquipo = new ArrayList<Carta>();
	private boolean reintento = true;
	private boolean infoMostrado = false;
	private Timer timer;
	
	public static List<Carta> getCartasEquipo(){
		return CartaServicio.cartasEquipo;
	}
	
	@Override
	public void onCreate() {
		super.onCreate();
		timer = new Timer("cartaServicio");
	}
	
	@Override
	public void onDestroy() {
		super.onDestroy();
		CartaServicio.cartasJuego = new ArrayList<Carta>();
		CartaServicio.cartasEquipo = new ArrayList<Carta>();
		timer.cancel();
	}
	
 	@Override
 	public void onStart(final Intent intent, final int startId){
 		super.onStart(intent, startId);
 		timer.scheduleAtFixedRate(timerTask, RETARDO, PERIODO);
 	}
 	
	@Override
	public IBinder onBind(Intent arg0) {
		// TODO Auto-generated method stub
		return null;
	}

	private TimerTask timerTask = new TimerTask() {
		int resultado = -1;
		@Override
		public void run() {
			Context context = getApplicationContext();
			boolean conectado = Utils.estaConectado(context);
			if(reintento){
				if(conectado){
					resultado = ObtenerCartas();
					if((resultado == 0) || (resultado == 1))
						reintento = false;
				}
			}
			handler.sendEmptyMessage(resultado);	
		}
	};
	
	private Carta darCartaEquipo(){
		int posicion = (int)(Math.random()*CartaServicio.cartasJuego.size());
		Carta c = new Carta(CartaServicio.cartasJuego.get(posicion));
		if (c.getTipoAccion() == Carta.INGRESAR || c.getTipoAccion() == Carta.RETIRAR){
			aplicarCarta(c);
			c.setUsada(true);
		}
		CartaServicio.cartasEquipo.add(c);
		return c;
	}
	
	private int ObtenerCartas(){
		int resultado = 0; 
		try{
			DefaultHttpClient httpclient = new DefaultHttpClient();
			URI uri = new URI(GoldenStreets.urlServer+"carta/listar/?format=json");
			HttpGet httpget = new HttpGet(uri);
			LibreGeoSocial LGS = LibreGeoSocial.getInstance();
			httpclient.setCookieStore(LGS.getCookieStore());
			HttpResponse response = httpclient.execute(httpget);
			HttpEntity entity = response.getEntity();
			String str = Utils.deStreamAString(entity.getContent());
			resultado = procesarRespuestaCartasListar(str);
			
		}catch(URISyntaxException urisex){
			resultado = -1;
			Log.e(TAG, urisex.getMessage());
	    }catch (IOException e) {
	    	resultado = -1;
	    	Log.e(TAG,e.getMessage());
	    }
		return resultado;
	}
	
	private int procesarRespuestaCartasListar (String str) {
    	String ultMensaje = null;
    	int ultCode = 0;
        try {
        	JSONObject json;
        	json = new JSONObject(str.toString());

			if (json.has("code"))
				ultCode = json.getInt("code");
			else
				ultCode = -1;
			
			if (json.has("descripcion"))
				ultMensaje = json.getString("descripcion");
			else
				ultMensaje = null;
			
			if (json.has("cartas") && ultMensaje.toLowerCase().equals("ok")){
				JSONArray cartasObjects = json.getJSONArray("cartas");
				CartaServicio.cartasJuego = new ArrayList<Carta>();
				int cartaIdentificador;
				String cartaNombre;
				int cartaTipoAccion;
				int cartaCantidad;
				String cartaDescripcion;
				boolean cartaUsada;
				
				for (int i = 0; i < cartasObjects.length()-1; i++){
					JSONObject JSONCarta = (JSONObject)cartasObjects.get(i);
					cartaIdentificador = JSONCarta.getInt("identificador");
					cartaNombre = JSONCarta.getString("nombre");
					cartaTipoAccion = JSONCarta.getInt("tipoAccion");
					cartaCantidad = JSONCarta.getInt("cantidad");
					cartaDescripcion = JSONCarta.getString("informacion");
					cartaUsada = JSONCarta.getBoolean("usada");
					CartaServicio.cartasJuego.add(new Carta(cartaIdentificador, cartaNombre, cartaTipoAccion,
							cartaCantidad, cartaDescripcion, cartaUsada));
				}
			}		
			if (ultCode != 200)
				return -2;
			if (CartaServicio.cartasJuego.size() == 0)
				return 1;

		} catch (Exception e) {	
			ultCode = 500;
			ultMensaje = "Error in json parsed: " + e.getMessage();
			Log.e(TAG, ultMensaje);
			return -2;
		}
		return 0;
	}
	
	private void aplicarCarta(Carta c){
		//int resultado;
		try{
			DefaultHttpClient httpclient = new DefaultHttpClient();
			int idJuego = JuegoListar.getJuego().getIdentificador();
			int idEquipo = JuegoDatosListar.getEquipo().getIdentificador();
			URI uri = new URI(GoldenStreets.urlServer+"juego/"+idJuego+"/equipo/"+idEquipo+"/carta/usar/?format=json");
			HttpPost httpost = new HttpPost(uri);
			LibreGeoSocial LGS = LibreGeoSocial.getInstance();
			httpclient.setCookieStore(LGS.getCookieStore());
			List <NameValuePair> datos = new ArrayList <NameValuePair>();
            datos.add(new BasicNameValuePair("cantidad",String.valueOf(c.getCantidad())));
            datos.add(new BasicNameValuePair("tipoAccion",String.valueOf(c.getTipoAccion())));
            httpost.setEntity(new UrlEncodedFormEntity(datos, HTTP.UTF_8));
			
			HttpResponse response = httpclient.execute(httpost);
			HttpEntity entity = response.getEntity();
			String str = Utils.deStreamAString(entity.getContent());
			procesarRespuestaPatrimonioModificar(str);
			
		}catch(URISyntaxException urisex){
			//resultado = -1;
			Log.e(TAG, urisex.getMessage());
	    }catch (IOException e) {
	    	//resultado = -1;
	    	Log.e(TAG,e.getMessage());
	    }
	}
	
	private int procesarRespuestaPatrimonioModificar(String str) {
		int ultCode;
    	String ultMensaje;
        try {
        	JSONObject json;
			json = new JSONObject(str);
			
			if (json.has("code"))
				ultCode = json.getInt("code");
			else
				ultCode = -1;
			
			if (json.has("descripcion"))
				ultMensaje = json.getString("descripcion");
			else
				ultMensaje = null;
			 
			if (ultCode != 200){
				return -2;
			}
			
		} catch (JSONException e) {	
			ultCode = 500;
			ultMensaje = "Error in json parsed: " + e.getMessage();
			Log.e(TAG, ultMensaje);
			return -2;
		}
		return 0;
	}
	
    private Handler handler = new Handler()
    {
        public void handleMessage(Message msg)
        {
        	int result = msg.what;	
			if (result == 0){        		
        		Carta c = darCartaEquipo();
        		Intent i = new Intent("CartaServicio.NUEVA");
        		i.putExtra("carta", c);
        		getApplicationContext().sendOrderedBroadcast(i, null);
        	}else if (result == 1){        		
        		if(!infoMostrado){
        			infoMostrado = true;
        			Intent i = new Intent("CartaServicio.SIN");            
            		getApplicationContext().sendOrderedBroadcast(i, null);
        		}
        	}
        }
    };
}