package com.Goldenstreet.App.GoldenStreet;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Point;
import android.graphics.Paint.Style;
import android.graphics.drawable.Drawable;
import android.widget.Toast;

import com.google.android.maps.GeoPoint;
import com.google.android.maps.ItemizedOverlay;
import com.google.android.maps.MapView;
import com.google.android.maps.OverlayItem;
import com.google.android.maps.Projection;

/**
 * Clase auxiliar que aade al equipo elegido por el usuario una capa de informacin  que se
 * visualizar cuando el equipo pulse sobre l en el mapa, as como un rea de accin para realizar
 * diferentes acciones sobre las calles.
 * 
 * @author Rubn Quero Castrillo <rubenquero@gmail.com>
 * 
 */

public class EquipoItemizedOverlay extends ItemizedOverlay<OverlayItem>{
	
	private OverlayItem mOverlay;
	private int meters;
	private Context context;	
	
	public EquipoItemizedOverlay(Context context, Drawable defaultMarker) {
		super(boundCenterBottom(defaultMarker));
		this.context = context;
	}

	@Override
	protected OverlayItem createItem(int i) {
		return mOverlay;
	}

	@Override
	public int size() {
		return 1;
	}
	
	public void addOverlay(OverlayItem overlay){
		mOverlay = overlay;
		populate();
	}

	@Override
    public void draw(Canvas canvas, MapView mapView, boolean shadow) {
    	Paint paint = new Paint();
    	GeoPoint miPosicion = mOverlay.getPoint();
    	Point screenCurrentPoint = new Point();
		Projection projection = mapView.getProjection();
		projection.toPixels(miPosicion, screenCurrentPoint);
		
		int radius = metersToRadius(mapView, miPosicion.getLatitudeE6() / 1000000);
    	paint.setAntiAlias(true);
    	paint.setStyle(Style.FILL_AND_STROKE);
    	paint.setColor(Color.GREEN);
    	paint.setAlpha(40);    	
    	  	
    	canvas.drawCircle(screenCurrentPoint.x, screenCurrentPoint.y, radius, paint);
		super.draw(canvas, mapView, shadow);
	}
	
	@Override
	protected boolean onTap(int index) {
		Toast.makeText(context, mOverlay.getSnippet(), Toast.LENGTH_LONG).show();
		return super.onTap(index);
	}
	
    public void setMeters(int meters) {
    	this.meters = meters;
    }
    
	public int metersToRadius(MapView mapView, double latitude) {
		return (int) (mapView.getProjection().metersToEquatorPixels(meters) * (1/ Math.cos(Math.toRadians(latitude))));
	}
}