package com.Goldenstreet.App.GoldenStreet;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.util.Log;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.TextView;
import android.widget.Toast;

import com.Goldenstreet.App.R;
import com.LibreGeoSocial.Api.LibreGeoSocial;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.impl.client.DefaultHttpClient;
import org.json.JSONArray;
import org.json.JSONObject;

/**
 * Muestra al equipo el tiempo que lleva jugando y el lmite de la partida, tambin se visualiza
 * informacin sobre su patrimonio, y una clasificacin de todos los equipos participantes.
 * 
 * @author Rubn Quero Castrillo <rubenquero@gmail.com>
 * 
 */

public class Estadisticas extends Activity{
	static private String TAG = "Estadisticas";
	
	private static final int MENU_INFORMAR = Menu.FIRST;
	
	private static TextView textTiempo; 
	private TextView textEstadisticas;
	private TextView textClasificacion;
    
    public static TextView getTextTiempo() {
		return textTiempo;
	}

	@Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.estadisticas);
        textTiempo = (TextView) findViewById(R.id.textTiempo);
        textEstadisticas = (TextView)this.findViewById(R.id.textEstadisticas);
        textClasificacion = (TextView)this.findViewById(R.id.textClasificacion);
    }
    
    @Override
    protected void onResume(){
    	super.onResume();
    	registerReceiver(receiver, new IntentFilter("CartaServicio.NUEVA"));
    	registerReceiver(receiver, new IntentFilter("CartaServicio.SIN"));
    	estadisticas();
    }
    
    @Override
    protected void onPause(){
        super.onPause();       
        unregisterReceiver(receiver);
    }
    
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        super.onCreateOptionsMenu(menu);
        menu.add(0, MENU_INFORMAR, 0 , getString(R.string.informacion))
        .setIcon(R.drawable.ic_menu_informacion);
        return true;
    }
    
    @Override
    public boolean onOptionsItemSelected (MenuItem item) {
    	switch (item.getItemId()) {
    		case MENU_INFORMAR:
    			showAlertInformacion();
    			break;
    	}
    	return super.onOptionsItemSelected(item);
    }   
    
    @Override
	public boolean onKeyDown(int keyCode, KeyEvent event){
		if (keyCode == KeyEvent.KEYCODE_BACK) {
    		AlertDialog.Builder builder = new AlertDialog.Builder(this);
    		builder.setMessage(getString(R.string.salirJuego))
		           .setCancelable(false)
		           .setPositiveButton(getString(R.string.butSi), new DialogInterface.OnClickListener() {
		               public void onClick(DialogInterface dialog, int id) {
		            	   Utils.cerrarPartida(Estadisticas.this);	   
		               }
		           })
		           .setNegativeButton(getString(R.string.butNo), new DialogInterface.OnClickListener() {
		               public void onClick(DialogInterface dialog, int id) {
		                    dialog.cancel();
		               }
		           });
		    AlertDialog alert = builder.create();
		    alert.setTitle(getString(R.string.aviso));
		    alert.setIcon(R.drawable.ic_dialog_advertencia);
		    alert.show();
	    	return true;
	    }else{
	    	return false;
	    }
	}
    
	private void showToastConexion(){
		Toast toast = Toast.makeText(getBaseContext(),
				getString(R.string.errConexion),
				Toast.LENGTH_SHORT);
		toast.show();
	}
	
	private void showToastDatos(){
		Toast toast = Toast.makeText(getBaseContext(),
				getString(R.string.errDatos),
				Toast.LENGTH_SHORT);
		toast.show();
	}
	
    private void showAlertInformacion(){
    	AlertDialog.Builder builder = new AlertDialog.Builder(this);
    	builder.setMessage(getString(R.string.infoEstadisticas))
    	       .setCancelable(false)
    	       .setPositiveButton(getString(R.string.butContinuar), new DialogInterface.OnClickListener() {
    	           public void onClick(DialogInterface dialog, int id) {
    	        	   dialog.cancel();
    	           }
    	       });
    	AlertDialog alert = builder.create();
    	alert.setTitle(getString(R.string.informacion));
    	alert.setIcon(R.drawable.ic_dialog_informacion);
    	alert.show();
    }
    
    private void showAlertFinJuego(){    	
    	AlertDialog.Builder builder = new AlertDialog.Builder(this);
    	builder.setMessage(getString(R.string.infoFinJuego))
    	       .setCancelable(false)
    	       .setPositiveButton(getString(R.string.butContinuar), new DialogInterface.OnClickListener() {
    	           public void onClick(DialogInterface dialog, int id) {
    	        	   dialog.cancel();
    	           }
    	       });
    	AlertDialog alert = builder.create();
    	alert.setTitle(getString(R.string.finJuego));
    	alert.setIcon(R.drawable.ic_dialog_informacion);
    	alert.show();
    }
    
	private void estadisticas(){
		int resultado;		
		try{
			DefaultHttpClient httpclient = new DefaultHttpClient();
			int idJuego = JuegoListar.getJuego().getIdentificador();
			URI uri = new URI(GoldenStreets.urlServer+"juego/"+idJuego+"/patrimonio/listar/?format=json");
			HttpGet httpget = new HttpGet(uri);
			LibreGeoSocial LGS = LibreGeoSocial.getInstance();
			httpclient.setCookieStore(LGS.getCookieStore());
			HttpResponse response = httpclient.execute(httpget);
			HttpEntity entity = response.getEntity();
			String str = Utils.deStreamAString(entity.getContent());
			resultado = procesarRespuestaPatrimonio(str);
			if (resultado == 0){
				if (Partida.getFinPartida()){
					int duracionJuego = JuegoListar.getJuego().getDuracion();					
					textTiempo.setText(String.format(getString(R.string.tiempoJuego), duracionJuego, duracionJuego));
					showAlertFinJuego();
				}				
			}else if(resultado == -2){				
				showToastDatos();				
			}
		}catch(URISyntaxException urisex){
			Log.e(TAG, urisex.getMessage());
			showToastConexion();
	    }catch (IOException e) {
	    	Log.e(TAG,e.getMessage());
	    	showToastConexion();
	    }
	}
	
	private int procesarRespuestaPatrimonio (String str) {
		String ultMensaje = null;
		Integer ultCode = 0;
        try {
        	JSONObject json;
        	json = new JSONObject(str.toString());

			if (json.has("code"))
				ultCode = json.getInt("code");
			else
				ultCode = -1;
			
			if (json.has("descripcion"))
				ultMensaje = json.getString("descripcion");
			else
				ultMensaje = null;
			
			Equipo equipoSelec = JuegoDatosListar.getEquipo();
			if (json.has("patrimonios") && ultMensaje.toLowerCase().equals("ok")){
				JSONArray patrimoniosObjects = json.getJSONArray("patrimonios");
				int numeroPatrimonios = patrimoniosObjects.length()-1;
				String equipoNombre;
				int equipoDinero;
				int equipoValPatrimonio;
				int equipoNumPropiedades;
				String clasificacion = new String();
				for (int i = 0; i < numeroPatrimonios; i++){
					JSONObject JSONEquipo = (JSONObject)patrimoniosObjects.get(i);
					equipoNombre = JSONEquipo.getString("nombre");
					equipoDinero = JSONEquipo.getInt("dinero");
					equipoValPatrimonio = JSONEquipo.getInt("valPatrimonio");
					equipoNumPropiedades = JSONEquipo.getInt("numPropiedades");
					
					String patrimonio = String.format(getString(R.string.estadisticasEquipo),(i+1), equipoNombre,
							equipoDinero, equipoValPatrimonio, equipoNumPropiedades);
					
					clasificacion = clasificacion.concat(patrimonio);
					if (equipoSelec.getGroupName().compareTo(equipoNombre) == 0){
						JuegoDatosListar.getEquipo().getPatrimonio().setDinero(equipoDinero);
						JuegoDatosListar.getEquipo().getPatrimonio().setValPatrimonio(equipoValPatrimonio);
						JuegoDatosListar.getEquipo().getPatrimonio().setValPatrimonio(equipoNumPropiedades);
						textEstadisticas.setText(patrimonio);
					}
				}
				textClasificacion.setText(clasificacion);
			}
			if (ultCode != 200)
				return -2; 

		} catch (Exception e) {	
			ultCode = 500;
			ultMensaje = "Error in json parsed: " + e.getMessage();
			Log.e(TAG, ultMensaje);
			return -2;
		}
		return 0;
	}
	
    private BroadcastReceiver receiver = new BroadcastReceiver() {        
        @Override
        public void onReceive(Context context, Intent intent) {
        	String accion = intent.getAction();
        	if (accion.equals("CartaServicio.NUEVA")) {
        		AlertDialog alert = AccionesRecibidasBroadcast.accionCartaServicioNueva(context, intent);
		    	alert.show();
        	} else if(accion.equals("CartaServicio.SIN")) {
        		AlertDialog alert = AccionesRecibidasBroadcast.accionCartaServicioSin(context, intent);
		    	alert.show();
        	}
        }
    };
}