package com.Goldenstreet.App.GoldenStreet;

import android.app.AlertDialog;
import android.app.ListActivity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import com.Goldenstreet.App.R;
import com.LibreGeoSocial.Api.LibreGeoSocial;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.impl.client.DefaultHttpClient;
import org.json.JSONArray;
import org.json.JSONObject;

/**
 * Muestra el listado de los equipos disponibles con los que el usuario puede participar y
 * se almacena el equipo seleccionado por el usuario, para obtener en las siguientes
 * pantallas y durante la partida datos referidos a su equipo. Tambin se obtienen datos
 * esenciales del juego (calles, edificios) para verificar que es vlida.
 * 
 * @author Rubn Quero Castrillo <rubenquero@gmail.com>
 * 
 */

public class JuegoDatosListar extends ListActivity implements Runnable{ 
	static private String TAG = "JuegoDatosListar";
	
	private static Equipo[] equipos;
	private static Calle[] calles;
	private static Equipo equipo = null;

	private static ArrayList<String> listNombre;
	EventsAdapter mAdapter;
	private ProgressDialog progressDialog = null;
	
	public static Equipo[] getEquipos() {
		return equipos;
	}	
	
	public static Calle[] getCalles() {
		return calles;
	}
	
	public static Equipo getEquipo() {
		return equipo;
	}	
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		equipo = null;
		datosListar();
	}
	
	@Override
	protected void onListItemClick(ListView l, View v, int position, long id) {
		equipo = equipos[position];
		
		Intent i = new Intent(JuegoDatosListar.this, PartidaInfo.class);
        this.startActivity(i);
        JuegoDatosListar.this.setResult(RESULT_OK);
        JuegoDatosListar.this.finish();
	}
	
	private void showToastConexion(){
		Toast toast = Toast.makeText(getBaseContext(),
				getString(R.string.errConexion),
				Toast.LENGTH_SHORT);
		toast.show();
		JuegoDatosListar.this.setResult(RESULT_OK);
		JuegoDatosListar.this.finish();
	}
	
	private void showToastDatos(){
		Toast toast = Toast.makeText(getBaseContext(),
				getString(R.string.errDatos),
				Toast.LENGTH_SHORT);
		toast.show();
		JuegoDatosListar.this.setResult(RESULT_OK);
		JuegoDatosListar.this.finish();
	}

	private void showAlertListaVacia(){
		AlertDialog.Builder builder = new AlertDialog.Builder(JuegoDatosListar.this);
		builder.setMessage(R.string.noDatos)
	           .setCancelable(false)
	           .setPositiveButton(R.string.butContinuar, new DialogInterface.OnClickListener() {
	               public void onClick(DialogInterface dialog, int id) {
	            	   JuegoDatosListar.this.setResult(RESULT_OK);
	            	   JuegoDatosListar.this.finish();
	               }
	           });
	    AlertDialog alert = builder.create();
	    alert.setTitle(R.string.titLisPartidaDatos);
	    alert.setIcon(R.drawable.ic_dialog_informacion);
		alert.show();
	}

	private void datosListar(){
		String title = getString(R.string.espere);
        String text = getString(R.string.obtenerDatos);
		
		progressDialog = ProgressDialog.show(this, title, text, true, false);
		progressDialog.setIcon(R.drawable.ic_dialog_conexion);
        Thread thread = new Thread(this);
        thread.start();
	}
	
	public void run() {
		Looper.prepare();
		conexionDatosListar();
		Looper.myLooper().quit();
		Looper.loop();
	}
	
	public void conexionDatosListar() {
		int resultado;
		try{
			DefaultHttpClient httpclient = new DefaultHttpClient();
			int idJuego = JuegoListar.getJuego().getIdentificador();
			URI uri = new URI(GoldenStreets.urlServer+"juego/"+idJuego+"/datos/listar/?format=json");
			HttpGet httpget = new HttpGet(uri);
			LibreGeoSocial LGS = LibreGeoSocial.getInstance();
			httpclient.setCookieStore(LGS.getCookieStore());
			HttpResponse response = httpclient.execute(httpget);
			HttpEntity entity = response.getEntity();
			String str = Utils.deStreamAString(entity.getContent());
			resultado = procesarRespuestaDatosListar(str);
			handler.sendEmptyMessage(resultado);
			
		}catch(URISyntaxException urisex){
			Log.e(TAG, urisex.getMessage());
			handler.sendEmptyMessage(-1);
	    }catch (IOException e) {
	    	Log.e(TAG,e.getMessage());
	    	handler.sendEmptyMessage(-1);
	    }
	}
	
	private int procesarRespuestaDatosListar (String str) {
		String ultMensaje = null;
		int ultCode = 0;
	    try {
	    	JSONObject json;
	    	json = new JSONObject(str.toString());
	
			if (json.has("code"))
				ultCode = json.getInt("code");
			else
				ultCode = -1;
			
			if (json.has("descripcion"))
				ultMensaje = json.getString("descripcion");
			else
				ultMensaje = null;
			
			listNombre = new ArrayList<String>();
			int numeroEquipos = 0;
			
			if (json.has("equipos") && json.has("calles") && ultMensaje.toLowerCase().equals("ok")){
				JSONArray equiposObjects = json.getJSONArray("equipos");
				numeroEquipos = equiposObjects.length()-1;
				equipos = new Equipo[numeroEquipos];
				int equipoIdentificador;
				String equipoNombre;
				int equipoIdGrupo;
				Double equipoLatitud = 0.0;
				Double equipoLongitud = 0.0;
				Double equipoAltitud = 0.0;
				Double equipoRadio = 0.0;
				String equipoSince = null;
	
				for (int i = 0; i < numeroEquipos; i++){
					JSONObject JSONEquipo = (JSONObject)equiposObjects.get(i);
					equipoIdentificador = JSONEquipo.getInt("identificador");
					equipoNombre = JSONEquipo.getString("nombre");
					equipoIdGrupo = JSONEquipo.getInt("idGrupo");
					
					equipos[i] = new Equipo(equipoIdentificador, equipoNombre, equipoIdGrupo,
							equipoLatitud, equipoLongitud, equipoAltitud, equipoRadio, equipoSince);
					
					listNombre.add(equipos[i].getGroupName());
				}
				
				JSONArray callesObjects = json.getJSONArray("calles");
				int numeroCalles = callesObjects.length()-1;
				calles = new Calle[numeroCalles];
				int calleIdentificador;
				int equipoId;
				String calleNombre;
				int callePrecio;
				Double calleLatitud;
				Double calleLongitud;
				Double calleAltitud = 0.0;
				Double calleRadio = 0.0;
				String calleSince = null;				
				String edificioNombre = null;
				int edificioBonificacion = 0;
				int edificioPrecio = 0;
	
				for (int i = 0; i < numeroCalles; i++){
					JSONObject JSONCalle = (JSONObject)callesObjects.get(i);
					calleIdentificador = JSONCalle.getInt("identificador");
					equipoId = JSONCalle.getInt("idPropietario");
					if (JSONCalle.has("nombreEdificio")){
						edificioNombre = JSONCalle.getString("nombreEdificio");
						edificioBonificacion = JSONCalle.getInt("bonificacionEdificio");
						edificioPrecio = JSONCalle.getInt("precioEdificio");
					}
					calleNombre = JSONCalle.getString("nombre");
					callePrecio = JSONCalle.getInt("precio");
					calleLatitud = JSONCalle.getDouble("latitud");
					calleLongitud = JSONCalle.getDouble("longitud");	
					
					calles[i] = new Calle(calleIdentificador, equipoId, edificioNombre, 
							edificioPrecio, edificioBonificacion, calleNombre, callePrecio,
							calleLatitud, calleLongitud, calleAltitud, calleRadio, calleSince);
				}
			}		
			if (ultCode != 200)
				return -2;
			if (numeroEquipos == 0)
				return 1;
	
		} catch (Exception e) {	
			ultCode = 500;
			ultMensaje = "Error in json parsed: " + e.getMessage();
			Log.e(TAG, ultMensaje);
			return -2;
		}
		return 0;
	}

	private Handler handler = new Handler() {
		@Override
		public void handleMessage(Message msg) {
			int result = msg.what;			
			progressDialog.cancel();			
			if (result == -2){        		
				showToastDatos();
        	}else if (result == -1){
        		showToastConexion();
        	}else if (result == 0){        		
				mAdapter = new EventsAdapter(JuegoDatosListar.this);
				setListAdapter(mAdapter);
        	}else if (result == 1){
        		showAlertListaVacia();        		
        	}
		}
	};
	
    public static class EventsAdapter extends BaseAdapter {

		private Context mContext;
		
		public EventsAdapter (Context c) {
            mContext = c;
        }

		public int getCount() {
			// TODO Auto-generated method stub
			return JuegoDatosListar.listNombre.size();
		}

		public Object getItem(int position) {
			// TODO Auto-generated method stub
			return position;
		}

		public long getItemId(int position) {
			// TODO Auto-generated method stub
			return position;
		}

		public View getView(int position, View convertView, ViewGroup parent) {
			// TODO Auto-generated method stub
			View view;
			
	        if (convertView == null) {
	            LayoutInflater inflater = (LayoutInflater) mContext
	                    .getSystemService(Context.LAYOUT_INFLATER_SERVICE);
	            view = inflater.inflate(R.layout.equipolistar, null);
	        } else {
	            view = convertView;
	        }
			
			TextView textNombre = (TextView) view.findViewById(R.id.textNombre);
			textNombre.setText((String)listNombre.get(position));
			
			return view;       
		}
	}
}