package com.Goldenstreet.App.GoldenStreet;

import android.app.AlertDialog;
import android.app.ListActivity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import com.Goldenstreet.App.R;
import com.LibreGeoSocial.Api.LibreGeoSocial;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.impl.client.DefaultHttpClient;
import org.json.JSONArray;
import org.json.JSONObject;

/**
 * Muestra el listado de los juegos disponibles en la que el usuario puede participar y
 * se almacena el juego seleccionado por el usuario, para obtener en las siguientes
 * pantallas y durante la partida datos referidos al juego.
 * 
 * @author Rubn Quero Castrillo <rubenquero@gmail.com>
 * 
 */

public class JuegoListar extends ListActivity implements Runnable { 
	static private String TAG = "JuegoListar";
	
	private Juego[] juegos;
	private static Juego juego = null;	
	private static ArrayList<String> listNombre;
	private static ArrayList<String> listUbicacion;
	private static ArrayList<String> listFecha;
	private static ArrayList<String> listDinero;
	private static ArrayList<String> listDuracion;
	
	private EventsAdapter mAdapter;
	private ProgressDialog progressDialog = null;
	
	public static Juego getJuego() {
		return juego;
	}

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		juego = null;
		juegoListar();
	}
	
	@Override
	protected void onListItemClick(ListView l, View v, int position, long id) {
		juego = juegos[position];
		
		Intent i = new Intent(JuegoListar.this, JuegoDatosListar.class);
        this.startActivity(i);
        JuegoListar.this.setResult(RESULT_OK);
        JuegoListar.this.finish();
	}
	
	private void showToastConexion(){
		Toast toast = Toast.makeText(getBaseContext(),
				getString(R.string.errConexion),
				Toast.LENGTH_SHORT);
		toast.show();
		JuegoListar.this.setResult(RESULT_OK);
		JuegoListar.this.finish();
	}
	
	private void showToastDatos(){
		Toast toast = Toast.makeText(getBaseContext(),
				getString(R.string.errDatos),
				Toast.LENGTH_SHORT);
		toast.show();
		JuegoListar.this.setResult(RESULT_OK);
		JuegoListar.this.finish();
	}

	private void showAlertListaVacia(){
		AlertDialog.Builder builder = new AlertDialog.Builder(JuegoListar.this);
		builder.setMessage(R.string.noJuegos)
	           .setCancelable(false)
	           .setPositiveButton(R.string.butContinuar, new DialogInterface.OnClickListener() {
	               public void onClick(DialogInterface dialog, int id) {
	            	   JuegoListar.this.setResult(RESULT_OK);
	            	   JuegoListar.this.finish();
	               }
	           });
	    AlertDialog alert = builder.create();
	    alert.setTitle(R.string.titLisJuegos);
	    alert.setIcon(R.drawable.ic_dialog_informacion);
		alert.show();
	}
	
	private void juegoListar(){
		String title = getString(R.string.espere);
        String text = getString(R.string.obtenerJuego);
		
		progressDialog = ProgressDialog.show(this, title, text, true, false);
		progressDialog.setIcon(R.drawable.ic_dialog_conexion);
        Thread thread = new Thread(this);
        thread.start();
	}
	
	public void run() {
		Looper.prepare();
		conexionJuegoListar();
		Looper.myLooper().quit();
		Looper.loop();
	}
	
	public void conexionJuegoListar(){
		int resultado;
		try{
			DefaultHttpClient httpclient = new DefaultHttpClient();
			URI uri = new URI(GoldenStreets.urlServer+"juego/listar/?format=json");
			HttpGet httpget = new HttpGet(uri);
			LibreGeoSocial LGS = LibreGeoSocial.getInstance();
			httpclient.setCookieStore(LGS.getCookieStore());
			HttpResponse response = httpclient.execute(httpget);
			HttpEntity entity = response.getEntity();
			String str = Utils.deStreamAString(entity.getContent());
			resultado = procesarRespuestaJuegoListar(str);
			
			handler.sendEmptyMessage(resultado);				
			
		}catch(URISyntaxException urisex){
			Log.e(TAG, urisex.getMessage());
			handler.sendEmptyMessage(-1);
	    }catch (IOException e) {
	    	Log.e(TAG,e.getMessage());
	    	handler.sendEmptyMessage(-1);
	    }
	}
	
	private int procesarRespuestaJuegoListar(String str) {
		String ultMensaje = null;
		int ultCode = 0;
		
	    try {
	    	JSONObject json;
	    	json = new JSONObject(str.toString());
	
			if (json.has("code"))
				ultCode = json.getInt("code");
			else
				ultCode = -1;
			
			if (json.has("descripcion"))
				ultMensaje = json.getString("descripcion");
			else
				ultMensaje = null;
	
			listNombre = new ArrayList<String>();
			listUbicacion = new ArrayList<String>();
			listFecha = new ArrayList<String>();
			listDinero = new ArrayList<String>();
			listDuracion = new ArrayList<String>();
			int numeroJuegos = 0;
						
			if (json.has("juegos") && ultMensaje.toLowerCase().equals("ok")){
				JSONArray juegosObjects = json.getJSONArray("juegos");
				numeroJuegos = juegosObjects.length()-1;
				juegos = new Juego[numeroJuegos];
				int juegoIdentificador;
				String juegoNombre;
				String juegoUbicacion;
				String juegoFecha;
				int juegoDinero;
				int juegoDuracion;
	
				for (int i = 0; i < numeroJuegos; i++){
					JSONObject JSONJuego = (JSONObject)juegosObjects.get(i);
					juegoIdentificador = JSONJuego.getInt("identificador");
					juegoNombre = JSONJuego.getString("nombre");
					juegoUbicacion = JSONJuego.getString("ubicacion");
					juegoFecha = JSONJuego.getString("fecha");
					juegoDinero = Integer.valueOf(JSONJuego.getString("dinero"));
					juegoDuracion = Integer.valueOf(JSONJuego.getString("duracion"));
	
					juegos[i] = new Juego(juegoIdentificador, juegoNombre, 
							juegoUbicacion, juegoFecha, juegoDinero, juegoDuracion);
	
					listNombre.add(getString(R.string.nombreJuego).concat(juegos[i].getNombre()));
					listUbicacion.add(getString(R.string.ubicacionJuego).concat(juegos[i].getUbicacion()));
					listFecha.add(juegoFecha);
					listDinero.add(getString(R.string.dineroInicial).concat(String.valueOf(juegos[i].getDinero())));
					listDuracion.add(getString(R.string.duracionJuego).concat(String.valueOf(juegos[i].getDuracion())));
				}
			}
			if (ultCode != 200)
				return -2;
			if (numeroJuegos == 0)
				return 1;

	
		} catch (Exception e) {	
			ultCode = 500;
			ultMensaje = "Error in json parsed: " + e.getMessage();
			Log.e(TAG, ultMensaje);
			return -2;
		}
		return 0;
	}

	private Handler handler = new Handler() {
		@Override
		public void handleMessage(Message msg) {
			int result = msg.what;			
			progressDialog.cancel();			
			if (result == -2){        		
				showToastDatos();
        	}else if (result == -1){
        		showToastConexion();
        	}else if (result == 0){        		
				mAdapter = new EventsAdapter(JuegoListar.this);
				setListAdapter(mAdapter);
        	}else if (result == 1){
        		showAlertListaVacia();        		
        	}
		}
	};
	
    public static class EventsAdapter extends BaseAdapter {

		private Context mContext;
		
		public EventsAdapter (Context c) {
            mContext = c;
        }

		public int getCount() {
			// TODO Auto-generated method stub
			return JuegoListar.listNombre.size();
		}

		public Object getItem(int position) {
			// TODO Auto-generated method stub
			return position;
		}

		public long getItemId(int position) {
			// TODO Auto-generated method stub
			return position;
		}

		public View getView(int position, View convertView, ViewGroup parent) {
			// TODO Auto-generated method stub
			View view;
			
	        if (convertView == null) {
	            LayoutInflater inflater = (LayoutInflater) mContext
	                    .getSystemService(Context.LAYOUT_INFLATER_SERVICE);
	            view = inflater.inflate(R.layout.juegolistar, null);
	        } else {
	            view = convertView;
	        }
	        
			TextView textNombre = (TextView) view.findViewById(R.id.textNombre);
			textNombre.setText((String)listNombre.get(position));
			
			TextView textUbicacion = (TextView) view.findViewById(R.id.textUbicacion);
			textUbicacion.setText((String)listUbicacion.get(position));
			
			TextView textDineroInicial = (TextView) view.findViewById(R.id.textDineroInicial);
			textDineroInicial.setText((String)listDinero.get(position));
			
			TextView textDuracion = (TextView) view.findViewById(R.id.textDuracion);
			textDuracion.setText((String)listDuracion.get(position));
			
			return view;       
		}
	}
}