package com.Goldenstreet.App.GoldenStreet;

import android.app.AlertDialog;
import android.app.ListActivity;
import android.app.ProgressDialog;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import com.Goldenstreet.App.R;
import com.LibreGeoSocial.Api.LibreGeoSocial;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.impl.client.DefaultHttpClient;
import org.json.JSONArray;
import org.json.JSONObject;

/**
 * Muestra la lista de los mensajes enviados/recibidos, con una breve descripcin de ellos. Si el
 * equipo selecciona un mensaje se pasar a una nueva activity en la que se mostrar de forma
 * detallada la informacin sobre el mensaje seleccionado (emisor, destinatario, etc.).
 * 
 * @author Rubn Quero Castrillo <rubenquero@gmail.com>
 * 
 */

public class MensajeListar extends ListActivity implements Runnable { 
	static private String TAG = "MensajeListar";
	
	private static final int MENU_INFORMAR = Menu.FIRST;
	
	private static Mensaje[] mensajes;
	public static Mensaje mensaje = null;
	
	static private ArrayList<String> listDelEquipo;
	static private ArrayList<String> listParaEquipo;
	static private ArrayList<String> listTexto;
	EventsAdapter mAdapter;
	private ProgressDialog progressDialog = null;
	
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		mensaje = null;
		mensajeListar();
	}
	
	@Override
	protected void onResume() {
		super.onResume();
		registerReceiver(receiver, new IntentFilter("CartaServicio.NUEVA"));
    	registerReceiver(receiver, new IntentFilter("CartaServicio.SIN"));
	}

	@Override
	protected void onPause() {
		super.onPause();
		unregisterReceiver(receiver);
		finish();
	}
	
	protected void onListItemClick(ListView l, View v, int position, long id) {
		mensaje = mensajes[position];
		
		Intent i = new Intent(MensajeListar.this, MensajeInfo.class);
        this.startActivity(i);
        MensajeListar.this.setResult(RESULT_OK);
		MensajeListar.this.finish();
	}
	
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        super.onCreateOptionsMenu(menu); 
        menu.add(0, MENU_INFORMAR, 0 , getString(R.string.informacion))
        .setIcon(R.drawable.ic_menu_informacion);
        return true;
    }
    
    @Override
    public boolean onOptionsItemSelected (MenuItem item) {
    	switch (item.getItemId()) {
    		case MENU_INFORMAR:
    			showAlertInformacion();
    			break;
    	}
    	return super.onOptionsItemSelected(item);
    }
	
	private void showToastConexion(){
		Toast toast = Toast.makeText(getBaseContext(),
				getString(R.string.errConexion),
				Toast.LENGTH_SHORT);
		toast.show();
		MensajeListar.this.setResult(RESULT_OK);
		MensajeListar.this.finish();
	}
	
	private void showToastDatos(){
		Toast toast = Toast.makeText(getBaseContext(),
				getString(R.string.errDatos),
				Toast.LENGTH_SHORT);
		toast.show();
		MensajeListar.this.setResult(RESULT_OK);
		MensajeListar.this.finish();
	}

	private void showAlertListaVacia(){
		AlertDialog.Builder builder = new AlertDialog.Builder(MensajeListar.this);
		builder.setMessage(R.string.noMensajes)
	           .setCancelable(false)
	           .setPositiveButton(R.string.butContinuar, new DialogInterface.OnClickListener() {
	               public void onClick(DialogInterface dialog, int id) {
	            	   MensajeListar.this.setResult(RESULT_OK);
	            	   MensajeListar.this.finish();
	               }
	           });
	    AlertDialog alert = builder.create();
	    alert.setTitle(R.string.titLisMensajes);
	    alert.setIcon(R.drawable.ic_dialog_informacion);
		alert.show();
	}
	
	private void showAlertInformacion(){
    	AlertDialog.Builder builder = new AlertDialog.Builder(this);
    	builder.setMessage(getString(R.string.infoMensajeListar))
    	       .setCancelable(false)
    	       .setPositiveButton(getString(R.string.butContinuar), new DialogInterface.OnClickListener() {
    	           public void onClick(DialogInterface dialog, int id) {
    	        	   dialog.cancel();
    	           }
    	       });
    	AlertDialog alert = builder.create();
    	alert.setTitle(getString(R.string.informacion));
    	alert.setIcon(R.drawable.ic_dialog_informacion);
    	alert.show();
    }

	private void mensajeListar(){
        String title = getString(R.string.espere);
        String text = getString(R.string.obtenerMensaje);
		
		progressDialog = ProgressDialog.show(this, title, text, true, false);
		progressDialog.setIcon(R.drawable.ic_dialog_conexion);
        Thread thread = new Thread(this);
        thread.start();
	}
	
	public void run() {
		Looper.prepare();
		conexionMensajeListar();
		Looper.myLooper().quit();
		Looper.loop();
	}
	
	public void conexionMensajeListar() {
		int resultado;
		try{
			DefaultHttpClient httpclient = new DefaultHttpClient();
			int idJuego = JuegoListar.getJuego().getIdentificador();
			int idEquipo = JuegoDatosListar.getEquipo().getIdentificador();
			URI uri = new URI(GoldenStreets.urlServer+"juego/"+idJuego+"/equipo/"+idEquipo+"/mensaje/listar/?format=json");
			HttpGet httpget = new HttpGet(uri);
			LibreGeoSocial LGS = LibreGeoSocial.getInstance();
			httpclient.setCookieStore(LGS.getCookieStore());
			HttpResponse response = httpclient.execute(httpget);
			HttpEntity entity = response.getEntity();
			String str = Utils.deStreamAString(entity.getContent());
			resultado = procesarRespuestaMensajeListar(str);
		
			handler.sendEmptyMessage(resultado);		

		}catch(URISyntaxException urisex){
			Log.e(TAG, urisex.getMessage());
			handler.sendEmptyMessage(-1);
	    }catch (IOException e) {
	    	Log.e(TAG,e.getMessage());
	    	handler.sendEmptyMessage(-1);
	    }
	}
	
	private int procesarRespuestaMensajeListar (String str) {
		String ultMensaje = null;    	
		int ultCode = 0;
		
	    try {
	    	JSONObject json;
	    	json = new JSONObject(str.toString());
	
			if (json.has("code"))
				ultCode = json.getInt("code");
			else
				ultCode = -1;
			
			if (json.has("descripcion"))
				ultMensaje = json.getString("descripcion");
			else
				ultMensaje = null;
			
			listDelEquipo = new ArrayList<String>();
			listParaEquipo = new ArrayList<String>();
			listTexto = new ArrayList<String>();
			int numeroMensajes = 0;
			
			if (json.has("mensajes") && ultMensaje.toLowerCase().equals("ok")){
				JSONArray mensajesObjects = json.getJSONArray("mensajes");
				numeroMensajes = mensajesObjects.length()-1;
				mensajes = new Mensaje[numeroMensajes];
				int mensajeIdentificador;
				String mensajeDelEquipo;
				String mensajeParaEquipo;
				String mensajeTexto;
	
				for (int i = 0; i < numeroMensajes; i++){
					JSONObject JSONMensaje = (JSONObject)mensajesObjects.get(i);
					mensajeIdentificador = JSONMensaje.getInt("identificador");
					mensajeDelEquipo = JSONMensaje.getString("delEquipo");
					mensajeParaEquipo = JSONMensaje.getString("paraEquipo");
					mensajeTexto = JSONMensaje.getString("texto");
	
					mensajes[i] = new Mensaje(mensajeIdentificador, mensajeDelEquipo, 
							mensajeParaEquipo, mensajeTexto);
					
					listDelEquipo.add(getString(R.string.delEquipo).concat(mensajes[i].getDelEquipo()));
					listParaEquipo.add(getString(R.string.paraEquipo).concat(mensajes[i].getParaEquipo()));
					listTexto.add(mensajes[i].getTexto());
				}
			}		
			if (ultCode != 200)
				return -2;
			if (numeroMensajes == 0)
				return 1;
	
		} catch (Exception e) {	
			ultCode = 500;
			ultMensaje = "Error in json parsed: " + e.getMessage();
			Log.e(TAG, ultMensaje);
			return -2;
		}
		return 0;
	}

	private Handler handler = new Handler() {
		@Override
		public void handleMessage(Message msg) {
			int result = msg.what;		
			progressDialog.cancel();			
			if (result == -2){        		
				showToastDatos();
        	}else if (result == -1){
        		showToastConexion();
        	}else if (result == 0){        		
				mAdapter = new EventsAdapter(MensajeListar.this);
				setListAdapter(mAdapter);
        	}else if (result == 1){
        		showAlertListaVacia();        		
        	}
		}
	};
	
    public static class EventsAdapter extends BaseAdapter {

		private Context mContext;
		
		public EventsAdapter (Context c) {
            mContext = c;
        }

		public int getCount() {
			// TODO Auto-generated method stub
			return listDelEquipo.size();
		}

		public Object getItem(int position) {
			// TODO Auto-generated method stub
			return position;
		}

		public long getItemId(int position) {
			// TODO Auto-generated method stub
			return position;
		}

		public View getView(int position, View convertView, ViewGroup parent) {
			// TODO Auto-generated method stub
			View view;
			
	        if (convertView == null) {
	            LayoutInflater inflater = (LayoutInflater) mContext
	                    .getSystemService(Context.LAYOUT_INFLATER_SERVICE);
	            view = inflater.inflate(R.layout.mensajelistar, null);
	        } else {
	            view = convertView;
	        }
	        
	        TextView textDe = (TextView) view.findViewById(R.id.textDe);
	        textDe.setText((String)listDelEquipo.get(position));
			
			TextView textPara = (TextView) view.findViewById(R.id.textPara);
			textPara.setText((String)listParaEquipo.get(position));
			
			TextView textTexto = (TextView) view.findViewById(R.id.textTexto);
			textTexto.setText((String)listTexto.get(position));
			
	        LinearLayout linear = (LinearLayout)view.findViewById(R.id.linearLayout);
	        if (listDelEquipo.get(position).toLowerCase().equals("de:"+JuegoDatosListar.getEquipo().getGroupName().toLowerCase()))
	        	linear.setBackgroundDrawable(view.getResources().getDrawable(R.drawable.back_list_rojo));
	        else
	        	if (listDelEquipo.get(position).toLowerCase().indexOf("organizador") > 0)
	        		linear.setBackgroundDrawable(view.getResources().getDrawable(R.drawable.back_list_azul));
	        	else
	        		linear.setBackgroundDrawable(view.getResources().getDrawable(R.drawable.back_list_verde));
	        
			return view;       
		}
	}
    
    private BroadcastReceiver receiver = new BroadcastReceiver() {        
        @Override
        public void onReceive(Context context, Intent intent) {
        	String accion = intent.getAction();
        	if (accion.equals("CartaServicio.NUEVA")) {
        		AlertDialog alert = AccionesRecibidasBroadcast.accionCartaServicioNueva(context, intent);
		    	alert.show();
        	} else if(accion.equals("CartaServicio.SIN")) {
        		AlertDialog alert = AccionesRecibidasBroadcast.accionCartaServicioSin(context, intent);
		    	alert.show();
        	}
        }
    };
}