package com.Goldenstreet.App.GoldenStreet;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.util.Log;
import android.view.Gravity;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.WindowManager;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.AdapterView.OnItemSelectedListener;

import com.Goldenstreet.App.R;
import com.LibreGeoSocial.Api.LibreGeoSocial;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.List;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.NameValuePair;
import org.apache.http.protocol.HTTP;
import org.json.JSONException;
import org.json.JSONObject;

/**
 * Muestra un formulario para la creacin y envo de una nueva mensaje. Como destinatario
 * el equipo podr elegir a otro equipo participante en la partida o al organizador
 * de la misma.
 * 
 * @author Rubn Quero Castrillo <rubenquero@gmail.com>
 * 
 */

public class MensajeNuevo extends Activity implements Runnable, OnClickListener {
	static private String TAG = "MensajeNuevo";
	
	private static final int MENU_INFORMAR = Menu.FIRST;
	private String equipoSeleccionado = null;
	private EditText editTexto;	
	private Button buttonEnviar;
	private ProgressDialog progressDialog = null;
	
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.mensajenuevo);
        
        TextView textDe = (TextView)this.findViewById(R.id.textDe);
        textDe.setText((String)getString(R.string.delEquipo).concat(JuegoDatosListar.getEquipo().getGroupName()));
        
        Spinner spinnerDestinatario = (Spinner) findViewById(R.id.spinnerDestinatario);
        ArrayList<String> destinatarios = new ArrayList<String>();
        destinatarios.add("organizador");
        try{
	        for (int i = 0; i < JuegoDatosListar.getEquipos().length; i++){
	        	destinatarios.add(i+1, JuegoDatosListar.getEquipos()[i].getGroupName());
	        }
        }catch(NullPointerException npe){
        	Log.d(TAG, npe.toString());
        }
        ArrayAdapter<String> spinnerArrayAdapter = new ArrayAdapter<String>(this, android.R.layout.simple_spinner_dropdown_item, destinatarios);
        spinnerDestinatario.setAdapter(spinnerArrayAdapter);
        spinnerDestinatario.setOnItemSelectedListener(new MyOnItemSelectedListener());

        editTexto = (EditText)this.findViewById(R.id.editTexto);
        getWindow().setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_STATE_HIDDEN);
        buttonEnviar = (Button)this.findViewById(R.id.buttonEnviar);
        buttonEnviar.setOnClickListener(this);
    }
    
    @Override
	protected void onResume() {
		super.onResume();
		registerReceiver(receiver, new IntentFilter("CartaServicio.NUEVA"));
    	registerReceiver(receiver, new IntentFilter("CartaServicio.SIN"));
	}

	@Override
	protected void onPause() {
		super.onPause();
		unregisterReceiver(receiver);
		finish();
	}
    
	public void onClick(View v) {
		if (buttonEnviar.isPressed()){
			if (editTexto.getText().toString().trim().equals("")){
	    	    Context context = getApplicationContext();
			    Toast toast = Toast.makeText(context, getString(R.string.mensajeVacio), Toast.LENGTH_SHORT);
			    toast.setGravity(Gravity.CENTER_VERTICAL, 0, 0);
			    toast.show();
	    	}
			else{
				mensajeNuevo();
			}
		}
	}
	
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        super.onCreateOptionsMenu(menu); 
        menu.add(0, MENU_INFORMAR, 0 , getString(R.string.informacion))
        .setIcon(R.drawable.ic_menu_informacion);
        return true;
    }
    
    @Override
    public boolean onOptionsItemSelected (MenuItem item) {
    	switch (item.getItemId()) {
    		case MENU_INFORMAR:
    			showAlertInformacion();
    			break;
    	}
    	return super.onOptionsItemSelected(item);
    }
	
	private void showToastConexion(){
		Toast toast = Toast.makeText(getBaseContext(),
				getString(R.string.errConexion),
				Toast.LENGTH_SHORT);
		toast.show();
		MensajeNuevo.this.setResult(RESULT_OK);
		MensajeNuevo.this.finish();
	}
	
	private void showToastDatos(){
		Toast toast = Toast.makeText(getBaseContext(),
				getString(R.string.errDatos),
				Toast.LENGTH_SHORT);
		toast.show();
		MensajeNuevo.this.setResult(RESULT_OK);
		MensajeNuevo.this.finish();
	}

	private void showToastAceptada(){
		Toast toast = Toast.makeText(getBaseContext(), 
				getString(R.string.mensajeEnviado), 
				Toast.LENGTH_LONG);
	    toast.show();
	    MensajeNuevo.this.setResult(RESULT_OK);
	    MensajeNuevo.this.finish();
	}
	
	private void mensajeNuevo(){
		String title = getString(R.string.envioMensaje);
        String text = getString(R.string.espere);    
		
		progressDialog = ProgressDialog.show(this, title, text, true, false);
		progressDialog.setIcon(R.drawable.ic_dialog_conexion);
        Thread thread = new Thread(this);
        thread.start();
	}
	
	private void showAlertInformacion(){
    	AlertDialog.Builder builder = new AlertDialog.Builder(this);
    	builder.setMessage(getString(R.string.infoMensajeNuevo))
    	       .setCancelable(false)
    	       .setPositiveButton(getString(R.string.butContinuar), new DialogInterface.OnClickListener() {
    	           public void onClick(DialogInterface dialog, int id) {
    	        	   dialog.cancel();
    	           }
    	       });
    	AlertDialog alert = builder.create();
    	alert.setTitle(getString(R.string.informacion));
    	alert.setIcon(R.drawable.ic_dialog_informacion);
    	alert.show();
    }
	
	@Override
	public void run() {
		Looper.prepare();
		conexionMensajeNuevo();
		Looper.myLooper().quit();
		Looper.loop();
	}
	
	private void conexionMensajeNuevo(){
		int idEquipoSeleccionado;
		int resultado;
		try{
			DefaultHttpClient httpclient = new DefaultHttpClient();
			int idJuego = JuegoListar.getJuego().getIdentificador();
			URI uri = new URI(GoldenStreets.urlServer+"juego/"+idJuego+"/mensaje/crear/?format=json");
            HttpPost httpost = new HttpPost(uri);
			LibreGeoSocial LGS = LibreGeoSocial.getInstance();
			httpclient.setCookieStore(LGS.getCookieStore());
			
            List <NameValuePair> datos = new ArrayList <NameValuePair>();
            datos.add(new BasicNameValuePair("emisor", Integer.toString(JuegoDatosListar.getEquipo().getIdentificador())));
            if (equipoSeleccionado.equals("organizador")){
            	datos.add(new BasicNameValuePair("destinatario", "organizador"));
            }else{
            	datos.add(new BasicNameValuePair("destinatario", "unEquipo"));
            	idEquipoSeleccionado = Equipo.encontraPorNombre(JuegoDatosListar.getEquipos(), equipoSeleccionado).getIdentificador();
            	datos.add(new BasicNameValuePair("equipos", Integer.toString(idEquipoSeleccionado)));
            }
            datos.add(new BasicNameValuePair("texto", editTexto.getText().toString().trim()));
            httpost.setEntity(new UrlEncodedFormEntity(datos, HTTP.UTF_8));
            
            HttpResponse response = httpclient.execute(httpost);
            HttpEntity entity = response.getEntity();
            String str = Utils.deStreamAString(entity.getContent());
            resultado = procesarRespuestaMensajeNuevo(str);		
    		
            handler.sendEmptyMessage(resultado);
            
    	}catch(URISyntaxException urisex){
    		Log.d(TAG, urisex.getMessage());
    		handler.sendEmptyMessage(-1);
        }catch (IOException e) {
            Log.e(TAG,e.getMessage());
            handler.sendEmptyMessage(-1);
        }
	}
	
	private int procesarRespuestaMensajeNuevo(String str) {
		int ultCode;
    	String ultMensaje;
        try {
        	JSONObject json;
			json = new JSONObject(str);
			
			if (json.has("code"))
				ultCode = json.getInt("code");
			else
				ultCode = -1;
			
			if (json.has("descripcion"))
				ultMensaje = json.getString("descripcion");
			else
				ultMensaje = null;
			 
			if (ultCode != 200){
				return -2;
			}
			
		} catch (JSONException e) {	
			ultCode = 500;
			ultMensaje = "Error in json parsed: " + e.getMessage();
			Log.e(TAG, ultMensaje);
			return -2;
		}
		return 0;
	}
	
    private Handler handler = new Handler() {
    	@Override
		public void handleMessage(Message msg) {
    		int result = msg.what;
    		progressDialog.cancel();			
			if (result == -2){        		
				showToastDatos();
        	}else if (result == -1){
        		showToastConexion();
        	}else if (result == 0){        		
        		showToastAceptada();
        	}
		}
	};

	private class MyOnItemSelectedListener implements OnItemSelectedListener{
    	public void onItemSelected(AdapterView<?> parent, View view, int pos, long id) {
    		equipoSeleccionado = parent.getItemAtPosition(pos).toString(); 
    	}

    	public void onNothingSelected(AdapterView<?> parent) {
    		// Do nothing.
    	}
    }
	
    private BroadcastReceiver receiver = new BroadcastReceiver() {        
        @Override
        public void onReceive(Context context, Intent intent) {
        	String accion = intent.getAction();
        	if (accion.equals("CartaServicio.NUEVA")) {
        		AlertDialog alert = AccionesRecibidasBroadcast.accionCartaServicioNueva(context, intent);
		    	alert.show();
        	} else if(accion.equals("CartaServicio.SIN")) {
        		AlertDialog alert = AccionesRecibidasBroadcast.accionCartaServicioSin(context, intent);
		    	alert.show();
        	}
        }
    };
}