package com.Goldenstreet.App.GoldenStreet;

import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Intent;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.util.Log;
import android.widget.Toast;

import com.Goldenstreet.App.R;
import com.LibreGeoSocial.Api.LibreGeoSocial;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.Timer;
import java.util.TimerTask;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.json.JSONException;
import org.json.JSONObject;

/**
 * Servicio que peridicamente consultar si el equipo ha recibido un nuevo mensaje, y avisar
 * a travs de una notificacin al equipo.
 * 
 * @author Rubn Quero Castrillo <rubenquero@gmail.com>
 * 
 */

public class MensajeServicio extends Service{
	static private String TAG = "MensajeServicio";
	
	private static final int RETARDO = 10*1000; 
	private static final int PERIODO = 60*1000;
	private static final int MENSAJE_RECIBIDO = 3;

	private int mensajeId;
	private int ultMensajeId;
	private NotificationManager notificationManager;
	private Timer timer;	

	@Override
	public void onCreate() {
		super.onCreate();
		mensajeId = 0;
		ultMensajeId = 0;
		notificationManager = (NotificationManager)getSystemService(NOTIFICATION_SERVICE);
		timer = new Timer("mensajeServicio");
	}
	
	@Override
	public void onDestroy() {
		super.onDestroy();
		timer.cancel();
		notificationManager.cancelAll();
	}
	
 	@Override
 	public void onStart(final Intent intent, final int startId){
 		super.onStart(intent, startId);
 		timer.scheduleAtFixedRate(timerTask, RETARDO, PERIODO);
 	}
 	
	@Override
	public IBinder onBind(Intent arg0) {
		// TODO Auto-generated method stub
		return null;
	}
 	
	private void showToastConexion(){			
		Toast toast = Toast.makeText(getBaseContext(),
				getBaseContext().getString(R.string.errConexion),
				Toast.LENGTH_SHORT);
		toast.show();
	}
	
	private void showToastDatos(){
		Toast toast = Toast.makeText(getBaseContext(),
				getString(R.string.errDatos),
				Toast.LENGTH_SHORT);
		toast.show();
	}

	private TimerTask timerTask = new TimerTask() {
		@Override
		public void run() {
			mensajeHistorico();
		}
	};
	
	private void mensajeHistorico(){
		int resultado; 
		try{
			DefaultHttpClient httpclient = new DefaultHttpClient();
			int idJuego = JuegoListar.getJuego().getIdentificador();
			int idEquipo = JuegoDatosListar.getEquipo().getIdentificador();
			URI uri = new URI(GoldenStreets.urlServer+"juego/"+idJuego+"/equipo/"+idEquipo+"/mensaje/historico/?format=json");
			HttpGet httpget = new HttpGet(uri);
			LibreGeoSocial LGS = LibreGeoSocial.getInstance();
			httpclient.setCookieStore(LGS.getCookieStore());
			HttpResponse response = httpclient.execute(httpget);
			HttpEntity entity = response.getEntity();
			String str = Utils.deStreamAString(entity.getContent());
			resultado = procesarRespuestaMensajeHistorico(str);
			
    		handler.sendEmptyMessage(resultado);

		}catch(URISyntaxException urisex){
			Log.e(TAG, urisex.getMessage());
			handler.sendEmptyMessage(-1);
	    }catch (IOException e) {
	    	Log.e(TAG,e.getMessage());
	    	handler.sendEmptyMessage(-1);
	    }
	}
	
	private int procesarRespuestaMensajeHistorico (String str) {
		int ultCode;
    	String ultMensaje;
        try {
        	JSONObject json;
			json = new JSONObject(str);
			
			if (json.has("code"))
				ultCode = json.getInt("code");
			else
				ultCode = -1;
			
			if (json.has("descripcion"))
				ultMensaje = json.getString("descripcion");
			else
				ultMensaje = null;
			 
			if (json.has("ultMensajeRecibidoId"))
				ultMensajeId = json.getInt("ultMensajeRecibidoId");
			else
				ultMensajeId = 0;
			
			if (ultCode != 200){
				return -2;
			}
			
		} catch (JSONException e) {	
			ultCode = 500;
			ultMensaje = "Error in json parsed: " + e.getMessage();
			Log.e(TAG, ultMensaje);
			return -2;
		}
		return 0;
	}
	
	private void mensajeRecibidoNotification()
    {
		CharSequence title = getString(R.string.notTituloMensajeRecibido);
        CharSequence message = getString(R.string.notMensajeMensajeRecibido);
        CharSequence tickerText = getString(R.string.notTextoMensajeRecibido);
        int icon = R.drawable.ic_stat_notify_mensaje_recibido;
        long when = System.currentTimeMillis();
        
        Notification notification = new Notification(icon, tickerText, when);
        notification.defaults |= Notification.DEFAULT_SOUND;
        notification.defaults |= Notification.DEFAULT_VIBRATE;
        notification.defaults |= Notification.DEFAULT_LIGHTS;  
        notification.flags |= Notification.FLAG_SHOW_LIGHTS;
        notification.flags |= Notification.FLAG_AUTO_CANCEL;

        Intent notificationIntent = new Intent(MensajeServicio.this, MensajeListar.class);
        notificationIntent.setAction(String.valueOf(when));
        PendingIntent pendingIntent = PendingIntent.getActivity(this, 0, notificationIntent, PendingIntent.FLAG_UPDATE_CURRENT);
        notification.setLatestEventInfo(MensajeServicio.this, title, message, pendingIntent);
       
        notificationManager.notify(MENSAJE_RECIBIDO, notification);
    }		
	
    private Handler handler = new Handler() {
    	@Override
		public void handleMessage(Message msg) {
			int result = msg.what;	
			if (result == -2){        		
				showToastDatos();
        	}else if (result == -1){
        		showToastConexion();
        	}else if (result == 0){        		
        		if(ultMensajeId > mensajeId){
					mensajeId = ultMensajeId;
					mensajeRecibidoNotification();
				}
        	}
		}
	};
}