package com.Goldenstreet.App.GoldenStreet;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.List;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.protocol.HTTP;
import org.json.JSONException;
import org.json.JSONObject;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import com.Goldenstreet.App.R;
import com.LibreGeoSocial.Api.LibreGeoSocial;

/**
 * Muestra de forma detallada la informacin de la oferta seleccionada (ofertante, propietario, propiedad,
 * precio de la oferta y una imagen con la respuesta). El equipo siempre podr realizar diferentes acciones
 * sobre la oferta, siempre y cuando no haya sido respondida por el propietario o retirada por el ofertante.
 * 
 * @author Rubn Quero Castrillo <rubenquero@gmail.com>
 * 
 */

public class OfertaInfo extends Activity implements OnClickListener {
	static private String TAG = "OfertaInfo";
	
	private Button buttonOpcion;
	private Button buttonRechazar;
	private String tipoOferta;
	private int versionCalle;
	private ProgressDialog progressDialog = null;
	
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.ofertainfo);
        Oferta oferta = OfertaListar.getOferta();
        
        Bundle bundle;
		bundle = this.getIntent().getExtras();
    	tipoOferta = (String)bundle.getString("tipoOferta");
    	versionCalle = bundle.getInt("versionCalle");
        
    	TextView textOfertante = (TextView)this.findViewById(R.id.textOfertante);
        textOfertante.setText((String)getString(R.string.ofertante).concat(oferta.getOfertante()));
        
        TextView textPropietario = (TextView)this.findViewById(R.id.textPropietario);
        textPropietario.setText((String)getString(R.string.propietario).concat(oferta.getPropietario()));
        
        TextView textPropiedad = (TextView)this.findViewById(R.id.textPropiedad);
        textPropiedad.setText((String)getString(R.string.propiedad).concat(oferta.getNombreCalle()));
        
        TextView textPrecioOferta = (TextView)this.findViewById(R.id.textPrecioOferta);
        textPrecioOferta.setText((String)getString(R.string.precioOferta).concat(String.valueOf(oferta.getCantidad())));
        
        buttonOpcion = (Button)this.findViewById(R.id.buttonOpcion);
        buttonRechazar = (Button)this.findViewById(R.id.buttonRechazar);
        
        ImageView imageRespuesta = (ImageView)this.findViewById(R.id.imageRespuesta);
		if(oferta.getRespondida()){
			if(oferta.getAceptada())
				imageRespuesta.setImageResource(R.drawable.ic_list_oferta_aceptada);
			else
				imageRespuesta.setImageResource(R.drawable.ic_list_oferta_rechazada);			
			buttonOpcion.setVisibility(View.GONE);
			buttonRechazar.setVisibility(View.GONE);
		}else{
			if (tipoOferta.compareTo("enviada") == 0){
	        	buttonOpcion.setText(getString(R.string.butRetirar));
	        	buttonRechazar.setVisibility(View.GONE);
	        }else{
	        	buttonOpcion.setText(getString(R.string.butAceptar));
	        	buttonRechazar.setVisibility(View.VISIBLE);
	        	buttonRechazar.setOnClickListener(this);
	        }
			imageRespuesta.setImageResource(R.drawable.ic_list_oferta_sinrespuesta);
			buttonOpcion.setVisibility(View.VISIBLE);
			buttonOpcion.setOnClickListener(this);
		}
    }
    
	@Override
	protected void onResume() {
		super.onResume();
		registerReceiver(receiver, new IntentFilter("CartaServicio.NUEVA"));
    	registerReceiver(receiver, new IntentFilter("CartaServicio.SIN"));
	}

	@Override
	protected void onPause() {
		super.onPause();
		unregisterReceiver(receiver);
		finish();
	}
    
    public void onClick(View v) {
    	String title = getString(R.string.espere);
	    String text = getString(R.string.procesandoAccion);
	    
    	progressDialog = ProgressDialog.show(this, title, text, true, false);
	    progressDialog.setIcon(R.drawable.ic_dialog_conexion);	    
    	if ( buttonOpcion.isPressed() ){
    		if (tipoOferta.compareTo("enviada") == 0){
    			Thread thread = new Thread(retirarOferta);
    		    thread.start();
    		}else{
    			Thread thread = new Thread(aceptarOferta);
    		    thread.start();
    		}
        }else if (buttonRechazar.isPressed() ){
        	Thread thread = new Thread(rechazarOferta);
        	thread.start();
        }
    }
    
	private void showToastConexion(){
		Toast toast = Toast.makeText(getBaseContext(),
				getString(R.string.errConexion),
				Toast.LENGTH_SHORT);
		toast.show();
	}
	
	private void showToastDatos(){
		Toast toast = Toast.makeText(getBaseContext(),
				getString(R.string.errDatos),
				Toast.LENGTH_SHORT);
		toast.show();
	}
	
	private void showToastAceptada(){
		Toast toast = Toast.makeText(getBaseContext(),
				getString(R.string.ofertaAceptada),
				Toast.LENGTH_LONG);
		toast.show();
	}
	
	private void showToastRechazadaOferta(){
		Toast toast = Toast.makeText(getBaseContext(),
				getString(R.string.ofertaRechazada),
				Toast.LENGTH_LONG);
		toast.show();
	}
	
	private void showToastRechazadaCalle(){
		Toast toast = Toast.makeText(getBaseContext(),
				getString(R.string.ofertaCalleRechazada),
				Toast.LENGTH_LONG);
		toast.show();
	}
	
	private void showToastSinDinero(){
		Toast toast = Toast.makeText(getBaseContext(),
				getString(R.string.ofertaSinDinero),
				Toast.LENGTH_LONG);
		toast.show();
	}
    
    private Runnable aceptarOferta = new Runnable(){
		@Override
		public void run(){
			Looper.prepare();
			conexionAceptarOferta();
			Looper.myLooper().quit();
			Looper.loop();
		}
		
		public void conexionAceptarOferta(){
			int resultado;
			try{
				DefaultHttpClient httpclient = new DefaultHttpClient();
				int idJuego = JuegoListar.getJuego().getIdentificador();
				int idEquipo = JuegoDatosListar.getEquipo().getIdentificador();
				
				URI uri = new URI(GoldenStreets.urlServer+"juego/"+idJuego+"/equipo/"+idEquipo+"/oferta/"+OfertaListar.getOferta().getIdentificador()+"/aceptar/?format=json");
				HttpPost httpost = new HttpPost(uri);
				LibreGeoSocial LGS = LibreGeoSocial.getInstance();
				httpclient.setCookieStore(LGS.getCookieStore());
				
				List <NameValuePair> datos = new ArrayList <NameValuePair>();
	            datos.add(new BasicNameValuePair("calleVersion", String.valueOf(versionCalle)));
	            httpost.setEntity(new UrlEncodedFormEntity(datos, HTTP.UTF_8));
	            
	            HttpResponse response = httpclient.execute(httpost);
	            HttpEntity entity = response.getEntity();
				String str = Utils.deStreamAString(entity.getContent());
				resultado = procesarRespuestaAceptarOferta(str);
				handler.sendEmptyMessage(resultado);  		

			}catch(URISyntaxException urisex){
				Log.e(TAG, urisex.getMessage());
				handler.sendEmptyMessage(-1);
		    }catch (IOException e) {
		    	Log.e(TAG,e.getMessage());
		    	handler.sendEmptyMessage(-1);
		    }
		}
		
		private int procesarRespuestaAceptarOferta(String str) {
			String ultMensaje = null;
			int ultCode = 0;
			
			try {
	        	JSONObject json;
				json = new JSONObject(str);
				
				if (json.has("code"))
					ultCode = json.getInt("code");
				else
					ultCode = -1;
				
				if (json.has("descripcion"))
					ultMensaje = json.getString("descripcion");
				else
					ultMensaje = null;
				
				if (ultCode != 200){
					if (ultCode == 400 && ultMensaje.toLowerCase().compareTo("calle con modificaciones") == 0)
						return -5;
					if (ultCode == 400 && ultMensaje.toLowerCase().compareTo("oferta con modificaciones") == 0)
						return -4;
					if (ultCode == 400 && ultMensaje.toLowerCase().compareTo("sin dinero") == 0)
						return -3;
					return -2;
				}
			} catch (JSONException e) {	
				ultCode = 500;
				ultMensaje = "Error in json parsed: " + e.getMessage();
				Log.e(TAG, ultMensaje);
				return -2;
			}
			return 0;
		}	    
	};
    
	private Runnable rechazarOferta = new Runnable(){
		@Override
		public void run(){
			Looper.prepare();
			conexionRechazarOferta();
			Looper.myLooper().quit();
			Looper.loop();
		}
		
		public void conexionRechazarOferta(){
			int resultado;
			try{
				DefaultHttpClient httpclient = new DefaultHttpClient();
				int idJuego = JuegoListar.getJuego().getIdentificador();
				int idEquipo = JuegoDatosListar.getEquipo().getIdentificador();
				
				URI uri = new URI(GoldenStreets.urlServer+"juego/"+idJuego+"/equipo/"+idEquipo+"/oferta/"+OfertaListar.getOferta().getIdentificador()+"/rechazar/?format=json");
				HttpPost httpost = new HttpPost(uri);
				LibreGeoSocial LGS = LibreGeoSocial.getInstance();
				httpclient.setCookieStore(LGS.getCookieStore());
	            
	            HttpResponse response = httpclient.execute(httpost);
	            HttpEntity entity = response.getEntity();
				String str = Utils.deStreamAString(entity.getContent());
				resultado = procesarRespuestaRechazarOferta(str);
				handler.sendEmptyMessage(resultado);  		

			}catch(URISyntaxException urisex){
				Log.e(TAG, urisex.getMessage());
				handler.sendEmptyMessage(-1);
		    }catch (IOException e) {
		    	Log.e(TAG,e.getMessage());
		    	handler.sendEmptyMessage(-1);
		    }
		}
		
		private int procesarRespuestaRechazarOferta(String str) {
			String ultMensaje = null;
			int ultCode = 0;
			
			try {
	        	JSONObject json;
				json = new JSONObject(str);
				
				if (json.has("code"))
					ultCode = json.getInt("code");
				else
					ultCode = -1;
				
				if (json.has("descripcion"))
					ultMensaje = json.getString("descripcion");
				else
					ultMensaje = null;
				
				if (ultCode != 200){
					if (ultCode == 400 && ultMensaje.toLowerCase().compareTo("oferta con modificaciones") == 0)
						return -4;
					return -2;
				}
			} catch (JSONException e) {	
				ultCode = 500;
				ultMensaje = "Error in json parsed: " + e.getMessage();
				Log.e(TAG, ultMensaje);
				return -2;
			}
			return 0;
		}
	};
	
	private Runnable retirarOferta = new Runnable(){
		@Override
		public void run(){
			Looper.prepare();
			conexionRetirarOferta();
			Looper.myLooper().quit();
			Looper.loop();
		}
		
		public void conexionRetirarOferta(){
			int resultado;
			try{
				DefaultHttpClient httpclient = new DefaultHttpClient();
				int idJuego = JuegoListar.getJuego().getIdentificador();
				int idEquipo = JuegoDatosListar.getEquipo().getIdentificador();
				
				URI uri = new URI(GoldenStreets.urlServer+"juego/"+idJuego+"/equipo/"+idEquipo+"/oferta/"+OfertaListar.getOferta().getIdentificador()+"/retirar/?format=json");
				HttpPost httpost = new HttpPost(uri);
				LibreGeoSocial LGS = LibreGeoSocial.getInstance();
				httpclient.setCookieStore(LGS.getCookieStore());
	            
	            HttpResponse response = httpclient.execute(httpost);
	            HttpEntity entity = response.getEntity();
				String str = Utils.deStreamAString(entity.getContent());
				resultado = procesarRespuestaRetirarOferta(str);
				handler.sendEmptyMessage(resultado);  		

			}catch(URISyntaxException urisex){
				Log.e(TAG, urisex.getMessage());
				handler.sendEmptyMessage(-1);
		    }catch (IOException e) {
		    	Log.e(TAG,e.getMessage());
		    	handler.sendEmptyMessage(-1);
		    }
		}
		
		private int procesarRespuestaRetirarOferta(String str) {
			String ultMensaje = null;
			int ultCode = 0;
			
			try {
	        	JSONObject json;
				json = new JSONObject(str);
				
				if (json.has("code"))
					ultCode = json.getInt("code");
				else
					ultCode = -1;
				
				if (json.has("descripcion"))
					ultMensaje = json.getString("descripcion");
				else
					ultMensaje = null;
				
				if (ultCode != 200){
					if (ultCode == 400 && ultMensaje.toLowerCase().compareTo("oferta con modificaciones") == 0)
						return -4;
					return -2;
				}
			} catch (JSONException e) {	
				ultCode = 500;
				ultMensaje = "Error in json parsed: " + e.getMessage();
				Log.e(TAG, ultMensaje);
				return -2;
			}
			return 0;
		}
	};
	
	private Handler handler = new Handler()
    {
    	@Override
    	public void handleMessage(Message msg)
        {
        	int result = msg.what;
        	progressDialog.cancel();
        	if(result == -5){
        		showToastRechazadaCalle();
        	}else if(result == -4){
        		showToastRechazadaOferta();
        	}else if(result == -3){
        		showToastSinDinero();
        	}else if(result == -2){
        		showToastDatos();
        	}else if(result == -1){
        		showToastConexion();
        	}else if(result == 0){
        		showToastAceptada();
        	}
    		OfertaInfo.this.setResult(RESULT_OK);
    		OfertaInfo.this.finish();
        }
    };
    
    private BroadcastReceiver receiver = new BroadcastReceiver() {        
        @Override
        public void onReceive(Context context, Intent intent) {
        	String accion = intent.getAction();
        	if (accion.equals("CartaServicio.NUEVA")) {
        		AlertDialog alert = AccionesRecibidasBroadcast.accionCartaServicioNueva(context, intent);
		    	alert.show();
        	} else if(accion.equals("CartaServicio.SIN")) {
        		AlertDialog alert = AccionesRecibidasBroadcast.accionCartaServicioSin(context, intent);
		    	alert.show();
        	}
        }
    };
}