package com.Goldenstreet.App.GoldenStreet;

import android.app.AlertDialog;
import android.app.ListActivity;
import android.app.ProgressDialog;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import com.Goldenstreet.App.R;
import com.LibreGeoSocial.Api.LibreGeoSocial;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.impl.client.DefaultHttpClient;
import org.json.JSONArray;
import org.json.JSONObject;

/**
 * Muestra la lista de ofertas enviadas/recibidas segn la opcin elegida en la tab Ofertas.
 * Si el equipo selecciona una oferta se pasar a una nueva activity en la que se mostrar de forma
 * detallada informacin sobre la oferta seleccionada (ofertante, propietario, propiedad, etc.).
 * 
 * @author Rubn Quero Castrillo <rubenquero@gmail.com>
 * 
 */

public class OfertaListar extends ListActivity implements Runnable { 
	static private String TAG = "OfertaListar";
	
	private static final int MENU_INFORMAR = Menu.FIRST;
	
	private static Oferta[] ofertasEnviadas;
	private static Oferta[] ofertasRecibidas;
	private static Oferta oferta = null;
	private String tipoOferta;
	static private ArrayList<String> listPropietario;
	static private ArrayList<String> listOfertante;
	static private ArrayList<String> listPrecio;
	static private ArrayList<String> listCalle;
	static private ArrayList<Integer> listVersion;
	static private ArrayList<Boolean> listRespondida;
	static private ArrayList<Boolean> listAceptada;
	EventsAdapter mAdapter;
	private ProgressDialog progressDialog = null;
	
	public static Oferta getOferta(){
		return OfertaListar.oferta;
	}
	
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		Bundle bundle;
		bundle = this.getIntent().getExtras();
    	tipoOferta = (String)bundle.getString("tipoOferta");
		oferta = null;
		ofertaListar();
	}
	
	@Override
	protected void onResume() {
		super.onResume();
		registerReceiver(receiver, new IntentFilter("CartaServicio.NUEVA"));
    	registerReceiver(receiver, new IntentFilter("CartaServicio.SIN"));
	}

	@Override
	protected void onPause() {
		super.onPause();
		unregisterReceiver(receiver);
		finish();
	}
	
	protected void onListItemClick(ListView l, View v, int position, long id) {
		if (tipoOferta.equals("enviada"))
			oferta = ofertasEnviadas[position];
		else
			oferta = ofertasRecibidas[position];
		
		Intent i = new Intent(OfertaListar.this, OfertaInfo.class);
		i.putExtra("tipoOferta", tipoOferta);
		i.putExtra("versionCalle", listVersion.get(position));
        this.startActivity(i);
        OfertaListar.this.setResult(RESULT_OK);
		OfertaListar.this.finish();
	}
	
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        super.onCreateOptionsMenu(menu); 
        menu.add(0, MENU_INFORMAR, 0 , getString(R.string.informacion))
        .setIcon(R.drawable.ic_menu_informacion);
        return true;
    }
    
    @Override
    public boolean onOptionsItemSelected (MenuItem item) {
    	switch (item.getItemId()) {
    		case MENU_INFORMAR:
    			showAlertInformacion();
    			break;
    	}
    	return super.onOptionsItemSelected(item);
    }
	
	private void showToastConexion(){
		Toast toast = Toast.makeText(getBaseContext(),
				getString(R.string.errConexion),
				Toast.LENGTH_SHORT);
		toast.show();
		OfertaListar.this.setResult(RESULT_OK);
		OfertaListar.this.finish();
	}
	
	private void showToastDatos(){
		Toast toast = Toast.makeText(getBaseContext(),
				getString(R.string.errConexion),
				Toast.LENGTH_SHORT);
		toast.show();
		OfertaListar.this.setResult(RESULT_OK);
		OfertaListar.this.finish();
	}

	private void showAlertListaVacia(){
		AlertDialog.Builder builder = new AlertDialog.Builder(OfertaListar.this);
		builder.setMessage(R.string.noOfertas)
	           .setCancelable(false)
	           .setPositiveButton(R.string.butContinuar, new DialogInterface.OnClickListener() {
	               public void onClick(DialogInterface dialog, int id) {
	            	   OfertaListar.this.setResult(RESULT_OK);
	            	   OfertaListar.this.finish();
	               }
	           });
	    AlertDialog alert = builder.create();
	    alert.setTitle(R.string.titLisOfertas);
	    alert.setIcon(R.drawable.ic_dialog_informacion);
		alert.show();
	}
	
	private void showAlertInformacion(){
    	AlertDialog.Builder builder = new AlertDialog.Builder(this);
    	builder.setMessage(getString(R.string.infoOfertaListar))
    	       .setCancelable(false)
    	       .setPositiveButton(getString(R.string.butContinuar), new DialogInterface.OnClickListener() {
    	           public void onClick(DialogInterface dialog, int id) {
    	        	   dialog.cancel();
    	           }
    	       });
    	AlertDialog alert = builder.create();
    	alert.setTitle(getString(R.string.informacion));
    	alert.setIcon(R.drawable.ic_dialog_informacion);
    	alert.show();
    }

	private void ofertaListar(){
        String title = getString(R.string.espere);
        String text = getString(R.string.obtenerOferta);
		
		progressDialog = ProgressDialog.show(this, title, text, true, false);
		progressDialog.setIcon(R.drawable.ic_dialog_conexion);
        Thread thread = new Thread(this);
        thread.start();
	}
	
	public void run() {
		Looper.prepare();
		conexionOfertaListar();
		Looper.myLooper().quit();
		Looper.loop();
	}
	
	public void conexionOfertaListar() {
		int resultado;
		try{
			DefaultHttpClient httpclient = new DefaultHttpClient();
			int idJuego = JuegoListar.getJuego().getIdentificador();
			int idEquipo = JuegoDatosListar.getEquipo().getIdentificador();
			URI uri = new URI(GoldenStreets.urlServer+"juego/"+idJuego+"/equipo/"+idEquipo+"/oferta/"+tipoOferta+"/listar/?format=json");
			HttpGet httpget = new HttpGet(uri);
			LibreGeoSocial LGS = LibreGeoSocial.getInstance();
			httpclient.setCookieStore(LGS.getCookieStore());
			HttpResponse response = httpclient.execute(httpget);
			HttpEntity entity = response.getEntity();
			String str = Utils.deStreamAString(entity.getContent());
			resultado = procesarRespuestaOfertaListar(str);
			
			handler.sendEmptyMessage(resultado);

		}catch(URISyntaxException urisex){
			Log.e(TAG, urisex.getMessage());
			handler.sendEmptyMessage(-1);
	    }catch (IOException e) {
	    	Log.e(TAG,e.getMessage());
	    	handler.sendEmptyMessage(-1);
	    }
	}
	
	private int procesarRespuestaOfertaListar (String str) {
		String ultMensaje = null;    	
		int ultCode = 0;
		
	    try {
	    	JSONObject json;
	    	json = new JSONObject(str.toString());
	
			if (json.has("code"))
				ultCode = json.getInt("code");
			else
				ultCode = -1;
			
			if (json.has("descripcion"))
				ultMensaje = json.getString("descripcion");
			else
				ultMensaje = null;
			
			listPropietario = new ArrayList<String>();
			listOfertante = new ArrayList<String>();
			listPrecio = new ArrayList<String>();
			listCalle = new ArrayList<String>();
			listVersion = new ArrayList<Integer>();
			listRespondida = new ArrayList<Boolean>();
			listAceptada = new ArrayList<Boolean>();
			int numeroOfertas = 0;
			
			if (json.has("ofertas") && ultMensaje.toLowerCase().equals("ok")){
				JSONArray ofertasObjects = json.getJSONArray("ofertas");
				numeroOfertas = ofertasObjects.length()-1;				
				if (tipoOferta.equals("enviada"))
					ofertasEnviadas = new Oferta[numeroOfertas];
				else
					ofertasRecibidas = new Oferta[numeroOfertas];
				int ofertaIdentificador;
				int ofertaPropietarioId;
				String ofertaPropietario;
				int ofertaOfertanteId;
				String ofertaOfertante;
				String ofertaCalle;
				int calleVersion;
				int ofertaCantidad;
				boolean ofertaRespondida; 
				boolean ofertaAceptada; 
	
				for (int i = 0; i < numeroOfertas; i++){
					JSONObject JSONOferta = (JSONObject)ofertasObjects.get(i);
					ofertaIdentificador = JSONOferta.getInt("identificador");
					ofertaPropietarioId = JSONOferta.getInt("propietarioId");
					ofertaPropietario = JSONOferta.getString("propietarioNombre");
					ofertaOfertanteId = JSONOferta.getInt("ofertanteId");
					ofertaOfertante = JSONOferta.getString("ofertanteNombre");
					ofertaCalle = JSONOferta.getString("calleNombre");
					calleVersion = JSONOferta.getInt("calleVersion");
					ofertaCantidad = JSONOferta.getInt("cantidad");
					ofertaRespondida = JSONOferta.getBoolean("respondida"); 
					ofertaAceptada = JSONOferta.getBoolean("aceptada"); 

					Oferta oferta = new Oferta(ofertaIdentificador, ofertaPropietarioId, ofertaPropietario, ofertaOfertanteId,
							ofertaOfertante, ofertaCalle, ofertaCantidad, ofertaRespondida, ofertaAceptada);
					if (tipoOferta.equals("enviada"))
						ofertasEnviadas[i] = oferta;
					else
						ofertasRecibidas[i] = oferta;
					
					listPropietario.add(getString(R.string.propietario).concat(ofertaPropietario));
					listOfertante.add(getString(R.string.ofertante).concat(ofertaOfertante));
					listPrecio.add(getString(R.string.precioOferta).concat(String.valueOf(ofertaCantidad)));
					listCalle.add(getString(R.string.propiedad).concat(ofertaCalle));
					listVersion.add(calleVersion);
					listRespondida.add(ofertaRespondida);
					listAceptada.add(ofertaAceptada);
				}
			}			
			if (ultCode != 200)
				return -2;
			if (numeroOfertas == 0)
				return 1;
	
		} catch (Exception e) {	
			ultCode = 500;
			ultMensaje = "Error in json parsed: " + e.getMessage();
			Log.e(TAG, ultMensaje);
			return -2;
		}
		return 0;
	}

	private Handler handler = new Handler() {
		@Override
		public void handleMessage(Message msg) {
			int result = msg.what;			
			progressDialog.cancel();			
			if (result == -2){        		
				showToastDatos();
        	}else if (result == -1){
        		showToastConexion();
        	}else if (result == 0){        		
				mAdapter = new EventsAdapter(OfertaListar.this);
				setListAdapter(mAdapter);
        	}else if (result == 1){
        		showAlertListaVacia();        		
        	}
		}
	};
	
    public static class EventsAdapter extends BaseAdapter {

		private Context mContext;
		
		public EventsAdapter (Context c) {
            mContext = c;
        }

		public int getCount() {
			// TODO Auto-generated method stub
			return listPropietario.size();
		}

		public Object getItem(int position) {
			// TODO Auto-generated method stub
			return position;
		}

		public long getItemId(int position) {
			// TODO Auto-generated method stub
			return position;
		}

		public View getView(int position, View convertView, ViewGroup parent) {
			// TODO Auto-generated method stub
			View view;
			
	        if (convertView == null) {
	            LayoutInflater inflater = (LayoutInflater) mContext
	                    .getSystemService(Context.LAYOUT_INFLATER_SERVICE);
	            view = inflater.inflate(R.layout.ofertalistar, null);
	        } else {
	            view = convertView;
	        }
	        
	        ImageView imageRespuesta = (ImageView) view.findViewById(R.id.imageRespuesta);
			if(listRespondida.get(position))
				if(listAceptada.get(position))
					imageRespuesta.setImageResource(R.drawable.ic_list_oferta_aceptada);
				else
					imageRespuesta.setImageResource(R.drawable.ic_list_oferta_rechazada);
			else
				imageRespuesta.setImageResource(R.drawable.ic_list_oferta_sinrespuesta);
	        
	        TextView textPropietario = (TextView) view.findViewById(R.id.textPropietario);
	        textPropietario.setText((String)listPropietario.get(position));
			
			TextView textOfertante = (TextView) view.findViewById(R.id.textOfertante);
			textOfertante.setText((String)listOfertante.get(position));
			
			TextView textPrecioOferta = (TextView) view.findViewById(R.id.textPrecioOferta);
			textPrecioOferta.setText((String)listPrecio.get(position));
			
			TextView textCalle = (TextView) view.findViewById(R.id.textCalle);
			textCalle.setText((String)listCalle.get(position));
				        
			return view;       
		}
	}
    
    private BroadcastReceiver receiver = new BroadcastReceiver() {        
        @Override
        public void onReceive(Context context, Intent intent) {
        	String accion = intent.getAction();
        	if (accion.equals("CartaServicio.NUEVA")) {
        		AlertDialog alert = AccionesRecibidasBroadcast.accionCartaServicioNueva(context, intent);
		    	alert.show();
        	} else if(accion.equals("CartaServicio.SIN")) {
        		AlertDialog alert = AccionesRecibidasBroadcast.accionCartaServicioSin(context, intent);
		    	alert.show();
        	}
        }
    };
}