package com.Goldenstreet.App.GoldenStreet;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.util.Log;
import android.view.Gravity;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

import com.Goldenstreet.App.R;
import com.LibreGeoSocial.Api.LibreGeoSocial;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.List;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.NameValuePair;
import org.apache.http.protocol.HTTP;
import org.json.JSONException;
import org.json.JSONObject;

/**
 * Muestra un formulario con la informacin de la propiedad(calle) y dinero del equipo para la
 * creacin y envo de una nueva oferta.
 * 
 * @author Rubn Quero Castrillo <rubenquero@gmail.com>
 * 
 */

public class OfertaNuevo extends Activity implements OnClickListener {
	static private String TAG = "OfertaNuevo";
	
	private LinearLayout linearLayout;
	private TextView textCalle;
	private TextView textDineroDisponible;
	private EditText editPrecioOferta;
	private Button buttonEnviar;
	private ProgressDialog progressDialog = null;
	private int calleIdentificador;
	private String calleNombre;
	private int callePrecio;
	private boolean estaConstruido;
	private boolean estaSaboteado;
	private int calleVerion;
	private String edificioNombre;
	private int propietarioIdentificador;
	private String propietarioNombre;
	private int ofertanteDinero;
	private int precioOferta;
	
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.ofertanuevo);
        
        Bundle bundle;
        bundle = getIntent().getExtras();
        calleIdentificador = bundle.getInt("calleIdentificador");

        linearLayout = (LinearLayout)this.findViewById(R.id.linearLayout);
        textCalle = (TextView)this.findViewById(R.id.textCalle);
        textDineroDisponible = (TextView)this.findViewById(R.id.textDineroDisponible);
        editPrecioOferta = (EditText)this.findViewById(R.id.editPrecioOferta);
        
        getWindow().setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_STATE_HIDDEN);
        linearLayout.setVisibility(View.GONE);
        
        buttonEnviar = (Button)this.findViewById(R.id.buttonEnviar);
        buttonEnviar.setOnClickListener(this);
    }
    
    @Override
	protected void onResume() {
		super.onResume();
		registerReceiver(receiver, new IntentFilter("CartaServicio.NUEVA"));
    	registerReceiver(receiver, new IntentFilter("CartaServicio.SIN"));
    	String title = getString(R.string.espere);
	    String text = getString(R.string.obtenerDatos);
			
	    progressDialog = ProgressDialog.show(this, title, text, true, false);
	    progressDialog.setIcon(R.drawable.ic_dialog_conexion);
	    
    	Thread thread = new Thread(obtenerDatos);
	    thread.start();
	}

	@Override
	protected void onPause() {
		super.onPause();
		unregisterReceiver(receiver);
		finish();
	}
    
	public void onClick(View v) {
		if (buttonEnviar.isPressed()){
			if (editPrecioOferta.getText().toString().trim().equals("")){
	    	    Context context = getApplicationContext();
			    Toast toast = Toast.makeText(context, getString(R.string.mensajeVacio), Toast.LENGTH_SHORT);
			    toast.setGravity(Gravity.CENTER_VERTICAL, 0, 0);
			    toast.show();
	    	}
			else{
				precioOferta = Integer.valueOf(editPrecioOferta.getText().toString());
				if (ofertanteDinero < precioOferta){
					Context context = getApplicationContext();
				    Toast toast = Toast.makeText(context, getString(R.string.calleOfertaSinDinero), Toast.LENGTH_SHORT);
				    toast.setGravity(Gravity.CENTER_VERTICAL, 0, 0);
				    toast.show();
				}else{
					String title = getString(R.string.espere);
				    String text = getString(R.string.ofertaCalle);
						
				    progressDialog = ProgressDialog.show(this, title, text, true, false);
				    progressDialog.setIcon(R.drawable.ic_dialog_conexion);
					Thread thread = new Thread(enviarOferta);
				    thread.start();
				}
			}
		}
	}
	
	private void showToastConexion(){			
		Toast toast = Toast.makeText(this,
				getString(R.string.errConexion),
				Toast.LENGTH_SHORT);
		toast.show();
		OfertaNuevo.this.setResult(RESULT_OK);
		OfertaNuevo.this.finish();
	}
	
	private void showToastAceptada(){
		Toast toast = Toast.makeText(this,
				getString(R.string.calleOfertaAceptada),
				Toast.LENGTH_LONG);
		toast.show();
		OfertaNuevo.this.setResult(RESULT_OK);
		OfertaNuevo.this.finish();
	}
	
	private void showToastRechazada(){
		Toast toast = Toast.makeText(this,
				getString(R.string.calleOfertaRechazada),
				Toast.LENGTH_LONG);
		toast.show();
		OfertaNuevo.this.setResult(RESULT_OK);
		OfertaNuevo.this.finish();
	}
	
	private Runnable obtenerDatos = new Runnable(){
		@Override
		public void run(){
			Looper.prepare();
			conexionObtenerDatos();
			Looper.myLooper().quit();
			Looper.loop();
		}
		
		public void conexionObtenerDatos(){
			int resultado;
			try{
				DefaultHttpClient httpclient = new DefaultHttpClient();
				int idJuego = JuegoListar.getJuego().getIdentificador();
				int idEquipo = JuegoDatosListar.getEquipo().getIdentificador();

				URI uri = new URI(GoldenStreets.urlServer+"juego/"+idJuego+"/equipo/"+idEquipo+"/calle/"+calleIdentificador+"/ofertar/?format=json");
				HttpGet httpget = new HttpGet(uri);
				LibreGeoSocial LGS = LibreGeoSocial.getInstance();
				httpclient.setCookieStore(LGS.getCookieStore());
				HttpResponse response = httpclient.execute(httpget);
				HttpEntity entity = response.getEntity();
				String str = Utils.deStreamAString(entity.getContent());
				resultado = procesarRespuestaObtenerDatos(str);
				handler.sendEmptyMessage(resultado);  		

			}catch(URISyntaxException urisex){
				Log.e(TAG, urisex.getMessage());
				handler.sendEmptyMessage(1);
		    }catch (IOException e) {
		    	Log.e(TAG,e.getMessage());
		    	handler.sendEmptyMessage(1);
		    }
		}
		
		private int procesarRespuestaObtenerDatos(String str) {
			String ultMensaje = null;
			int ultCode = 0;
			
		    try {
		    	JSONObject json;
		    	json = new JSONObject(str.toString());
		
				if (json.has("code"))
					ultCode = json.getInt("code");
				else
					ultCode = -1;
				
				if (json.has("descripcion"))
					ultMensaje = json.getString("descripcion");
				else
					ultMensaje = null;
							
				if (json.has("calle") && ultMensaje.toLowerCase().equals("ok")){
					JSONObject CalleObject = json.getJSONObject("calle");
					calleNombre = CalleObject.getString("nombre");
					callePrecio = CalleObject.getInt("precio");
					estaConstruido = CalleObject.getBoolean("estaConstruido");
					estaSaboteado = CalleObject.getBoolean("estaSaboteado");
					calleVerion = CalleObject.getInt("version");
					edificioNombre = CalleObject.getString("nombreEdificio");
					propietarioIdentificador = CalleObject.getInt("idPropietario");
					propietarioNombre = CalleObject.getString("nombrePropietario");
					ofertanteDinero = CalleObject.getInt("dineroOfertante");
				}
				if (ultCode != 200)
					return 1;
		
			} catch (Exception e) {	
				ultCode = 500;
				ultMensaje = "Error in json parsed: " + e.getMessage();
				Log.e(TAG, ultMensaje);
				return 1;
			}
			return 0;
		}
		
	    private Handler handler = new Handler()
	    {
	    	@Override
	        public void handleMessage(Message msg)
	        {
	        	int result = msg.what;
	        	progressDialog.cancel();
	        	if(result == 0){
	        		textCalle.setText(String.format(getString(R.string.infoCalle), calleNombre, callePrecio, propietarioNombre,
		        			edificioNombre, estaConstruido, estaSaboteado));
	        		textDineroDisponible.setText(getString(R.string.dineroDisponible).concat(ofertanteDinero+""));
	        		linearLayout.setVisibility(View.VISIBLE);
	        	}else if(result == 1){
	        		showToastConexion();
	        	}
	        }
	    };
	};
	
	private Runnable enviarOferta = new Runnable(){
		@Override
		public void run(){
			Looper.prepare();
			conexionEnviarOferta();
			Looper.loop();
		}
		
		public void conexionEnviarOferta(){
			int resultado;
			try{
				DefaultHttpClient httpclient = new DefaultHttpClient();
				int idJuego = JuegoListar.getJuego().getIdentificador();
				int idEquipo = JuegoDatosListar.getEquipo().getIdentificador();
				
				URI uri = new URI(GoldenStreets.urlServer+"juego/"+idJuego+"/equipo/"+idEquipo+"/calle/"+calleIdentificador+"/ofertar/?format=json");
				HttpPost httpost = new HttpPost(uri);
				LibreGeoSocial LGS = LibreGeoSocial.getInstance();
				httpclient.setCookieStore(LGS.getCookieStore());
				
				List <NameValuePair> datos = new ArrayList <NameValuePair>();
	            datos.add(new BasicNameValuePair("propietarioId", String.valueOf(propietarioIdentificador)));
	            datos.add(new BasicNameValuePair("ofertaCantidad", String.valueOf(precioOferta)));
	            datos.add(new BasicNameValuePair("calleVersion", String.valueOf(calleVerion)));
	            httpost.setEntity(new UrlEncodedFormEntity(datos, HTTP.UTF_8));
	            
	            HttpResponse response = httpclient.execute(httpost);
	            HttpEntity entity = response.getEntity();
				String str = Utils.deStreamAString(entity.getContent());
				resultado = procesarRespuestaEnviarOferta(str);
				handler.sendEmptyMessage(resultado);  		

			}catch(URISyntaxException urisex){
				Log.e(TAG, urisex.getMessage());
				handler.sendEmptyMessage(2);
		    }catch (IOException e) {
		    	Log.e(TAG,e.getMessage());
		    	handler.sendEmptyMessage(2);
		    }
		}
		
		private int procesarRespuestaEnviarOferta(String str) {
			String ultMensaje = null;
			int ultCode = 0;
			
			try {
	        	JSONObject json;
				json = new JSONObject(str);
				
				if (json.has("code"))
					ultCode = json.getInt("code");
				else
					ultCode = -1;
				
				if (json.has("descripcion"))
					ultMensaje = json.getString("descripcion");
				else
					ultMensaje = null;
				
				if (ultCode != 200){
					if (ultCode == 400 && ultMensaje.toLowerCase().compareTo("calle con modificaciones") == 0)
						return 1;
					return 2;
				}
			} catch (JSONException e) {	
				ultCode = 500;
				ultMensaje = "Error in json parsed: " + e.getMessage();
				Log.e(TAG, ultMensaje);
				return 2;
			}
			return 0;
		}
		
	    private Handler handler = new Handler()
	    {
	    	@Override
	    	public void handleMessage(Message msg)
	        {
	        	int result = msg.what;
	        	progressDialog.cancel();
	        	if(result == 0){
	        		showToastAceptada();
	        	}else if(result == 1){
	        		showToastRechazada();
	        	}else if(result == 2){
	        		showToastConexion();
	        	}	        	 
	        }
	    };
	};
	
    private BroadcastReceiver receiver = new BroadcastReceiver() {        
        @Override
        public void onReceive(Context context, Intent intent) {
        	String accion = intent.getAction();
        	if (accion.equals("CartaServicio.NUEVA")) {
        		AlertDialog alert = AccionesRecibidasBroadcast.accionCartaServicioNueva(context, intent);
		    	alert.show();
        	} else if(accion.equals("CartaServicio.SIN")) {
        		AlertDialog alert = AccionesRecibidasBroadcast.accionCartaServicioSin(context, intent);
		    	alert.show();
        	}
        }
    };
}