package com.Goldenstreet.App.GoldenStreet;

import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.util.Log;
import android.widget.Toast;

import com.Goldenstreet.App.R;
import com.LibreGeoSocial.Api.LibreGeoSocial;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.Timer;
import java.util.TimerTask;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.json.JSONException;
import org.json.JSONObject;

/**
 * Servicio que peridicamente consultar si el equipo ha recibido una respuesta a sus ofertas
 * enviadas o ha recibido una nueva oferta por alguna de sus propiedades, y avisar a travs de
 * una notificacin al equipo.
 * 
 * @author Rubn Quero Castrillo <rubenquero@gmail.com>
 * 
 */

public class OfertaServicio extends Service{
	static private String TAG = "OfertaServicio";
	
	private static final int RETARDO = 5*1000; 
	private static final int PERIODO = 30*1000;
	private static final int OFERTA_RECIBIDA_ID = 1;
	private static final int OFERTA_RESPONDIDA_ID = 2;
	private int ofertaRespondida;
	private int ofertaRecibida;
	private int ultOfertaRespondida;
	private int ultOfertaRecibida;
	private NotificationManager notificationManager;
	private Timer timer;	

	@Override
	public void onCreate() {
		super.onCreate();
		ofertaRespondida = 0;
		ofertaRecibida = 0;
		ultOfertaRespondida = 0;
		ultOfertaRecibida = 0;
		notificationManager = (NotificationManager)getSystemService(NOTIFICATION_SERVICE);
		timer = new Timer("ofertaServicio");
	}
	
	@Override
	public void onDestroy() {
		super.onDestroy();
		timer.cancel();
		notificationManager.cancelAll();
	}
	
 	@Override
 	public void onStart(final Intent intent, final int startId){
 		super.onStart(intent, startId);
 		timer.scheduleAtFixedRate(timerTask, RETARDO, PERIODO);
 	}
 	
	@Override
	public IBinder onBind(Intent arg0) {
		// TODO Auto-generated method stub
		return null;
	}
 	
	private void showToastConexion(){			
		Toast toast = Toast.makeText(getBaseContext(),
				getBaseContext().getString(R.string.errConexion),
				Toast.LENGTH_SHORT);
		toast.show();
	}
	
	private void showToastDatos(){
		Toast toast = Toast.makeText(getBaseContext(),
				getString(R.string.errDatos),
				Toast.LENGTH_SHORT);
		toast.show();
	}

	private TimerTask timerTask = new TimerTask() {
		@Override
		public void run() {
			ofertaHistorico();
		}
	};
	
	private void ofertaHistorico(){
		int resultado; 
		try{
			DefaultHttpClient httpclient = new DefaultHttpClient();
			int idJuego = JuegoListar.getJuego().getIdentificador();
			int idEquipo = JuegoDatosListar.getEquipo().getIdentificador();
			URI uri = new URI(GoldenStreets.urlServer+"juego/"+idJuego+"/equipo/"+idEquipo+"/oferta/historico/?format=json");
			HttpGet httpget = new HttpGet(uri);
			LibreGeoSocial LGS = LibreGeoSocial.getInstance();
			httpclient.setCookieStore(LGS.getCookieStore());
			HttpResponse response = httpclient.execute(httpget);
			HttpEntity entity = response.getEntity();
			String str = Utils.deStreamAString(entity.getContent());
			resultado = procesarRespuestaOfertaHistorico(str);
			
    		handler.sendEmptyMessage(resultado);

		}catch(URISyntaxException urisex){
			Log.e(TAG, urisex.getMessage());
			handler.sendEmptyMessage(-1);
	    }catch (IOException e) {
	    	Log.e(TAG,e.getMessage());
	    	handler.sendEmptyMessage(-1);
	    }
	}
	
	private int procesarRespuestaOfertaHistorico (String str) {
		int ultCode;
    	String ultMensaje;
        try {
        	JSONObject json;
			json = new JSONObject(str);
			
			if (json.has("code"))
				ultCode = json.getInt("code");
			else
				ultCode = -1;
			
			if (json.has("descripcion"))
				ultMensaje = json.getString("descripcion");
			else
				ultMensaje = null;
			 
			if (json.has("ofertasRespondidas"))
				ultOfertaRespondida = json.getInt("ofertasRespondidas");
			else
				ultOfertaRespondida = 0;
				
			if (json.has("ofertasRecibidas"))
				ultOfertaRecibida = json.getInt("ofertasRecibidas");
			else
				ultOfertaRecibida = 0;
			
			if (ultCode != 200){
				return -2;
			}
			
		} catch (JSONException e) {	
			ultCode = 500;
			ultMensaje = "Error in json parsed: " + e.getMessage();
			Log.e(TAG, ultMensaje);
			return -2;
		}
		return 0;
	}
	
	private void ofertaRespondidaNotification()
    {
		CharSequence title = getString(R.string.notTituloOfertaRespondida);
        CharSequence message = getString(R.string.notMensajeOfertaRespondida);
        CharSequence tickerText = getString(R.string.notTextoOfertaRespondida);
        int icon = R.drawable.ic_stat_notify_oferta_respondida;
        long when = System.currentTimeMillis();
        
        Notification notification = new Notification(icon, tickerText, when);
        notification.defaults |= Notification.DEFAULT_SOUND;
        notification.defaults |= Notification.DEFAULT_VIBRATE;
        notification.defaults |= Notification.DEFAULT_LIGHTS;  
        notification.flags |= Notification.FLAG_SHOW_LIGHTS;
        notification.flags |= Notification.FLAG_AUTO_CANCEL;

        Intent notificationIntent = new Intent(OfertaServicio.this, OfertaListar.class);
        notificationIntent.setAction(String.valueOf(when));
        Bundle bundle = new Bundle();
        bundle.putString("tipoOferta", "enviada");
        notificationIntent.putExtras(bundle);
        PendingIntent pendingIntent = PendingIntent.getActivity(this, 0, notificationIntent, PendingIntent.FLAG_UPDATE_CURRENT);
        notification.setLatestEventInfo(OfertaServicio.this, title, message, pendingIntent);
       
        notificationManager.notify(OFERTA_RESPONDIDA_ID, notification);
    }	
	
	private void ofertaRecibidaNotification()
    {
        CharSequence title = getString(R.string.notTituloOfertaRecibida);
        CharSequence message = getString(R.string.notMensajeOfertaRecibida);
        CharSequence tickerText = getString(R.string.notTextoOfertaRecibida);
        int icon = R.drawable.ic_stat_notify_oferta_recibida;
        long when = System.currentTimeMillis();
        
        Notification notification = new Notification(icon, tickerText, when);
        notification.defaults |= Notification.DEFAULT_SOUND;
        notification.defaults |= Notification.DEFAULT_VIBRATE;
        notification.defaults |= Notification.DEFAULT_LIGHTS;  
        notification.flags |= Notification.FLAG_SHOW_LIGHTS;
        notification.flags |= Notification.FLAG_AUTO_CANCEL;
 
        Intent notificationIntent = new Intent(OfertaServicio.this, OfertaListar.class);
        notificationIntent.setAction(String.valueOf(when));
        Bundle bundle = new Bundle();
        bundle.putString("tipoOferta", "recibida");
        notificationIntent.putExtras(bundle);
        PendingIntent pendingIntent = PendingIntent.getActivity(this, 0, notificationIntent, PendingIntent.FLAG_UPDATE_CURRENT);
        notification.setLatestEventInfo(OfertaServicio.this, title, message, pendingIntent);

        notificationManager.notify(OFERTA_RECIBIDA_ID, notification);
    }
		
    private Handler handler = new Handler() {
    	@Override
		public void handleMessage(Message msg) {
			int result = msg.what;	
			if (result == -2){        		
				showToastDatos();
        	}else if (result == -1){
        		showToastConexion();
        	}else if (result == 0){        		
        		if(ultOfertaRecibida > ofertaRecibida){
					ofertaRecibida = ultOfertaRecibida;
					ofertaRecibidaNotification();					
				}
				if(ultOfertaRespondida > ofertaRespondida){
					ofertaRespondida = ultOfertaRespondida;
					ofertaRespondidaNotification();
				}
        	}
		}
	};
}