package com.Goldenstreet.App.GoldenStreet;

import java.util.Timer;
import java.util.TimerTask;

import com.Goldenstreet.App.R;

import android.app.TabActivity;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.widget.TabHost;
import android.widget.TextView;

/**
 * Muestra al usuario diferentes tabs (pestaas) con las que el usuario podr realizar diferentes
 * acciones (informacin sobre los equipos participantes, sistema de cartas, mapa de localizacin
 * de los diferentes recursos de la partida, sistema de ofertas, sistema de mensajera).
 * Tambin se iniciar el tiempo del equipo que esta jugando, para informarle del fin de la partida,
 * cuando este llegue a su lmite.
 * 
 * @author Rubn Quero Castrillo <rubenquero@gmail.com>
 * 
 */

public class Partida extends TabActivity{
	//static private String TAG = "Partida";
	
    private static final int RETARDO = 0;
	private static final int PERIODO = 60*1000;
	
	public static boolean finPartida = false;
	private TextView textTiempo;
    private int minutosJuego;
    private int duracionJuego;
    private Timer timer;
    
	public static boolean getFinPartida() {
		return finPartida;
	}

	@Override
    protected void onCreate(Bundle savedInstanceState) {     
		super.onCreate(savedInstanceState);		
			
		final TabHost tabHost = getTabHost();
		
		tabHost.addTab(tabHost.newTabSpec(getString(R.string.estadisticas))
                .setIndicator(getString(R.string.estadisticas), this.getResources().getDrawable(R.drawable.ic_tab_estadisticas))
                .setContent(new Intent(this, Estadisticas.class)));
		
		tabHost.addTab(tabHost.newTabSpec(getString(R.string.cartas))				
				.setIndicator(getString(R.string.cartas), this.getResources().getDrawable(R.drawable.ic_tab_cartas))
                .setContent(new Intent(this, CartaListar.class)));
		
		tabHost.addTab(tabHost.newTabSpec(getString(R.string.mapa))				
				.setIndicator(getString(R.string.mapa), this.getResources().getDrawable(R.drawable.ic_tab_mapas))
                .setContent(new Intent(this, PosicionamientoMapa.class)));
		
		tabHost.addTab(tabHost.newTabSpec(getString(R.string.ofertas))				
				.setIndicator(getString(R.string.ofertas), this.getResources().getDrawable(R.drawable.ic_tab_ofertas))
                .setContent(new Intent(this, OfertaMenu.class)));
		
		tabHost.addTab(tabHost.newTabSpec(getString(R.string.mensajes))				
				.setIndicator(getString(R.string.mensajes), this.getResources().getDrawable(R.drawable.ic_tab_mensajes))
                .setContent(new Intent(this, MensajeMenu.class)));
		
		tabHost.computeScroll();
		
		finPartida = false;
		minutosJuego = 0;
		duracionJuego = JuegoListar.getJuego().getDuracion();
		textTiempo = Estadisticas.getTextTiempo();
		
		launchTask();
	}
	
	public void onDestroy(){
    	super.onDestroy();
    	timer.cancel();
    	finPartida = true;
	}
	
    private Handler handler = new Handler()
    {
        public void handleMessage(Message msg)
        {
        	int result = msg.what;
        	if (result == 0){        		
        		textTiempo.setText(String.format(getString(R.string.tiempoJuego), minutosJuego, duracionJuego));
        		minutosJuego++;
        	}else if (result == 1){
        		finPartida = true;        		
        		Utils.cerrarPartida(Partida.this);        		
        		Intent intent = new Intent(Partida.this, Estadisticas.class);
         	   	Partida.this.startActivity(intent);
        	}	
        }
    };
    
    private void launchTask(){		
		timer = new Timer();
		timer.scheduleAtFixedRate(new TimerTask() {
			@Override
			public void run() {
				if(minutosJuego < duracionJuego){		        		
					handler.sendEmptyMessage(0);
				}else{
		        	handler.sendEmptyMessage(1);
				}
			}
		}
		,RETARDO, PERIODO);
	}
}