package com.Goldenstreet.App.GoldenStreet;

import android.app.Service;
import android.content.Intent;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.util.Log;
import android.widget.Toast;

import com.Goldenstreet.App.R;
import com.LibreGeoSocial.Api.LibreGeoSocial;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.Timer;
import java.util.TimerTask;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.json.JSONException;
import org.json.JSONObject;

/**
 * Servicio que peridicamente consultar la propiedades (calles) del equipo, para ingresar/retirar
 * los beneficios/perdidas que han producido.
 * 
 * @author Rubn Quero Castrillo <rubenquero@gmail.com>
 * 
 */

public class PatrimonioServicio extends Service{
	static private String TAG = "PatrimonioServicio";
	
	private static final int RETARDO = 120*1000; 
	private static final int PERIODO = 120*1000;
	private Timer timer;	
	
	@Override
	public void onCreate() {
		super.onCreate();
		timer = new Timer("patrimonioServicio");
	}
	
	@Override
	public void onDestroy() {
		super.onDestroy();
		timer.cancel();
	}
	
 	@Override
 	public void onStart(final Intent intent, final int startId){
 		super.onStart(intent, startId);
 		timer.scheduleAtFixedRate(timerTask, RETARDO, PERIODO);
 	}
 	
	@Override
	public IBinder onBind(Intent arg0) {
		// TODO Auto-generated method stub
		return null;
	}

	private TimerTask timerTask = new TimerTask() {
		@Override
		public void run() {
			patrimonioModificar();
		}
	};
	
	private void showToastConexion(){			
		Toast toast = Toast.makeText(getBaseContext(),
				getBaseContext().getString(R.string.errConexion),
				Toast.LENGTH_SHORT);
		toast.show();
	}
	
	private void showToastDatos(){
		Toast toast = Toast.makeText(getBaseContext(),
				getString(R.string.errDatos),
				Toast.LENGTH_SHORT);
		toast.show();
	}
	
	private void showToastAceptada(){
		Toast toast = Toast.makeText(getBaseContext(),
				getBaseContext().getString(R.string.patrimonioModificadoAceptada),
				Toast.LENGTH_LONG);
		toast.show();
	}
	
	private void patrimonioModificar(){
		int resultado; 
		try{
			DefaultHttpClient httpclient = new DefaultHttpClient();
			int idJuego = JuegoListar.getJuego().getIdentificador();
			int idEquipo = JuegoDatosListar.getEquipo().getIdentificador();
			URI uri = new URI(GoldenStreets.urlServer+"juego/"+idJuego+"/equipo/"+idEquipo+"/propiedades/patrimonio/modificar/?format=json");
			HttpGet httpget = new HttpGet(uri);
			LibreGeoSocial LGS = LibreGeoSocial.getInstance();
			httpclient.setCookieStore(LGS.getCookieStore());
			HttpResponse response = httpclient.execute(httpget);
			HttpEntity entity = response.getEntity();
			String str = Utils.deStreamAString(entity.getContent());
			resultado = procesarRespuestaPatrimonioModificar(str);
			
			handler.sendEmptyMessage(resultado);

		}catch(URISyntaxException urisex){
			Log.e(TAG, urisex.getMessage());
			handler.sendEmptyMessage(-1);
	    }catch (IOException e) {
	    	Log.e(TAG,e.getMessage());
	    	handler.sendEmptyMessage(-1);
	    }
	}
	
	private int procesarRespuestaPatrimonioModificar (String str) {
		int ultCode;
    	String ultMensaje;
        try {
        	JSONObject json;
			json = new JSONObject(str);
			
			if (json.has("code"))
				ultCode = json.getInt("code");
			else
				ultCode = -1;
			
			if (json.has("descripcion"))
				ultMensaje = json.getString("descripcion");
			else
				ultMensaje = null;
			 
			if (ultCode != 200){
				return -2;
			}
			
		} catch (JSONException e) {	
			ultCode = 500;
			ultMensaje = "Error in json parsed: " + e.getMessage();
			Log.e(TAG, ultMensaje);
			return -2;
		}
		return 0;
	}
	
    private Handler handler = new Handler() {
    	@Override
		public void handleMessage(Message msg) {
			int result = msg.what;	
			if (result == -2){        		
				showToastDatos();
        	}else if (result == -1){
        		showToastConexion();
        	}else if (result == 0){        		
        		showToastAceptada();
        	}
		}
	};
}