package com.Goldenstreet.App.GoldenStreet;

import android.app.AlertDialog;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.graphics.drawable.Drawable;
import android.location.Location;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.ImageView;
import android.widget.Toast;

import com.Goldenstreet.App.R;
import com.LibreGeoSocial.Api.LibreGeoSocial;
import com.google.android.maps.GeoPoint;
import com.google.android.maps.MapActivity;
import com.google.android.maps.MapController;
import com.google.android.maps.MapView;
import com.google.android.maps.Overlay;
import com.google.android.maps.OverlayItem;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.json.JSONArray;
import org.json.JSONObject;

/**
 * Muestra un mapa con la posicin de todos los recursos que forman la partida (equipos participantes
 * y calles), cada recurso queda identificado por diferentes iconos. Dependiendo del estado de las
 * calles, del propietario y si estn dentro del rea de accin del equipo podr realizar diferentes
 * acciones con el men (comprar, construir, ofertar, sabotear y demoler).
 * 
 * @author Rubn Quero Castrillo <rubenquero@gmail.com>
 * 
 */

public class PosicionamientoMapa extends MapActivity implements OnClickListener{
	static private String TAG = "PosicionamientoMapa";
	
	private static final int RADIO_ACCION = 65;
	private static final int RETARDO = 0;
	private static final int PERIODO = 5*1000;
	
	private static final int MENU_COMPRAR = Menu.FIRST;
	private static final int MENU_VENDER = Menu.FIRST+1;
	private static final int MENU_CONSTRUIR = Menu.FIRST+2;
	private static final int MENU_OFERTAR = Menu.FIRST+3;
	private static final int MENU_SABOTEAR = Menu.FIRST+4;
	private static final int MENU_DEMOLER = Menu.FIRST+5;
	private static final int MENU_INFORMAR = Menu.FIRST+6;
	
	private Calle calleMenuObjetivo = null;
	private ImageView imageMiPosicion;
	private MapView mapView;
	private MapController mapControl;
	private DatosMapaItemizedOverlay datosMapaItemizedOverlay;
	private List<Overlay> mapOverlays;
	private Timer timer;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.localizacion);
        
        imageMiPosicion = (ImageView)this.findViewById(R.id.imageMiPosicion);
        imageMiPosicion.setOnClickListener(this);
        
        mapView = (MapView) findViewById(R.id.mapView);
        mapView.setSatellite(false);
        mapView.setBuiltInZoomControls(true);
        
        mapControl = mapView.getController();
        mapControl.setZoom(18);
       
        mapOverlays = mapView.getOverlays();
        centrarEnMiEquipo();
	}
	
	@Override
	protected void onResume() {
		super.onResume();
		registerReceiver(receiver, new IntentFilter("CartaServicio.NUEVA"));
    	registerReceiver(receiver, new IntentFilter("CartaServicio.SIN"));
		this.launchTask();
	}

	@Override
	protected void onPause() {
		super.onPause();
		unregisterReceiver(receiver);
		timer.cancel();
		timer.purge();
	}	
	
    public void onClick(View v) {
    	if ( imageMiPosicion.isPressed() ){
    		centrarEnMiEquipo();
        }
    }
	
	@Override
	public boolean onOptionsItemSelected (MenuItem item) {
		switch (item.getItemId()) {
		case MENU_COMPRAR:
			AccionMenuComprar.comprarCalleMapa(PosicionamientoMapa.this, calleMenuObjetivo);
			break;
		case MENU_VENDER:
			AccionMenuVender.venderCalleMapa(PosicionamientoMapa.this, calleMenuObjetivo);
			break;
		case MENU_CONSTRUIR:
			AccionMenuConstruir.construirCalleMapa(PosicionamientoMapa.this, calleMenuObjetivo);
			break;
		case MENU_OFERTAR:
			Intent i = new Intent(PosicionamientoMapa.this, OfertaNuevo.class);
    		i.putExtra("calleIdentificador", calleMenuObjetivo.getId());
            this.startActivity(i);
			break;
		case MENU_SABOTEAR:
			AccionMenuSabotear.sabotearCalleMapa(PosicionamientoMapa.this, calleMenuObjetivo);
			break;
		case MENU_DEMOLER:
			AccionMenuDemoler.demolerCalleMapa(PosicionamientoMapa.this, calleMenuObjetivo);
			break;
		case MENU_INFORMAR:
			showAlertInformacion();
			break;
		}
		return super.onOptionsItemSelected(item);
	}
	
	@Override  	
	public boolean onPrepareOptionsMenu(Menu menu) {
		menu.clear();
		if (calleMenuObjetivo != null){
			if(calleMenuObjetivo.getIdPropietario() < 0){
				menu.add(0, MENU_COMPRAR, 0 , getString(R.string.comprar))
			       .setIcon(R.drawable.ic_menu_comprar);
			}else{
				if(calleMenuObjetivo.getIdPropietario() == JuegoDatosListar.getEquipo().getIdentificador()){
					if(calleMenuObjetivo.getEstaConstruido()){
						if(calleMenuObjetivo.getEstaSaboteado())							
							menu.add(0, MENU_DEMOLER, 0 , getString(R.string.demoler))
						       .setIcon(R.drawable.ic_menu_demoler);
					}else
						menu.add(0, MENU_CONSTRUIR, 0 , getString(R.string.construir))
					       .setIcon(R.drawable.ic_menu_construir);
					menu.add(0, MENU_VENDER, 0 , getString(R.string.vender))
				       .setIcon(R.drawable.ic_menu_vender);
				}else{
					menu.add(0, MENU_OFERTAR, 0 , getString(R.string.ofertar))
				       .setIcon(R.drawable.ic_menu_comprar);
					if(calleMenuObjetivo.getEstaConstruido()){
						if(!calleMenuObjetivo.getEstaSaboteado()){
							menu.add(0, MENU_SABOTEAR, 0 , getString(R.string.sabotear))
						       .setIcon(R.drawable.ic_menu_sabotear);
							menu.add(0, MENU_DEMOLER, 0 , getString(R.string.demoler))
						       .setIcon(R.drawable.ic_menu_demoler);
						}
					}
				}
			}
		}
		menu.add(0, MENU_INFORMAR, 0 , getString(R.string.informacion))
	       .setIcon(R.drawable.ic_menu_informacion);
	    
	    return super.onPrepareOptionsMenu(menu);
	}

	@Override
	public boolean onKeyDown(int keyCode, KeyEvent event){
		if (keyCode == KeyEvent.KEYCODE_BACK) {
			AlertDialog.Builder builder = new AlertDialog.Builder(this);
			builder.setMessage(getString(R.string.salirJuego))
		           .setCancelable(false)
		           .setPositiveButton(getString(R.string.butSi), new DialogInterface.OnClickListener() {
		               public void onClick(DialogInterface dialog, int id) {
		            	   Utils.cerrarPartida(PosicionamientoMapa.this);	
		               }
		           })
		           .setNegativeButton(getString(R.string.butNo), new DialogInterface.OnClickListener() {
		               public void onClick(DialogInterface dialog, int id) {
		                    dialog.cancel();
		               }
		           });
		    AlertDialog alert = builder.create();
		    alert.setTitle(getString(R.string.aviso));
		    alert.setIcon(R.drawable.ic_dialog_advertencia);
		    alert.show();
	    	return true;
	    }else{
	    	return false;
	    }
	}
	
	@Override
	protected boolean isRouteDisplayed() {
		return false;
	}

	private void showAlertInformacion(){
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setMessage(getString(R.string.infoMapa))
		       .setCancelable(false)
		       .setPositiveButton(getString(R.string.butContinuar), new DialogInterface.OnClickListener() {
		           public void onClick(DialogInterface dialog, int id) {
		        	   dialog.cancel();
		           }
		       });
		AlertDialog alert = builder.create();
		alert.setTitle(getString(R.string.informacion));
		alert.setIcon(R.drawable.ic_dialog_informacion);
		alert.show();
	}
	
	private void showToastConexion(){
		Toast toast = Toast.makeText(getBaseContext(),
				getString(R.string.errConexion),
				Toast.LENGTH_SHORT);
		toast.show();
	}
	
	private void showToastDatos(){
		Toast toast = Toast.makeText(getBaseContext(),
				getString(R.string.errDatos),
				Toast.LENGTH_SHORT);
		toast.show();
	}

	private void launchTask(){		
		timer = new Timer();
		timer.scheduleAtFixedRate(new TimerTask() {
			@Override
			public void run() {
				obtenerDatos();
			}
		}
		,RETARDO, PERIODO);
	}
	
	private void obtenerDatos(){
		int resultado;
		try{
			DefaultHttpClient httpclient = new DefaultHttpClient();
			int idJuego = JuegoListar.getJuego().getIdentificador();
			URI uri = new URI(GoldenStreets.urlServer+"juego/"+idJuego+"/datos/posicionar/?format=json");
			HttpGet httpget = new HttpGet(uri);
			LibreGeoSocial LGS = LibreGeoSocial.getInstance();
			httpclient.setCookieStore(LGS.getCookieStore());
			HttpResponse response = httpclient.execute(httpget);
			HttpEntity entity = response.getEntity();
			String str = Utils.deStreamAString(entity.getContent());
			resultado = procesarRespuestaDatos(str);
			handler.sendEmptyMessage(resultado);
			
		}catch(URISyntaxException urisex){
			Log.e(TAG, urisex.getMessage());
			handler.sendEmptyMessage(-1);
	    }catch (IOException e) {
	    	Log.e(TAG,e.getMessage());
	    	handler.sendEmptyMessage(-1);
	    }
	}
	
	private int procesarRespuestaDatos (String str) {
		String ultMensaje = null;
		int ultCode = 0;
	    try {
	    	JSONObject json;
	    	json = new JSONObject(str.toString());
	
			if (json.has("code"))
				ultCode = json.getInt("code");
			else
				ultCode = -1;
			
			if (json.has("descripcion"))
				ultMensaje = json.getString("descripcion");
			else
				ultMensaje = null;
			
			if (json.has("equipos") && json.has("calles")){
				JSONArray equiposObjects = json.getJSONArray("equipos");
				int numeroEquipos = equiposObjects.length()-1;
				int equipoIdentificador;
				Double equipoLatitud;
				Double equipoLongitud;	
				for (int i = 0; i < numeroEquipos; i++){
					JSONObject JSONEquipo = (JSONObject)equiposObjects.get(i);
					equipoIdentificador = JSONEquipo.getInt("identificador");
					equipoLatitud = JSONEquipo.getDouble("latitud");
					equipoLongitud = JSONEquipo.getDouble("longitud");
					
					Equipo equipo = Equipo.encontrarPorId(JuegoDatosListar.getEquipos(), equipoIdentificador);
					equipo.setLatitude(equipoLatitud);
					equipo.setLongitude(equipoLongitud);
				}
				
				JSONArray callesObjects = json.getJSONArray("calles");
				int numeroCalles = callesObjects.length()-1;
				int calleIdentificador;
				int equipoIdPropietario;
				boolean estaConstruido;
				boolean estaSaboteado;	
				for (int i = 0; i < numeroCalles; i++){
					JSONObject JSONCalle = (JSONObject)callesObjects.get(i);
					calleIdentificador = JSONCalle.getInt("identificador");
					equipoIdPropietario = JSONCalle.getInt("idPropietario");
					estaConstruido = JSONCalle.getBoolean("estaConstruido");
					estaSaboteado = JSONCalle.getBoolean("estaSaboteado");
					Calle calle = Calle.encontrarPorId(JuegoDatosListar.getCalles(), calleIdentificador);
					calle.setIdPropietario(equipoIdPropietario);
					calle.setEstaConstruido(estaConstruido);
					calle.setEstaSaboteado(estaSaboteado);
				}
			}		
			if (ultCode != 200)
				return -2;
	
		} catch (Exception e) {	
			ultCode = 500;
			ultMensaje = "Error in json parsed: " + e.getMessage();
			Log.e(TAG, ultMensaje);
			return -2;
		}
		return 0;
	}
	
	private void centrarEnMiEquipo(){
		Location miPosicion = PosicionamientoServicio.getPosicionActual();
		if (miPosicion != null){
			GeoPoint GeoP = new GeoPoint((int)(miPosicion.getLatitude() *1E6),(int)(miPosicion.getLongitude() *1E6));
			mapControl.animateTo(GeoP);
		}
	}
	
	private void posicionarMiEquipo(Location miPosicion){
        Drawable drawable = this.getResources().getDrawable(R.drawable.ic_map_mi_equipo);
		String informacion = null;
		EquipoItemizedOverlay equipoItemizedOverlay = new EquipoItemizedOverlay(this, drawable);	        
        GeoPoint GeoP = new GeoPoint((int)(miPosicion.getLatitude() *1E6),(int)(miPosicion.getLongitude() *1E6));
        Equipo miEquipo = JuegoDatosListar.getEquipo();
        informacion = String.format(getString(R.string.infoEquipo), miEquipo.getGroupName());
        OverlayItem overlayItem = new OverlayItem(GeoP, miEquipo.getGroupName(), informacion);
        equipoItemizedOverlay.setMeters(RADIO_ACCION);
        equipoItemizedOverlay.addOverlay(overlayItem);
		mapOverlays.add(equipoItemizedOverlay);
	}

	private void posicionarCalles(Location miPosicion){		
		Calle[] calles = JuegoDatosListar.getCalles();
    	Calle calle;
    	Drawable drawable;
    	Equipo propietario;
    	String nombrePropietario = "-";
    	String informacion;     	
    	GeoPoint geoP;
    	OverlayItem overlayItem;

		float[] resultados = new float[3];
		double latitudInicial = miPosicion.getLatitude();
		double longitudInicial = miPosicion.getLongitude();
		double latitudFinal;
		double longitudFinal;
		int numeroCalles = calles.length;
        for (int i = 0; i < numeroCalles; i++){
        	calle = calles[i];
    		latitudFinal = calle.getLatitude();
    		longitudFinal = calle.getLongitude();
        	Location.distanceBetween(latitudInicial, longitudInicial, latitudFinal, longitudFinal, resultados);	
        	if(resultados[0] < RADIO_ACCION && resultados[0] <= RADIO_ACCION)
        		calleMenuObjetivo = calle;
        	
        	int idPropietario = calle.getIdPropietario();
        	int idMiEquipo = JuegoDatosListar.getEquipo().getIdentificador();
        	
        	if(idPropietario < 0){
        		nombrePropietario = "-";
        		drawable = this.getResources().getDrawable(R.drawable.ic_map_calle);
        	}else{
        		propietario = Equipo.encontrarPorId(JuegoDatosListar.getEquipos(), idPropietario);
        		nombrePropietario = propietario.getGroupName();
        		if(calle.getEstaConstruido())
	        		if(calle.getEstaSaboteado())
	        			if(idPropietario == idMiEquipo)
	        				drawable = this.getResources().getDrawable(R.drawable.ic_map_mi_saboteado);
	        			else
	        				drawable = this.getResources().getDrawable(R.drawable.ic_map_saboteado);
	        		else
	        			if(idPropietario == idMiEquipo)
	        				drawable = this.getResources().getDrawable(R.drawable.ic_map_mi_edificio);
	        			else
	        				drawable = this.getResources().getDrawable(R.drawable.ic_map_edificio);
        		else
        			if(idPropietario == idMiEquipo)
        				drawable = this.getResources().getDrawable(R.drawable.ic_map_mi_comprado);
        			else
        				drawable = this.getResources().getDrawable(R.drawable.ic_map_comprado);
        	}
        	
        	if(idPropietario == idMiEquipo){
		        informacion = String.format(getString(R.string.infoMiCalle), calle.getNombre(), calle.getPrecio(), nombrePropietario,
		        		calle.getNombreEdificio(), calle.getPrecioEdificio(), String.valueOf(calle.getEstaConstruido()), 
		        		String.valueOf(calle.getEstaSaboteado()), calle.getBonificacionEdificio());
	        }else{
	        	informacion = String.format(getString(R.string.infoCalle), calle.getNombre(), calle.getPrecio(), nombrePropietario,
	        			calle.getNombreEdificio(), calle.getEstaConstruido(), calle.getEstaSaboteado());
	        }
        	
	        geoP = new GeoPoint((int)(latitudFinal*1E6),(int)(longitudFinal *1E6));
	        
	        overlayItem = new OverlayItem(geoP, calle.getNombre(), informacion);	        
	        datosMapaItemizedOverlay = new DatosMapaItemizedOverlay(this, drawable);
	        datosMapaItemizedOverlay.addOverlay(overlayItem);
	        mapOverlays.add(datosMapaItemizedOverlay);  
        }
	}
	
	private void posicionarEquipos(){
		Equipo[] equipos = JuegoDatosListar.getEquipos();		
		Equipo equipo;		
		Drawable drawable = this.getResources().getDrawable(R.drawable.ic_map_equipo);
		String informacion;		
		GeoPoint GeoP;
		OverlayItem overlayItem;
		
		int numeroEquipos = equipos.length;
        for (int i = 0; i < numeroEquipos; i++){
        	equipo = equipos[i];	  
        	if (equipo.getIdentificador() != JuegoDatosListar.getEquipo().getIdentificador()){
        		datosMapaItemizedOverlay = new DatosMapaItemizedOverlay(this,drawable);	        
		        GeoP = new GeoPoint((int)(equipo.getLatitude() *1E6),(int)(equipo.getLongitude() *1E6));
		        informacion = String.format(getString(R.string.infoEquipo), equipo.getGroupName());
		        overlayItem = new OverlayItem(GeoP, equipo.getGroupName(), informacion);
		        
		        datosMapaItemizedOverlay.addOverlay(overlayItem);
		        mapOverlays.add(datosMapaItemizedOverlay); 
        	}
        }
	}
	 
	private Handler handler = new Handler() {
		@Override
		public void handleMessage(Message msg) {
			int result = msg.what;
			calleMenuObjetivo = null;
        	if (result == -2){        		
				showToastDatos();
        	}else if (result == -1){
        		showToastConexion();
        	}else if (result == 0){        		
        		mapOverlays.clear();
        		Location miPosicion = PosicionamientoServicio.getPosicionActual();
        		if (miPosicion != null){ 
        			posicionarEquipos();
        			posicionarCalles(miPosicion);
            		posicionarMiEquipo(miPosicion);
        		}
        		mapView.invalidate();
        	}
		}
	};
	
    private BroadcastReceiver receiver = new BroadcastReceiver() {        
        @Override
        public void onReceive(Context context, Intent intent) {
        	String accion = intent.getAction();
        	if (accion.equals("CartaServicio.NUEVA")) {
        		AlertDialog alert = AccionesRecibidasBroadcast.accionCartaServicioNueva(context, intent);
		    	alert.show();
        	} else if(accion.equals("CartaServicio.SIN")) {
        		AlertDialog alert = AccionesRecibidasBroadcast.accionCartaServicioSin(context, intent);
		    	alert.show();
        	}
        }
    };
}