package com.Goldenstreet.App.GoldenStreet;

import android.app.Service;
import android.content.Intent;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.os.IBinder;
import android.util.Log;

import com.LibreGeoSocial.Api.LibreGeoSocial;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.List;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.protocol.HTTP;
import org.json.JSONException;
import org.json.JSONObject;

/**
 * Servicio que peridicamente obtendr las posicin del equipo para envirsela al servidor
 * y que los dems equipos conozcan su ubicacin.
 * 
 * @author Rubn Quero Castrillo <rubenquero@gmail.com>
 * 
 */

public class PosicionamientoServicio extends Service implements LocationListener, Runnable {
	static private String TAG = "PosicionamientoServicio";
 
	private static final int DISTANCIA = 8; 
	private static final int PERIODO = 5*1000;
	private static LocationManager locationManager;
	private static Location posicionActual;
	
	public static LocationManager getLocationManager() {
		return locationManager;
	}
	public static void setLocationManager(LocationManager locationManager) {
		PosicionamientoServicio.locationManager = locationManager;
	}

	public static Location getPosicionActual() {
		return posicionActual;
	}

	@Override
	public void onCreate() {
		super.onCreate();		
		PosicionamientoServicio.posicionActual = locationManager.getLastKnownLocation(LocationManager.GPS_PROVIDER);
		locationManager.requestLocationUpdates(
			LocationManager.GPS_PROVIDER, 
			PERIODO, 
			DISTANCIA,
			PosicionamientoServicio.this
		);
		/*PosicionamientoServicio.posicionActual = locationManager.getLastKnownLocation(LocationManager.NETWORK_PROVIDER);
		locationManager.requestLocationUpdates(
			LocationManager.NETWORK_PROVIDER, 
			PERIODO, 
			DISTANCIA,
			PosicionamientoServicio.this
		);*/
	}
	
	@Override
	public void onDestroy() {
		super.onDestroy();
		if (locationManager != null)
			locationManager.removeUpdates(this);
	}
 	
	@Override
	public void onLocationChanged(Location location) {		
		if (location != null){
			PosicionamientoServicio.posicionActual = location;
			Thread thread = new Thread(this);
	        thread.start();
		}
	}

	@Override
	public void onProviderDisabled(String provider) {
		// TODO Auto-generated method stub
	}

	@Override
	public void onProviderEnabled(String provider) {
		// TODO Auto-generated method stub
	}

	@Override
	public void onStatusChanged(String provider, int status, Bundle extras) {
		// TODO Auto-generated method stub	
	}
	
	@Override
	public IBinder onBind(Intent arg0) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public void run() {
		posicionamientoModificar();
	}
	
	private void posicionamientoModificar(){
		//int resultado;
		try{
			DefaultHttpClient httpclient = new DefaultHttpClient();
			int idJuego = JuegoListar.getJuego().getIdentificador();
			int idEquipo = JuegoDatosListar.getEquipo().getIdentificador();
			URI uri = new URI(GoldenStreets.urlServer+"juego/"+idJuego+"/equipo/"+idEquipo+"/posicionar/?format=json");
			HttpPost httpost = new HttpPost(uri);
			LibreGeoSocial LGS = LibreGeoSocial.getInstance();
			httpclient.setCookieStore(LGS.getCookieStore());
			List <NameValuePair> datos = new ArrayList <NameValuePair>();
            datos.add(new BasicNameValuePair("latitud", String.valueOf(PosicionamientoServicio.posicionActual.getLatitude())));
            datos.add(new BasicNameValuePair("longitud", String.valueOf(PosicionamientoServicio.posicionActual.getLongitude())));
            httpost.setEntity(new UrlEncodedFormEntity(datos, HTTP.UTF_8));
			
			HttpResponse response = httpclient.execute(httpost);
			HttpEntity entity = response.getEntity();
			String str = Utils.deStreamAString(entity.getContent());
			procesarRespuestaPosicionamiento(str);
			
		}catch(URISyntaxException urisex){
			//resultado = -1;
			Log.e(TAG, urisex.getMessage());
	    }catch (IOException e) {
	    	//resultado = -1;
	    	Log.e(TAG,e.getMessage());
	    }
	}
	
	private int procesarRespuestaPosicionamiento(String str) {
		int ultCode;
    	String ultMensaje;
        try {
        	JSONObject json;
			json = new JSONObject(str);
			
			if (json.has("code"))
				ultCode = json.getInt("code");
			else
				ultCode = -1;
			
			if (json.has("descripcion"))
				ultMensaje = json.getString("descripcion");
			else
				ultMensaje = null;
			 
			if (ultCode != 200){
				return -2;
			}
			
		} catch (JSONException e) {	
			ultCode = 500;
			ultMensaje = "Error in json parsed: " + e.getMessage();
			Log.e(TAG, ultMensaje);
			return -2;
		}
		return 0;
	}
}