/*
 *
 *  Copyright (C) 2009 GSyC/LibreSoft, Universidad Rey Juan Carlos
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/. 
 *
 *  Authors : Roberto Calvo Palomino <rocapal@gsyc.es>
 *  		  Juan Francisco Gato Luis <jfcogato@gsyc.es>
 *
 */

package com.LibreGeoSocial.Api.Layers;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;

import android.graphics.Bitmap;
import android.os.Handler;
import android.util.Log;

import com.LibreGeoSocial.Api.GeoNode;
import com.LibreGeoSocial.Api.GeoNodePositionComparator;
import com.LibreGeoSocial.Api.LibreGeoSocial;

public class GenericLayer extends GeoNode implements Serializable {

	/** 
	 * 
	 */
	private static final long serialVersionUID = -6501648066678686473L;
	
	/**
	 * @uml.property  name="mPattern"
	 */
	private String mPattern;
	/**
	 * @uml.property  name="mPage"
	 */
	private Integer mPage;
	
	/**
	 * @uml.property  name="mLayer_type"
	 */
	private String mLayer_type;

	/**
	 * @uml.property  name="mName"
	 */
	private String mName;

	/**
	 * @uml.property  name="mDescription"
	 */
	private String mDescription;
	/**
	 * @uml.property  name="mWriteable"
	 */
	private Boolean mWriteable;

	/**
	 * @uml.property  name="mFree"
	 */
	private Boolean mFree;
	
	/**
	 * @uml.property  name="mArrayNodes"
	 */
	protected ArrayList<GeoNode> mArrayNodes = new ArrayList<GeoNode>();
	/**
	 * @uml.property  name="mCategories"
	 * @uml.associationEnd  multiplicity="(0 -1)" elementType="com.Goldenstreet.Api.Layers.Category"
	 */
	private ArrayList<Category> mCategories = new ArrayList<Category>();
	
	public GenericLayer (Integer id, String layer_type, String name, String description, String since,
			Double latitude, Double longitude, Double altitude, Double radius, String since_position){
		
		super(id, latitude, longitude, altitude, radius, since, since_position );
		
		mWriteable = false;
		mLayer_type = layer_type;
		mName = name;
		mFree = true;
		mDescription = description;
	}
	
	public boolean getWriteable (){
		return mWriteable;
	}
	public void setWriteable(Boolean mWriteable){
		this.mWriteable = mWriteable;
	}
	public String getNameLayer() {
		return mName;
	}
	public void setNameLayer(String mName) {
		this.mName = mName;
	}
	public String getPattern() {
		return mPattern;
	}
	public void setPattern(String mPattern) {
		this.mPattern = mPattern;
	}
	
	public ArrayList<GeoNode> getNodes()
	{
		return mArrayNodes;
	}
	
	public void setNodes ( ArrayList<GeoNode> nodes )
	{
		mArrayNodes = nodes;
	}
	public String getDescription(){
		return mDescription;
	}
	
	public Bitmap getIcon ()
	{
		return LibreGeoSocial.getInstance().getBitmap("layer/" + getId().toString() + "/icon/" );
	}

	public void resetPagination()
	{
		mPage=1;
	}
	
	public void nextPage()
	{
		mPage++;
	}
	
	public void previousPage()
	{
		if (mPage>1)
			mPage--;
	}
	
	public Integer getCurrentPage()
	{
		return mPage;
	}
	
	public Thread getThreadtoUpdate(final Handler hNotification,  final String[] categories) 
	{
		final Integer elems =15;
		
		Log.e("LibreGeoSocialLayer","getThreadtoUpdate");
		
		Thread th = new Thread()
		{
        	public void run()
        	{
        		if (GenericLayer.this.getId() == -1){
        			//If the layer is local
        			try {
                        sleep(1000);
                        hNotification.sendEmptyMessage(0);

                    } catch (InterruptedException e) {
                        Log.e ("NodeListLayer: ", e.getMessage().toString() );
                    }
        		}else{
        			final int size = categories.length;
        			ArrayList<GeoNode> auxArray = new ArrayList<GeoNode>();
        		
        			for (int i = 0; i < size; i++){
        				if (categories[i] != "x")
        					auxArray.addAll (LibreGeoSocial.getInstance().getLayerNodes (GenericLayer.this.getId(), 
        																	 getPattern(), 
        																	 categories[i], 
        																	 getRadius(),
        																	 getCurrentPage(),
        																	 elems));
        			}
        		
        			if (! auxArray.isEmpty() )
        			{
        				GeoNodePositionComparator comparator = new GeoNodePositionComparator();
        				Collections.sort(auxArray, comparator);
        			}
        		
        			mArrayNodes = auxArray;
        		
        			hNotification.sendEmptyMessage(0);
        		}
        	}
		};
		  
		return th;
	}
	
	public String[] getCategories (String take)
	{
		if (GenericLayer.this.getId() == -1){
			//for local layers
			//Log.e("Categoria","null");
			//String[] list = new String[1];
			//list[0] = "vacio";
			return null;
		}
		
		mCategories = LibreGeoSocial.getInstance().parseCategories(GenericLayer.this.getId());
		Integer size = mCategories.size();
		String[] list = new String[size];
		if (take.equalsIgnoreCase("id"))
			for (int i = 0; i < size; i++){
				list[i] = mCategories.get(i).getId().toString();				
			}
		else if (take.equals("name"))
			for (int i = 0; i < size; i++){
				list[i] = mCategories.get(i).getName();				
			}
		return list;
	}
}
