from apps.goldenstreet.models import *
from apps.goldenstreet.core import oferta_bd
from django.db.models import F
from django.db.models import Q

def crear(nombre, ubicacion, precio, juego):
    nuevaCalle = Calle()
    nuevaCalle.position = ubicacion
    nuevaCalle.nombre = nombre
    nuevaCalle.precio = precio
    nuevaCalle.juego = juego
    nuevaCalle.save()

    return True, nuevaCalle

def modificar(calle, nombre, ubicacion, precio):
    calle.nombre = nombre
    calle.position = ubicacion
    calle.precio = precio
    calle.save()

    return True, calle

def listar(juego):
    return True, juego.calle_set.all().order_by('id')

def coger(juego, calleId):
    try:
      calle = juego.calle_set.get(id=calleId)
      return True, calle
    except:
      return False, 'Calle invalida'

def listarPorEquipo(equipo):
      return True, equipo.calle_set.all()

def borrar(calle, usuarioId):
    edificios = calle.edificio_set.all()
    for edificio in edificios:
        edificio.delete()

    calle.delete(usuarioId)
    return True, 'Sin errores'

def comprar(juego, calleId, patrimonio, callePrecio):
    try:
        updated = juego.calle_set.filter(Q(id=calleId) & Q(equipo=None))\
                  .update(equipo=patrimonio.equipo,
                          version=F('version') + 1)
        if not updated:
            return False, 'Calle con modificaciones'
        else:
            patrimonio.valPatrimonio = F('valPatrimonio') + callePrecio
            patrimonio.dinero = F('dinero') - callePrecio
            patrimonio.numPropiedades = F('numPropiedades') + 1
            patrimonio.save() 
            return True, 'Ok'
    except:
        return False, 'Calle invalida'

def vender(juego, calleId, patrimonio, valorTotalCalle, valorVenta):
    try:
        updated = juego.calle_set.filter(Q(id=calleId))\
                  .update(equipo = None,
                          estaConstruido = False,
                          estaSaboteado = False,
                          version=F('version') + 1)
        if not updated:
            return False, 'Calle con modificaciones'
        else:
            patrimonio.valPatrimonio = F('valPatrimonio') - valorTotalCalle
            patrimonio.dinero = F('dinero') + valorVenta
            patrimonio.numPropiedades = F('numPropiedades') - 1
            patrimonio.save() 
            return True, 'Ok'
    except:
        return False, 'Calle invalida'
    
def construir(juego, calleId, patrimonio, edificioPrecio, edificioPrecioOriginal):
    try:
        updated = juego.calle_set.filter(Q(id=calleId) & Q(estaConstruido=False) & Q(estaSaboteado=False))\
                  .update(estaConstruido=True,
                          version=F('version') + 1)
        if not updated:
            return False, 'Calle con modificaciones'
        else:
            patrimonio.valPatrimonio = F('valPatrimonio') + edificioPrecioOriginal
            patrimonio.dinero = F('dinero') - edificioPrecio
            patrimonio.numPropiedades = F('numPropiedades')
            patrimonio.save() 
            return True, 'Ok'
    except:
        return False, 'Calle invalida'

def sabotear(juego, calleId):
    try:
        updated = juego.calle_set.filter(Q(id=calleId) & Q(estaSaboteado=False) & Q(estaConstruido=True))\
                  .update(estaSaboteado=True,
                          version=F('version') + 1)
        if not updated:
            return False, 'Calle con modificaciones'
        else:
            return True, 'Ok'
    except:
        return False, 'Calle invalida'

def demoler(juego, calleId, patrimonio, edificioValor):
    try:
        updated = juego.calle_set.filter(Q(id=calleId) & (Q(estaConstruido=True) | Q(estaSaboteado=True)))\
                  .update(estaSaboteado=False,
                          estaConstruido=False,
                          version=F('version') + 1)
        if not updated:
            return False, 'Calle con modificaciones'
        else:
            patrimonio.valPatrimonio = F('valPatrimonio') - edificioValor
            patrimonio.dinero = F('dinero')
            patrimonio.numPropiedades = F('numPropiedades')
            patrimonio.save()
            return True, 'Ok'
    except:
        return False, 'Calle invalida'

def ofertar(juego, calleId, calleVersion, calle, ofertaCantidad, ofertante, propietario):
    try:
        if not calle.version == calleVersion:
            return False, 'Calle con modificaciones'
        oferta_bd.crear(juego, propietario, ofertante, calle, ofertaCantidad)
        return True, 'Ok'
    except:
        return False, 'Calle invalida'
