from datetime import datetime

from django.db import models
from django.contrib.gis.db import models

from social.core.models import Social_node
from social.core.models import *


class Equipo(models.Model):
    juegos = models.ForeignKey("Juego", null=False)
    grupo = models.ForeignKey(Group, null=False)
    cartas = models.ManyToManyField("Carta", blank=True, null=True) #Eliminar

class Organizador(models.Model):
    usuario = models.ForeignKey(Person, null=False)
    juegos = models.ManyToManyField("Juego", blank=True, null=True)

class Patrimonio(models.Model):
    juego = models.ForeignKey("Juego",null=False)
    equipo = models.ForeignKey("Equipo",null=False)
    valPatrimonio = models.IntegerField(default=0, null=False)
    dinero = models.IntegerField(default=0, null=False)
    numPropiedades = models.IntegerField(default=0, null=False)

class Carta(models.Model):
    nombre = models.CharField(max_length=255, null=False)
    descripcion = models.CharField(max_length=255, null=False)
    tipoAccion = models.IntegerField(default=1, null=False) #1:Ingresar; 2:Retirar; 3:Sabotaje, ...
    cantidad = models.IntegerField(default=0, null=True)
    usada = models.BooleanField(default=False, null=False)

class Juego(models.Model):
    nombre = models.CharField(max_length=255, null=False)
    ubicacion = models.CharField(max_length=255, null=False)
    fechaInicio = models.DateField(default=datetime.now(), null=False)
    dineroInicial = models.IntegerField(default=0, null=False)
    duracion = models.IntegerField(default=0, null=False)
    estaCerrado = models.BooleanField(default=False, null=False)

class Calle(Social_node):
    juego = models.ForeignKey("Juego", null=False)
    equipo = models.ForeignKey("Equipo", blank=True, null=True)
    nombre = models.CharField(max_length=255, null=False)
    precio = models.IntegerField(default=0, null=False)
    estaConstruido = models.BooleanField(default=False, null=False)
    estaSaboteado = models.BooleanField(default=False, null=False)
    version = models.IntegerField(default=0, null=False)

class Edificio(models.Model):
    juego = models.ForeignKey("Juego", null=False)
    calle = models.ForeignKey("Calle", null=False)
    nombre = models.CharField(max_length=255, null=False)
    bonificacion = models.IntegerField(default=0, null=False)
    precio = models.IntegerField(default=0, null=False)

class Mensaje(models.Model):
    juego = models.ForeignKey("Juego", null=False)
    delOrganizador = models.ForeignKey("Organizador", blank=True, null=True, related_name="envio_organizador")
    delEquipo = models.ForeignKey("Equipo",blank=True, null=True, related_name="envio_equipo")
    paraOrganizador = models.ForeignKey("Organizador", blank=True, null=True, related_name="recibe_organizador")
    paraEquipos = models.ManyToManyField("Equipo", blank=True, null=True, related_name="recibe_equipo")
    text = models.TextField(null=False)

class Oferta(models.Model):
    juego = models.ForeignKey("Juego", null=False)
    propietario = models.ForeignKey("Equipo",null=False, related_name="propietario_equipo")
    ofertante = models.ForeignKey("Equipo",null=False, related_name="ofertante_equipo")
    calle = models.ForeignKey("Calle", null=False)
    cantidad = models.IntegerField(default=0, null=False)
    respondida = models.BooleanField(default=False, null=False)
    aceptada = models.BooleanField(default=False, null=False)
