# Create your views here.

from django.shortcuts import render_to_response
from django.http import HttpResponseRedirect
from django.contrib.auth import authenticate, login
from django.contrib.auth import logout

from django.contrib.gis.geos import Point

from social.core import api as api_lgs
from social.rest.forms import LoginForm

from apps.goldenstreet.core import organizador_bd
from apps.goldenstreet.core import juego_bd
from apps.goldenstreet.core import mensaje_bd
from apps.goldenstreet.core import equipo_bd
from apps.goldenstreet.core import calle_bd
from apps.goldenstreet.core import edificio_bd
from apps.goldenstreet.core import carta_bd
from apps.goldenstreet.core import patrimonio_bd
from apps.goldenstreet.core import oferta_bd
from apps.goldenstreet.models import *

USUARIO_ORGANIZADOR = 'organizador'
CLAVE_ORGANIZADOR = 'django'

def comprobar_formato(request):
    if 'format' in request.GET:
      formato = request.GET['format'].lower().strip()
    elif 'format' in request.POST:
      formato = request.POST['format'].lower().strip()
    else:
      formato = 'html'

    if formato == 'xml':
      return True, 'xml'
    elif formato == 'json':
      return True, 'json'
    elif formato == 'html':
      return True, 'html'
    else:
      return False, 'Formato invalido. Solo .xml, .json y .html .'


def inicio(request):
    if not request.user.is_authenticated():
        url = '/goldenstreet/login/'
        return HttpResponseRedirect(url)
    else:
        return render_to_response('index.html')

#----Usuario---- 
def usuario_login(request):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})
    if not request.user.is_authenticated():
        if request.method == 'GET':
            correcto, organizadores = organizador_bd.listar()
            if len(organizadores) == 0:
                organizador = {'username': USUARIO_ORGANIZADOR, 'password': CLAVE_ORGANIZADOR}
                correcto, idUsuario = api_lgs.user.create_or_modify(organizador, modify=False)
                if not correcto:
                    return render_to_response('fallo.' + formato, {'code': 500, 'descripcion': idUsuario})
                usuario = Person.objects.get(id=idUsuario);
                organizador_bd.crear(usuario)
                return render_to_response('login.' + formato, {'code': 200, 'descripcion': 'ok'})
            return render_to_response('login.' + formato, {'code': 200, 'descripcion': 'ok'})
        elif request.method == 'POST':
            loginform = LoginForm(request.POST)
            id = loginform.login(request)
            if id:
                url = '/goldenstreet/'
                return HttpResponseRedirect(url)
            else:
                msgerror = 'Usuario y/o password no encontrados.'
                return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': msgerror})
    else:
        url = '/goldenstreet/'
        return HttpResponseRedirect(url)

        
def usuario_logout(request):
    url = '/goldenstreet/login/'
    if not request.user.is_authenticated():
        return HttpResponseRedirect(url)
    else:
        logout(request)
        return HttpResponseRedirect(url)


#----Juego----    
def juego_crear(request):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})
    if not request.user.is_authenticated():
        if formato == 'html':
            url = '/goldenstreet/login/'
            return HttpResponseRedirect(url)
        else:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': 'formato incorrecto'})
    else:
        administrador = organizador_bd.comprobarTipoSession(request)
        if administrador == True:
            if request.method == 'GET':
                return render_to_response('juegocrear.' + formato, {'code': 200, 'descripcion': 'ok'})
            elif request.method == 'POST':
                nombre = request.POST['nombre']
                ubicacion = request.POST['ubicacion']
                fechaInicio = request.POST['fechaInicio']
                dineroInicial = request.POST['dineroInicial']
                duracionInicial = request.POST['duracionInicial']
                usuarioId = request.session['_auth_user_id']
                correcto, nuevoJuego = juego_bd.crear(nombre, ubicacion, fechaInicio, dineroInicial, duracionInicial, usuarioId)
                if not correcto:
                    return render_to_response('fallo.' + formato,{'code': 500, 'descripcion': nuevoJuego})
                else:
                    url = '/goldenstreet/juego/' + str(nuevoJuego.id) + '/mostrar/'
                    return HttpResponseRedirect(url)
        else:
            return render_to_response('fallo.' + formato,{'code': 200, 'descripcion': 'No tienes suficientes permisos para realizar esta accion'}) 


def juego_listar(request):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})

    if request.method != 'GET':
        return render_to_response('fallo.' + formato, {'code': 400, 'description': 'Metodo request invalido'})
    
    if formato == 'html':
        if not request.user.is_authenticated():
            url = '/goldenstreet/login/'
            return HttpResponseRedirect(url)

    correcto, juegos = juego_bd.listar()
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juegos})

    administrador = organizador_bd.comprobarTipoSession(request)
    if len(juegos) == 0:
        return render_to_response('juegolistar.' + formato, {'code': 200, 'descripcion': 'no hay juegos', 'admin': administrador})
    else:
        return render_to_response('juegolistar.' + formato, {'code': 200, 'descripcion': 'ok', 'juegos': juegos,'admin': administrador})
    

def juego_mostrar(request, juegoId):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})

    if request.method != 'GET':
        return render_to_response('fallo.' + formato, {'code': 400, 'description': 'Metodo request invalido'})
    
    if not request.user.is_authenticated():
        if formato == 'html':
            url = '/goldenstreet/login/'
            return HttpResponseRedirect(url)
        else:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': 'formato incorrecto'})
    else:
        correcto, juego = juego_bd.coger(juegoId)
        if not correcto:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})
        
        administrador = organizador_bd.comprobarTipoSession(request)
        return render_to_response('juegomostrar.' + formato, {'code': 200, 'descripcion': 'ok', 'juego': juego,'admin': administrador})


def juego_modificar(request, juegoId):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})
    if not request.user.is_authenticated():
        if formato == 'html':
            url = '/goldenstreet/login/'
            return HttpResponseRedirect(url)
        else:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': 'formato incorrecto'})
    else:
        administrador = organizador_bd.comprobarTipoSession(request)
        if administrador == True:
            if request.method == 'GET':
                correcto, juego = juego_bd.coger(juegoId)
                if not correcto:
                    return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})
                else:
                    return render_to_response('juegomodificar.' + formato, {'code': 200, 'descripcion': 'ok', 'juego': juego})
            elif request.method == 'POST':
                correcto, juego = juego_bd.coger(juegoId)
                if not correcto:
                    return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})

                nombre = request.POST['nombre']
                ubicacion = request.POST['ubicacion']
                fechaInicio = request.POST['fechaInicio']
                dinero = request.POST['dinero']
                duracion = request.POST['duracion']
                correcto, juegoModificado = juego_bd.modificar(juego, nombre, ubicacion, fechaInicio, dinero, duracion)
                if not correcto:
                    return render_to_response('fallo.' + formato, {'code': 500, 'descripcion': nuevoJuego})
                else:
                    url = '/goldenstreet/juego/' + str(juegoModificado.id) + '/mostrar/'
                    return HttpResponseRedirect(url)
        else:
            return render_to_response('fallo.' + formato, {'code': 200, 'descripcion': 'No tienes suficientes permisos para realizar esta accion'}) 
                

def juego_borrar(request, juegoId):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})
    if not request.user.is_authenticated():
        if formato == 'html':
            url = '/goldenstreet/login/'
            return HttpResponseRedirect(url)
        else:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': 'formato incorrecto'})
    else:
        administrador = organizador_bd.comprobarTipoSession(request)
        if administrador == True:
            correcto, juego = juego_bd.coger(juegoId)
            if not correcto:
                return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})
            
            usuarioId = request.session['_auth_user_id']
            correcto, msgerror = juego_bd.borrar(juego, usuarioId)
            if not correcto:
                return render_to_response('fallo.' + formato, {'code': 500, 'descripcion': msgerror})
            else:
                url = '/goldenstreet/juego/listar/'
                return HttpResponseRedirect(url)
        else:
            return render_to_response('fallo.' + formato, {'code': 200, 'descripcion': 'No tienes suficientes permisos para realizar esta accion'})


def juego_establecer_estado(request, juegoId):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})

    if request.method != 'GET':
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': 'Metodo request invalido'})
    
    if not request.user.is_authenticated():
        if formato == 'html':
            url = '/goldenstreet/login/'
            return HttpResponseRedirect(url)
        else:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': 'formato incorrecto'})
    else:
        administrador = organizador_bd.comprobarTipoSession(request)
        if administrador == True:
            correcto, juego = juego_bd.coger(juegoId)
            if not correcto:
                return render_to_response('fallo.' + formato, {'code': 4001, 'descripcion': juego})

            correcto, juego = juego_bd.establecerEstado(juego)
            if not correcto:
                return render_to_response('fallo.' + formato, {'code': 500, 'descripcion': juego})
            else:
                url = '/goldenstreet/juego/listar/'
                return HttpResponseRedirect(url)
        else:
            return render_to_response('fallo.' + formato, {'code': 200, 'descripcion': 'No tienes suficientes permisos para realizar esta accion'})


def juego_seguimiento(request, juegoId):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})
    if not request.user.is_authenticated():
        if formato == 'html':
            url = '/goldenstreet/login/'
            return HttpResponseRedirect(url)
        else:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': 'formato incorrecto'})
    else:
        correcto, juego = juego_bd.coger(juegoId)
        if not correcto:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})
  
        correcto, patrimonios = patrimonio_bd.listar(juego)
        if not correcto:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': patrimonios})

        correcto, calles = calle_bd.listar(juego)
        if not correcto:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': calles})
   
        correcto, edificios = edificio_bd.listarPorJuego(juego)
        if not correcto:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': edificios})
        
        correcto, mensajes = mensaje_bd.listar(juego)
        if not correcto:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': mensajes})
    
        administrador = organizador_bd.comprobarTipoSession(request)
        if administrador == True:
            return render_to_response('seguimiento.' + formato, {'code': 200, 'descripcion': 'ok', 'juego':juego, 'patrimonios':patrimonios, 'calles':calles, 'edificios':edificios, 'mensajes':mensajes, 'admin':administrador})
        else:
            return render_to_response('seguimiento.' + formato, {'code': 200, 'descripcion': 'ok', 'juego':juego, 'patrimonios':patrimonios, 'calles':calles, 'edificios':edificios, 'mensajes':mensajes})


#----Juego-Datos---- 
def juego_datos_listar(request, juegoId):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})
    
    if request.method != 'GET':
        return render_to_response('fallo.' + formato, {'code': 400, 'description': 'Metodo request invalido'})
    
    if formato == 'html':
        url = '/goldenstreet/login/'
        return HttpResponseRedirect(url)
    
    correcto, juego = juego_bd.coger(juegoId)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})
    
    correcto, equipos = equipo_bd.listar(juego)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': equipos})
   
    correcto, calles = calle_bd.listar(juego)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': calles})
    
    correcto, edificios = edificio_bd.listarPorJuego(juego)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': edificios})
                    
    administrador = organizador_bd.comprobarTipoSession(request)
    if len(equipos) == 0 or len(calles) == 0 or len(calles) != len(edificios):
        return render_to_response('partidadatoslistar.' + formato, {'code': 200, 'descripcion': 'Los datos del juego no estan completos', 'juego': juego, 'admin': administrador})
    else:
        return render_to_response('partidadatoslistar.' + formato, {'code': 200, 'descripcion': 'ok', 'juego': juego, 'equipos': equipos, 'calles': calles, 'edificios': edificios, 'admin': administrador})


def juego_datos_posicionar(request, juegoId):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})
    
    if request.method != 'GET':
        return render_to_response('fallo.' + formato, {'code': 400, 'description': 'Metodo request invalido'})

    if formato == 'html':
        url = '/goldenstreet/login/'
        return HttpResponseRedirect(url)
    
    correcto, juego = juego_bd.coger(juegoId)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})

    correcto, equipos = equipo_bd.listar(juego)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': equipos})
    
    correcto, calles = calle_bd.listar(juego)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': calles})
    else:
        administrador = organizador_bd.comprobarTipoSession(request)
        return render_to_response('partidadatoslocalizar.' + formato, {'code': 200, 'descripcion': 'ok', 'juego': juego, 'equipos': equipos, 'calles': calles, 'admin': administrador})

                               
#----Juego-Mensaje---- 
def mensaje_crear(request, juegoId):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})
    if request.method == 'GET':
        correcto, juego = juego_bd.coger(juegoId)
        if not correcto:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})
        
        correcto, equipos = equipo_bd.listar(juego)
        if not correcto:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': equipos})
        else:
            if formato == 'html':
                administrador = organizador_bd.comprobarTipoSession(request)
                return render_to_response('mensajecrear.' + formato, {'code': 200, 'descripcion': 'ok', 'juego':juego, 'equipos':equipos, 'admin':administrador})
    elif request.method == 'POST':
        destinatario = None
        emisorId = None
        correcto, juego = juego_bd.coger(juegoId)
        if not correcto:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})
        else:
            enviadoPor = request.POST['emisor']
            para = request.POST['destinatario']
            texto = request.POST['texto']
            if para == 'unEquipo':
                equipoId = request.POST['equipos']
                correcto, destinatario = equipo_bd.coger(juego, equipoId)
                if not correcto:
                    return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': destinatario})
            elif para == 'organizador':
                correcto, destinatario = organizador_bd.obtenerPorPartida(juego)
                if not correcto:
                    return render_to_response('fallo.' + formato, {'code': 402, 'descripcion': destinatario})
            if enviadoPor == 'organizador':
                emisorId = request.session['_auth_user_id']
            else:
                emisorId = enviadoPor;
            correcto, nuevoMensaje = mensaje_bd.crear(juego, para, enviadoPor, destinatario, emisorId, texto)
            if not correcto:
                return render_to_response('fallo.' + formato, {'code': 500, 'descripcion': nuevoMensaje})
            else:
                if formato == 'html':
                    url = '/goldenstreet/juego/' + str(juegoId) + '/seguimiento/'
                    return HttpResponseRedirect(url)
                elif formato == 'json':
                    return render_to_response('respuesta.' + formato, {'code': 200, 'descripcion': 'ok'})       


def mensaje_borrar(request, juegoId, mensajeId):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})
    if not request.user.is_authenticated():
        if formato == 'html':
            url = '/goldenstreet/login/'
            return HttpResponseRedirect(url)
        else:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': 'formato incorrecto'})
    else:
        administrador = organizador_bd.comprobarTipoSession(request)
        if administrador == True:
            correcto, juego = juego_bd.coger(juegoId)
            if not correcto:
                return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})
            
            correcto, mensaje = mensaje_bd.coger(juego, mensajeId)
            if not correcto:
                return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': mensaje})
            
            correcto, msgerror = mensaje_bd.borrar(mensaje)
            if not correcto:
                return render_to_response('fallo.' + formato, {'code': 500, 'descripcion': msgerror})
            else:
                url = '/goldenstreet/juego/'+juegoId+'/seguimiento/'
                return HttpResponseRedirect(url)
        else:
            return render_to_response('fallo.' + formato, {'code': 200, 'descripcion': 'No tienes suficientes permisos para realizar esta accion'})

        
#----Juego-Patrimonio---- 
def patrimonio_listar(request, juegoId):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})
    
    if request.method != 'GET':
        return render_to_response('fallo.' + formato, {'code': 400, 'description': 'Metodo request invalido'})

    if formato == 'html':
        if not request.user.is_authenticated():
            url = '/goldenstreet/login/'
            return HttpResponseRedirect(url)
    correcto, juego = juego_bd.coger(juegoId)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})
    else:
        correcto, patrimonios = patrimonio_bd.listar(juego)
        if not correcto:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': patrimonios})

        administrador = organizador_bd.comprobarTipoSession(request)
        if len(patrimonios) == 0:
            return render_to_response('patrimoniolistar.' + formato, {'code': 200, 'descripcion': 'No hay patrimonios', 'juego': juego, 'admin': administrador})
        else:
            return render_to_response('patrimoniolistar.' + formato, {'code': 200, 'descripcion': 'ok', 'juego': juego, 'patrimonios': patrimonios, 'admin': administrador})


#----Equipo----
def equipo_crear(request, juegoId):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})
    if not request.user.is_authenticated():
        if formato == 'html':
            url = '/goldenstreet/login/'
            return HttpResponseRedirect(url)
        else:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': 'formato incorrecto'})
    else:
        administrador = organizador_bd.comprobarTipoSession(request)
        if administrador == True:
            if request.method == 'GET':
                correcto, juego = juego_bd.coger(juegoId)
                if not correcto:
                    return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})
                else:
                    return render_to_response('equipocrear.' + formato, {'code': 200, 'descripcion': 'ok', 'juego': juego})
            elif request.method == 'POST':
                nombre = request.POST['nombre']
                correcto, juego = juego_bd.coger(juegoId)
                if not correcto:
                    return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})
                
                ubicacion = Point(0, 0, srid=4326)
                correcto, nuevoEquipo = equipo_bd.crear(nombre, ubicacion, juego)
                if not correcto:
                    return render_to_response('fallo.' + formato,  {'code': 500, 'descripcion': nuevoEquipo})
                else:
                    url = '/goldenstreet/juego/' + str(juegoId) + '/equipo/' + str(nuevoEquipo.id) + '/mostrar/'
                    return HttpResponseRedirect(url)
        else:
            return render_to_response('fallo.' + formato, {'code': 200, 'descripcion': 'No tienes suficientes permisos para realizar esta accion'})         


def equipo_listar(request, juegoId):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})

    if request.method != 'GET':
        return render_to_response('fallo.' + formato, {'code': 400, 'description': 'Metodo request invalido'})
    
    if formato == 'html':
        if not request.user.is_authenticated():
            url = '/goldenstreet/login/'
            return HttpResponseRedirect(url)
    correcto, juego = juego_bd.coger(juegoId)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})
    else:
        correcto, equipos = equipo_bd.listar(juego)
        if not correcto:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': equipos})
        
        administrador = organizador_bd.comprobarTipoSession(request)
        if len(equipos) == 0:
            return render_to_response('equipolistar.' + formato, {'code': 200, 'descripcion': 'No hay equipos', 'juego': juego, 'admin': administrador})
        else:
            return render_to_response('equipolistar.' + formato, {'code': 200, 'descripcion': 'ok', 'juego': juego, 'equipos': equipos, 'admin': administrador})

                
def equipo_mostrar(request, juegoId, equipoId):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})

    if request.method != 'GET':
        return render_to_response('fallo.' + formato, {'code': 400, 'description': 'Metodo request invalido'})

    if not request.user.is_authenticated():
        if formato == 'html':
            url = '/goldenstreet/login/'
            return HttpResponseRedirect(url)
        else:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': 'formato incorrecto'})
    else:
        correcto, juego = juego_bd.coger(juegoId)
        if not correcto:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})

        correcto, equipo = equipo_bd.coger(juego, equipoId)
        if not correcto:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': equipo})
            
        correcto, calles = calle_bd.listarPorEquipo(equipo)
        if not correcto:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': calles})
        else: 
            administrador = organizador_bd.comprobarTipoSession(request)
            return render_to_response('equipomostrar.' + formato, {'code': 200, 'descripcion': 'ok', 'juego': juego, 'equipo':equipo, 'calles':calles, 'admin': administrador})


def equipo_modificar(request, juegoId, equipoId):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})
    if not request.user.is_authenticated():
        if formato == 'html':
            url = '/goldenstreet/login/'
            return HttpResponseRedirect(url)
        else:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': 'formato incorrecto'})
    else:
        administrador = organizador_bd.comprobarTipoSession(request)
        if administrador == True:
            if request.method == 'GET':
                correcto, juego = juego_bd.coger(juegoId)
                if not correcto:
                    return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})
                
                correcto, equipo = equipo_bd.coger(juego, equipoId)
                if not correcto:
                    return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': equipo})
                else:
                    return render_to_response('equipomodificar.' + formato, {'code': 200, 'descripcion': 'ok', 'juego': juego, 'equipo': equipo})
            elif request.method == 'POST':
                nombre = request.POST['nombre']
                correcto, juego = juego_bd.coger(juegoId)
                if not correcto:
                    return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})

                correcto, equipo = equipo_bd.coger(juego, equipoId)
                if not correcto:
                    return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': equipo})
                
                correcto, equipoModificado = equipo_bd.modificar(equipo, nombre)
                if not correcto:
                    return render_to_response('fallo.' + formato,  {'code': 500, 'descripcion': equipoModificado})
                else:
                    url = '/goldenstreet/juego/' + str(juegoId) + '/equipo/' + str(equipoModificado.id) + '/mostrar/'
                    return HttpResponseRedirect(url)
        else:
            return render_to_response('fallo.' + formato, {'code': 200, 'descripcion': 'No tienes suficientes permisos para realizar esta accion'})

                
def equipo_borrar(request, juegoId, equipoId):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})
    if not request.user.is_authenticated():
        if formato == 'html':
            url = '/goldenstreet/login/'
            return HttpResponseRedirect(url)
        else:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': 'formato incorrecto'})
    else:
        administrador = organizador_bd.comprobarTipoSession(request)
        if administrador == True:
            correcto, juego = juego_bd.coger(juegoId)
            if not correcto:
                return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})
               
            correcto, equipo = equipo_bd.coger(juego, equipoId)
            if not correcto:
                return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': equipo})
            
            usuarioId = request.session['_auth_user_id']
            correcto, msgerror = equipo_bd.borrar(equipo, usuarioId)
            if not correcto:
                return render_to_response('fallo.' + formato, {'code': 500, 'descripcion': msgerror})
            else:
                url = '/goldenstreet/juego/' + str(juegoId) + '/equipo/listar/'
                return HttpResponseRedirect(url)
        else:
            return render_to_response('fallo.' + formato, {'code': 200, 'descripcion': 'No tienes suficientes permisos para realizar esta accion'})


def equipo_posicionar(request, juegoId, equipoId):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})

    if request.method != 'POST':
        return render_to_response('fallo.' + formato, {'code': 400, 'description': 'Metodo request invalido'})
    
    if formato == 'html':
        url = '/goldenstreet/login/'
        return HttpResponseRedirect(url)
    correcto, juego = juego_bd.coger(juegoId)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})
    
    correcto, equipo = equipo_bd.coger(juego, equipoId)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': equipo})
   
    equipoLatitud = float(request.POST['latitud'])
    equipoLongitud = float(request.POST['longitud'])
    ubicacion = Point(equipoLongitud, equipoLatitud, srid=4326)
    correcto, equipoPosicionado = equipo_bd.modificarPosicion(equipo, ubicacion)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 500, 'descripcion': equipoPosicionado})
    else:
        return render_to_response('respuesta.' + formato, {'code': 200, 'descripcion': 'ok'})

       
#----Equipo-Patrimonio----           
def equipo_patrimonio_mostrar(request, juegoId, equipoId):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})

    if request.method != 'GET':
        return render_to_response('fallo.' + formato, {'code': 400, 'description': 'Metodo request invalido'})

    if formato == 'html':
        if not request.user.is_authenticated():        
            url = '/goldenstreet/login/'
            return HttpResponseRedirect(url)
    correcto, juego = juego_bd.coger(juegoId)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})
    
    correcto, equipo = equipo_bd.coger(juego, equipoId)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': equipo})
    
    correcto, patrimonio = patrimonio_bd.coger(equipo)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': patrimonio})
    else: 
        administrador = organizador_bd.comprobarTipoSession(request)
        return render_to_response('patrimoniomostrar.' + formato, {'code': 200, 'descripcion': 'ok', 'juego': juego, 'equipo':equipo, 'patrimonio':patrimonio, 'admin': administrador})



def equipo_patrimonio_modificar(request, juegoId, equipoId):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})

    if request.method != 'GET':
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': 'Metodo request invalido'})

    if formato == 'html':
        url = '/goldenstreet/login/'
        return HttpResponseRedirect(url)
    correcto, juego = juego_bd.coger(juegoId)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})
    
    correcto, equipo = equipo_bd.coger(juego, equipoId)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': equipo})
    
    correcto, patrimonio = patrimonio_bd.coger(equipo)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': patrimonio})
    
    correcto, edificios = edificio_bd.listarPorEquipo(equipo)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': edificios})
                                        
    correcto, patrimonioModificado = patrimonio_bd.modificarPorPropiedades(patrimonio, edificios)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 500, 'descripcion': patrimonioModificado})
    else:
        return render_to_response('respuesta.' + formato, {'code': 200, 'descripcion': 'ok'})

        
#----Equipo-Mensaje----
def equipo_mensaje_listar(request, juegoId, equipoId):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})

    if request.method != 'GET':
        return render_to_response('fallo.' + formato, {'code': 400, 'description': 'Metodo request invalido'})

    if formato == 'html':
        if not request.user.is_authenticated():
            url = '/goldenstreet/login/'
            return HttpResponseRedirect(url)
    correcto, juego = juego_bd.coger(juegoId)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})
    
    correcto, equipo = equipo_bd.coger(juego, equipoId)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': equipo})
    
    correcto, mensajes = mensaje_bd.listarPorEquipo(juego, equipo)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': mensajes})
    
    administrador = organizador_bd.comprobarTipoSession(request)
    if len(mensajes) == 0:
        return render_to_response('mensajelistar.' + formato, {'code': 200, 'descripcion': 'No hay mensajes'})
    else:
        return render_to_response('mensajelistar.' + formato, {'code': 200, 'descripcion': 'ok', 'mensajes': mensajes})
                

def equipo_mensaje_historico(request, juegoId, equipoId):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})

    if request.method != 'GET':
        return render_to_response('fallo.' + formato, {'code': 400, 'description': 'Metodo request invalido'})

    if formato == 'html':
        url = '/goldenstreet/login/'
        return HttpResponseRedirect(url)
    correcto, juego = juego_bd.coger(juegoId)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})

    correcto, equipo = equipo_bd.coger(juego, equipoId)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': equipo})

    correcto, mensajesRecibidos = mensaje_bd.listarRecibidosPorEquipo(juego, equipo)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': ofertasRespondidas})

    if len(mensajesRecibidos) == 0:
        ultMensajeRecibidoId = 0
    else:
        ultMensajeRecibidoId = mensajesRecibidos[0].id

    return render_to_response('mensajehistorico.' + formato, {'code': 200, 'descripcion': 'ok', 'ultMensajeRecibidoId': ultMensajeRecibidoId})


#----Equipo-Oferta----
def equipo_oferta_listar(request, juegoId, equipoId, tipoOferta):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})

    if request.method != 'GET':
        return render_to_response('fallo.' + formato, {'code': 400, 'description': 'Metodo request invalido'})

    if formato == 'html':
        url = '/goldenstreet/login/'
        return HttpResponseRedirect(url)
    correcto, juego = juego_bd.coger(juegoId)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})

    correcto, equipo = equipo_bd.coger(juego, equipoId)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': equipo})
   
    if tipoOferta == 'enviada':
        correcto, ofertas = oferta_bd.listarPorEquipoEnviada(juego, equipo)
    else:
        correcto, ofertas = oferta_bd.listarPorEquipoRecibida(juego, equipo)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': ofertas})
    
    if len(ofertas) == 0:
        return render_to_response('ofertalistar.' + formato, {'code': 200, 'descripcion': 'No hay ofertas'})
    else:
        return render_to_response('ofertalistar.' + formato, {'code': 200, 'descripcion': 'ok', 'ofertas': ofertas})
                

def equipo_oferta_historico(request, juegoId, equipoId):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})

    if request.method != 'GET':
        return render_to_response('fallo.' + formato, {'code': 400, 'description': 'Metodo request invalido'})

    if formato == 'html':
        url = '/goldenstreet/login/'
        return HttpResponseRedirect(url)
    correcto, juego = juego_bd.coger(juegoId)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})

    correcto, equipo = equipo_bd.coger(juego, equipoId)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': equipo})
   
    correcto, ofertasRespondidas = oferta_bd.listarPorEquipoRespondida(juego, equipo)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': ofertasRespondidas})

    correcto, ofertasRecibidas = oferta_bd.listarPorEquipoRecibida(juego, equipo)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': ofertas})
    
    numOfertasRespondidas = len(ofertasRespondidas)
    if len(ofertasRecibidas) == 0:
        ultOfertaRecibida = 0
    else:
        ultOfertaRecibida = ofertasRecibidas[0].id

    return render_to_response('ofertahistorico.' + formato, {'code': 200, 'descripcion': 'ok', 'numOfertasRespondidas': numOfertasRespondidas, 'ultOfertaRecibida': ultOfertaRecibida})

    

#----Equipo-Carta----
def equipo_carta_usar(request, juegoId, equipoId):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})

    if request.method != 'POST':
        return render_to_response('fallo.' + formato, {'code': 400, 'description': 'Metodo request invalido'})

    if formato == 'html':
        url = '/goldenstreet/login/'
        return HttpResponseRedirect(url)
    correcto, juego = juego_bd.coger(juegoId)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})
    
    correcto, equipo = equipo_bd.coger(juego, equipoId)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': equipo})
   
    correcto, patrimonio = patrimonio_bd.coger(equipo)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': patrimonio})
    
    cantidad = int(request.POST['cantidad'])
    tipoAccion = request.POST['tipoAccion']
    correcto, patrimonioModificado = patrimonio_bd.modificarPorCarta(patrimonio, cantidad, tipoAccion);
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': patrimonioModificado})
    else:
        return render_to_response('respuesta.' + formato, {'code': 200, 'descripcion': 'ok'})


#----Equipo-Calle----
def equipo_calle_comprar(request, juegoId, equipoId, calleId):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})

    if request.method != 'POST':
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': 'Metodo request invalido'})

    if formato == 'html':
        url = '/goldenstreet/login/'
        return HttpResponseRedirect(url)
    correcto, juego = juego_bd.coger(juegoId)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})
    
    correcto, equipo = equipo_bd.coger(juego, equipoId)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': equipo})
    
    correcto, patrimonio = patrimonio_bd.coger(equipo)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': patrimonio})
   
    callePrecio = int(request.POST['callePrecio'].strip())
    if callePrecio > patrimonio.dinero:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': 'Sin dinero'})
    else:
        correcto, resultado = calle_bd.comprar(juego, calleId, patrimonio, callePrecio)
        if not correcto:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': resultado})
        else:
            return render_to_response('respuesta.' + formato, {'code': 200, 'descripcion': resultado})


def equipo_calle_construir(request, juegoId, equipoId, calleId):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})

    if request.method != 'POST':
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': 'Metodo request invalido'})

    if formato == 'html':
        url = '/goldenstreet/login/'
        return HttpResponseRedirect(url)
    correcto, juego = juego_bd.coger(juegoId)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})
    
    correcto, equipo = equipo_bd.coger(juego, equipoId)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': equipo})
    
    correcto, patrimonio = patrimonio_bd.coger(equipo)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': patrimonio})
    
    edificioPrecio = int(request.POST['edificioPrecio'].strip())
    edificioPrecioOriginal = int(request.POST['edificioPrecioOriginal'].strip())
    if edificioPrecio > patrimonio.dinero:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': 'Sin dinero'})
    else:
        correcto, resultado = calle_bd.construir(juego, calleId, patrimonio, edificioPrecio, edificioPrecioOriginal)
        if not correcto:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': resultado})
        else:
            return render_to_response('respuesta.' + formato, {'code': 200, 'descripcion': resultado})


def equipo_calle_sabotear(request, juegoId, equipoId, calleId):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})

    if request.method != 'POST':
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': 'Metodo request invalido'})

    if formato == 'html':
        url = '/goldenstreet/login/'
        return HttpResponseRedirect(url)
    correcto, juego = juego_bd.coger(juegoId)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})
           
    correcto, resultado = calle_bd.sabotear(juego, calleId)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': resultado})
    else:
        return render_to_response('respuesta.' + formato, {'code': 200, 'descripcion': resultado})
        

def equipo_calle_demoler(request, juegoId, equipoId, calleId):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})

    if request.method != 'POST':
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': 'Metodo request invalido'})

    if formato == 'html':
        url = '/goldenstreet/login/'
        return HttpResponseRedirect(url)
    correcto, juego = juego_bd.coger(juegoId)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})
   
    correcto, equipo = equipo_bd.coger(juego, equipoId)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': equipo})
    
    correcto, patrimonio = patrimonio_bd.coger(equipo)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': patrimonio})
    
    edificioValor = int(request.POST['edificioValor'].strip())
    correcto, resultado = calle_bd.demoler(juego, calleId, patrimonio, edificioValor)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': resultado})
    else:
        return render_to_response('respuesta.' + formato, {'code': 200, 'descripcion': resultado})


def equipo_calle_ofertar(request, juegoId, equipoId, calleId):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})
    
    if formato == 'html':
        url = '/goldenstreet/login/'
        return HttpResponseRedirect(url)
    correcto, juego = juego_bd.coger(juegoId)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})

    correcto, ofertante = equipo_bd.coger(juego, equipoId)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': ofertante})
    
    correcto, calle = calle_bd.coger(juego, calleId)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': calle})
    
    if request.method == 'GET':
        correcto, patrimonio = patrimonio_bd.coger(ofertante)
        if not correcto:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': patrimonio})
        
        correcto, edificios = edificio_bd.listar(calle)
        if not correcto:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': edificios})
        else: 
            return render_to_response('equipoofertar.' + formato, {'code': 200, 'descripcion': 'ok', 'calle': calle, 'edificios': edificios, 'patrimonio':patrimonio})

    if request.method == 'POST':
        propietarioId = request.POST['propietarioId'].strip()
        ofertaCantidad = int(request.POST['ofertaCantidad'].strip())
        calleVersion = int(request.POST['calleVersion'].strip())

        correcto, propietario = equipo_bd.coger(juego, propietarioId)
        if not correcto:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': propietario})
        
        correcto, resultado = calle_bd.ofertar(juego, calleId, calleVersion, calle, ofertaCantidad, ofertante, propietario)
        if not correcto:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': resultado})
        else:
            return render_to_response('respuesta.' + formato, {'code': 200, 'descripcion': resultado})


def equipo_oferta_aceptar(request, juegoId, equipoId, ofertaId):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})
    
    if formato == 'html':
        url = '/goldenstreet/login/'
        return HttpResponseRedirect(url)

    if request.method != 'POST':
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': 'Metodo request invalido'})
    
    correcto, juego = juego_bd.coger(juegoId)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})

    correcto, oferta = oferta_bd.coger(ofertaId)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': oferta})    

    correcto, propietario = equipo_bd.coger(juego, equipoId)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': propietario})

    correcto, propietarioMarcador = patrimonio_bd.coger(propietario)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': propietarioMarcador})
    
    calleVersion = request.POST['calleVersion'].strip()
    
    correcto, ofertante = equipo_bd.coger(juego, oferta.ofertante.id)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': ofertante})

    correcto, ofertanteMarcador = patrimonio_bd.coger(ofertante)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': ofertanteMarcador})

    if oferta.cantidad > ofertanteMarcador.dinero:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': 'Sin dinero'})

    correcto, calle = calle_bd.coger(juego, oferta.calle.id)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': calle})
    
    correcto, resultado = oferta_bd.aceptar(juego, oferta, calleVersion, propietarioMarcador, ofertanteMarcador)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': resultado})
    else:
        return render_to_response('respuesta.' + formato, {'code': 200, 'descripcion': resultado})

        
def equipo_oferta_rechazar(request, juegoId, equipoId, ofertaId):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})
    
    if formato == 'html':
        url = '/goldenstreet/login/'
        return HttpResponseRedirect(url)

    correcto, juego = juego_bd.coger(juegoId)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})
    
    correcto, resultado = oferta_bd.rechazar(juego, ofertaId)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': resultado})
    else:
        return render_to_response('respuesta.' + formato, {'code': 200, 'descripcion': resultado})


def equipo_oferta_retirar(request, juegoId, equipoId, ofertaId):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})
    
    if formato == 'html':
        url = '/goldenstreet/login/'
        return HttpResponseRedirect(url)

    correcto, juego = juego_bd.coger(juegoId)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})
    
    correcto, resultado = oferta_bd.retirar(juego, ofertaId)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': resultado})
    else:
        return render_to_response('respuesta.' + formato, {'code': 200, 'descripcion': resultado})


#----Calle----
def calle_crear(request, juegoId):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})
    if not request.user.is_authenticated():
        if formato == 'html':
            url = '/goldenstreet/login/'
            return HttpResponseRedirect(url)
        else:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': 'formato incorrecto'})
    else:
        administrador = organizador_bd.comprobarTipoSession(request)
        if administrador == True:
            if request.method == 'GET':
                correcto, juego = juego_bd.coger(juegoId)
                if not correcto:
                    return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})
                else:
                    return render_to_response('callecrear.' + formato, {'code': 200, 'descripcion': 'ok', 'juego': juego})
            elif request.method == 'POST':
                nombre = request.POST['nombre'].strip()
                latitud = float(request.POST['latitud'].strip().replace(',', '.'))
                longitud = float(request.POST['longitud'].strip().replace(',', '.'))
                ubicacion = Point(longitud, latitud, srid=4326)
                precio = request.POST['precio']
                correcto, juego = juego_bd.coger(juegoId)
                if not correcto:
                    return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})
                
                correcto, nuevaCalle = calle_bd.crear(nombre, ubicacion, precio, juego)
                if not correcto:
                    return render_to_response('fallo.' + formato,  {'code': 500, 'descripcion': nuevaCalle})
                else:
                    url = '/goldenstreet/juego/' + str(juegoId) + '/calle/' + str(nuevaCalle.id) + '/mostrar/'
                    return HttpResponseRedirect(url)
        else:
            return render_to_response('fallo.' + formato, {'code': 200, 'descripcion': 'No tienes suficientes permisos para realizar esta accion'})


def calle_listar(request, juegoId):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})

    if request.method != 'GET':
        return render_to_response('fallo.' + formato, {'code': 400, 'description': 'Metodo request invalido'})

    if formato == 'html':
        if not request.user.is_authenticated():
            url = '/goldenstreet/login/'
            return HttpResponseRedirect(url)
    correcto, juego = juego_bd.coger(juegoId)
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})
    else:
        correcto, calles = calle_bd.listar(juego)
        if not correcto:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': calles})
        
        administrador = organizador_bd.comprobarTipoSession(request)
        if len(calles) == 0:
            return render_to_response('callelistar.' + formato, {'code': 200, 'descripcion': 'No hay calles', 'juego': juego, 'admin': administrador})
        else:
            return render_to_response('callelistar.' + formato, {'code': 200, 'descripcion': 'ok', 'juego': juego, 'calles': calles, 'admin': administrador})

        
def calle_mostrar(request, juegoId, calleId):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})

    if request.method != 'GET':
        return render_to_response('fallo.' + formato, {'code': 400, 'description': 'Metodo request invalido'})

    if not request.user.is_authenticated():
        if formato == 'html':
            url = '/goldenstreet/login/'
            return HttpResponseRedirect(url)
        else:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': 'formato incorrecto'})
    else:
        correcto, juego = juego_bd.coger(juegoId)
        if not correcto:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})
        
        correcto, calle = calle_bd.coger(juego, calleId)
        if not correcto:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': calle})
        else: 
            administrador = organizador_bd.comprobarTipoSession(request)
            return render_to_response('callemostrar.' + formato, {'code': 200, 'descripcion': 'ok', 'juego': juego, 'calle':calle, 'admin': administrador})


def calle_modificar(request, juegoId, calleId):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})
    if not request.user.is_authenticated():
        if formato == 'html':
            url = '/goldenstreet/login/'
            return HttpResponseRedirect(url)
        else:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': 'formato incorrecto'})
    else:
        administrador = organizador_bd.comprobarTipoSession(request)
        if administrador == True:
            if request.method == 'GET':
                correcto, juego = juego_bd.coger(juegoId)
                if not correcto:
                    return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})
               
                correcto, calle = calle_bd.coger(juego, calleId)
                if not correcto:
                    return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': calle})
                else:
                    return render_to_response('callemodificar.' + formato, {'code': 200, 'descripcion': 'ok', 'juego': juego, 'calle': calle})
            elif request.method == 'POST':
                nombre = request.POST['nombre'].strip()
                latitud = float(request.POST['latitud'].strip().replace(',', '.'))
                longitud = float(request.POST['longitud'].strip().replace(',', '.'))
                ubicacion = Point(longitud, latitud, srid=4326)
                precio = request.POST['precio']
                correcto, juego = juego_bd.coger(juegoId)
                if not correcto:
                    return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})
                else:
                    correcto, calle = calle_bd.coger(juego, calleId)
                    if not correcto:
                        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': calle})
                    
                    correcto, calleModificada = calle_bd.modificar(calle, nombre, ubicacion, precio)
                    if not correcto:
                        return render_to_response('fallo.' + formato,  {'code': 500, 'descripcion': calleModificada})
                    else:
                        url = '/goldenstreet/juego/' + str(juegoId) + '/calle/' + str(calleModificada.id) + '/mostrar/'
                        return HttpResponseRedirect(url)
        else:
            return render_to_response('fallo.' + formato, {'code': 200, 'descripcion': 'No tienes suficientes permisos para realizar esta accion'})


def calle_borrar(request, juegoId, calleId):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})
    if not request.user.is_authenticated():
        if formato == 'html':
            url = '/goldenstreet/login/'
            return HttpResponseRedirect(url)
        else:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': 'formato incorrecto'})
    else:
        administrador = organizador_bd.comprobarTipoSession(request)
        if administrador == True:
            correcto, juego = juego_bd.coger(juegoId)
            if not correcto:
                return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})
             
            correcto, calle = calle_bd.coger(juego, calleId)
            if not correcto:
                return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': calle})
            
            usuarioId = request.session['_auth_user_id']
            correcto, msgerror = calle_bd.borrar(calle, usuarioId)
            if not correcto:
                return render_to_response('fallo.' + formato, {'code': 500, 'descripcion': msgerror})
            else:
                url = '/goldenstreet/juego/' + str(juegoId) + '/calle/listar/'
                return HttpResponseRedirect(url)
        else:
            return render_to_response('fallo.' + formato, {'code': 200, 'descripcion': 'No tienes suficientes permisos para realizar esta accion'})
        

#----Calle-Edificio----
def edificio_crear(request, juegoId, calleId):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})
    if not request.user.is_authenticated():
        if formato == 'html':
            url = '/goldenstreet/login/'
            return HttpResponseRedirect(url)
        else:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': 'formato incorrecto'})
    else:
        administrador = organizador_bd.comprobarTipoSession(request)
        if administrador == True:
            correcto, juego = juego_bd.coger(juegoId)
            if not correcto:
                return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})
            
            correcto, calle = calle_bd.coger(juego, calleId)
            if not correcto:
                return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': calle})
            
            correcto, edificios = edificio_bd.listar(calle)
            if not correcto:
                return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': edificios})
                                  
            if len(edificios) == 0:
                if request.method == 'GET':
                    return render_to_response('edificiocrear.' + formato, {'code': 200, 'descripcion': 'ok', 'juego': juego, 'calle': calle})
                elif request.method == 'POST':
                    nombre = request.POST['nombre']
                    bonificacion = request.POST['bonificacion']
                    precio = request.POST['precio']
                    correcto, juego = juego_bd.coger(juegoId)
                    if not correcto:
                        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})
                    
                    correcto, nuevoEdificio = edificio_bd.crear(nombre, bonificacion, precio, calle, juego)
                    if not correcto:
                        return render_to_response('fallo.' + formato,  {'code': 500, 'descripcion': nuevoEdificio})
                    else:
                        url = '/goldenstreet/juego/' + str(juegoId) + '/calle/' + str(calleId) + '/edificio/' + str(nuevoEdificio.id) + '/mostrar/'
                        return HttpResponseRedirect(url)
            else:
                return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': 'No se permite la creacion de mas edificios.'})
        else:
            return render_to_response('fallo.' + formato, {'code': 200, 'descripcion': 'No tienes suficientes permisos para realizar esta accion'})
                    

def edificio_listar(request, juegoId, calleId):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})

    if request.method != 'GET':
        return render_to_response('fallo.' + formato, {'code': 400, 'description': 'Metodo request invalido'})

    if not request.user.is_authenticated():
        if formato == 'html':
            url = '/goldenstreet/login/'
            return HttpResponseRedirect(url)
        else:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': 'formato incorrecto'})
    else:
        correcto, juego = juego_bd.coger(juegoId)
        if not correcto:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})
        
        correcto, calle = calle_bd.coger(juego, calleId)
        if not correcto:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': calles})
    
        correcto, edificios = edificio_bd.listar(calle)
        if not correcto:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': edificios})
        
        administrador = organizador_bd.comprobarTipoSession(request)
        if len(edificios) == 0:
            return render_to_response('edificiolistar.' + formato, {'code': 200, 'descripcion': 'No hay edificios', 'juego': juego, 'calle': calle, 'admin': administrador})
        else:
            return render_to_response('edificiolistar.' + formato, {'code': 200, 'descripcion': 'ok', 'juego': juego, 'calle': calle, 'edificios': edificios, 'admin': administrador})


def edificio_mostrar(request, juegoId, calleId, edificioId):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})

    if request.method != 'GET':
        return render_to_response('fallo.' + formato, {'code': 400, 'description': 'Metodo request invalido'})

    if not request.user.is_authenticated():
        if formato == 'html':
            url = '/goldenstreet/login/'
            return HttpResponseRedirect(url)
        else:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': 'formato incorrecto'})
    else:
        correcto, juego = juego_bd.coger(juegoId)
        if not correcto:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})
        
        correcto, calle = calle_bd.coger(juego, calleId)
        if not correcto:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': calle})
        
        correcto, edificio = edificio_bd.coger(calle, edificioId)
        if not correcto:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': edificio})
        else:
            administrador = organizador_bd.comprobarTipoSession(request)
            return render_to_response('edificiomostrar.' + formato, {'code': 200, 'descripcion': 'ok', 'juego': juego, 'calle':calle, 'edificio':edificio, 'admin': administrador})


def edificio_modificar(request, juegoId, calleId, edificioId):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})
    if not request.user.is_authenticated():
        if formato == 'html':
            url = '/goldenstreet/login/'
            return HttpResponseRedirect(url)
        else:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': 'formato incorrecto'})
    else:
        administrador = organizador_bd.comprobarTipoSession(request)
        if administrador == True:
            if request.method == 'GET':
                correcto, juego = juego_bd.coger(juegoId)
                if not correcto:
                    return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})
                
                correcto, calle = calle_bd.coger(juego, calleId)
                if not correcto:
                    return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': calle})
                
                correcto, edificio = edificio_bd.coger(calle, edificioId)
                if not correcto:
                    return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': edificio})
                else:
                    return render_to_response('edificiomodificar.' + formato, {'code': 200, 'descripcion': 'ok', 'juego': juego, 'calle': calle, 'edificio':edificio})

            elif request.method == 'POST':
                nombre = request.POST['nombre']
                bonificacion = request.POST['bonificacion']
                precio = request.POST['precio']
                correcto, juego = juego_bd.coger(juegoId)
                if not correcto:
                    return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})
               
                correcto, calle = calle_bd.coger(juego, calleId)
                if not correcto:
                    return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': calle})
                
                correcto, edificio = edificio_bd.coger(calle, edificioId)
                if not correcto:
                    return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': edificio})
                
                correcto, edificioModificado = edificio_bd.modificar(edificio, nombre, bonificacion, precio)
                if not correcto:
                    return render_to_response('fallo.' + formato,  {'code': 500, 'descripcion': edificioModificado})
                else:
                    url = '/goldenstreet/juego/' + str(juegoId) + '/calle/' + str(calleId) + '/edificio/' + str(edificioModificado.id) + '/mostrar/'
                    return HttpResponseRedirect(url)
        else:
            return render_to_response('fallo.' + formato, {'code': 200, 'descripcion': 'No tienes suficientes permisos para realizar esta accion'})

                    
def edificio_borrar(request, juegoId, calleId, edificioId):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})
    if not request.user.is_authenticated():
        if formato == 'html':
            url = '/goldenstreet/login/'
            return HttpResponseRedirect(url)
        else:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': 'formato incorrecto'})
    else:
        administrador = organizador_bd.comprobarTipoSession(request)
        if administrador == True:
            correcto, juego = juego_bd.coger(juegoId)
            if not correcto:
                return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': juego})
               
            correcto, calle = calle_bd.coger(juego, calleId)
            if not correcto:
                return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': calle})
               
            correcto, edificio = edificio_bd.coger(calle, edificioId)
            if not correcto:
                return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': edificio})
            
            correcto, msgerror = edificio_bd.borrar(edificio)
            if not correcto:
                return render_to_response('fallo.' + formato, {'code': 500, 'descripcion': msgerror})
            else:
                url = '/goldenstreet/juego/' + str(juegoId) + '/calle/' + str(calleId) + '/edificio/listar/'
                return HttpResponseRedirect(url)
        else:
            return render_to_response('fallo.' + formato, {'code': 200, 'descripcion': 'No tienes suficientes permisos para realizar esta accion'})


#----Carta----
def carta_crear(request):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})
    if not request.user.is_authenticated():
        if formato == 'html':
            url = '/goldenstreet/login/'
            return HttpResponseRedirect(url)
        else:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': 'formato incorrecto'})
    else:
        administrador = organizador_bd.comprobarTipoSession(request)
        if administrador == True:
            if request.method == 'GET':
                    return render_to_response('cartacrear.' + formato, {'code': 200, 'descripcion': 'ok', 'admin': administrador})
            elif request.method == 'POST':
                nombre = request.POST['nombre']
                descripcion = request.POST['descripcion']
                tipoAccion = request.POST['tipoAccion']
                cantidad = '0'
                if tipoAccion == '1' or tipoAccion == '2':
                    cantidad = request.POST['cantidad']
                correcto, nuevaCarta = carta_bd.crear(nombre, descripcion, tipoAccion, cantidad)
                if not correcto:
                    return render_to_response('fallo.' + formato,  {'code': 500, 'descripcion': nuevaCarta})
                else:
                    url = '/goldenstreet/carta/' + str(nuevaCarta.id) + '/mostrar/'
                    return HttpResponseRedirect(url)
        else:
            return render_to_response('fallo.' + formato, {'code': 200, 'descripcion': 'No tienes suficientes permisos para realizar esta accion'})
        

def carta_listar(request):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})

    if request.method != 'GET':
        return render_to_response('fallo.' + formato, {'code': 400, 'description': 'Metodo request invalido'})

    if formato == 'html':
        if not request.user.is_authenticated():
            url = '/goldenstreet/login/'
            return HttpResponseRedirect(url)
    correcto, cartas = carta_bd.listar()
    if not correcto:
        return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': cartas})
    
    administrador = organizador_bd.comprobarTipoSession(request)
    if len(cartas) == 0:
        return render_to_response('cartalistar.' + formato, {'code': 200, 'descripcion': 'No hay cartas', 'admin': administrador})
    else:
        return render_to_response('cartalistar.' + formato, {'code': 200, 'descripcion': 'ok', 'cartas': cartas, 'admin': administrador})

        
def carta_mostrar(request, cartaId):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})

    if request.method != 'GET':
        return render_to_response('fallo.' + formato, {'code': 400, 'description': 'Metodo request invalido'})

    if not request.user.is_authenticated():
        if formato == 'html':
            url = '/goldenstreet/login/'
            return HttpResponseRedirect(url)
        else:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': 'formato incorrecto'})
    else:
        correcto, carta = carta_bd.coger(cartaId)
        if not correcto:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': carta})
        else: 
            administrador = organizador_bd.comprobarTipoSession(request)
            return render_to_response('cartamostrar.' + formato, {'code': 200, 'descripcion': 'ok', 'carta':carta, 'admin': administrador})


def carta_modificar(request, cartaId):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})
    if not request.user.is_authenticated():
        if formato == 'html':
            url = '/goldenstreet/login/'
            return HttpResponseRedirect(url)
        else:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': 'formato incorrecto'})
    else:
        administrador = organizador_bd.comprobarTipoSession(request)
        if administrador == True:
            if request.method == 'GET':
                correcto, carta = carta_bd.coger(cartaId)
                if not correcto:
                    return render_to_response('fallo.' + formato,  {'code': 400, 'descripcion': carta})
                else:
                    return render_to_response('cartamodificar.' + formato, {'code': 200, 'descripcion': 'ok', 'carta': carta})
            elif request.method == 'POST':
                nombre = request.POST['nombre']
                descripcion = request.POST['descripcion']
                tipoAccion = request.POST['tipoAccion']
                cantidad = '0'
                if tipoAccion == '1' or tipoAccion == '1':
                    cantidad = request.POST['cantidad']
                correcto, carta = carta_bd.coger(cartaId)
                if not correcto:
                    return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': carta})
                
                correcto, cartaModificada = carta_bd.modificar(carta, nombre, descripcion, tipoAccion, cantidad)
                if not correcto:
                    return render_to_response('fallo.' + formato,  {'code': 500, 'descripcion': cartaModificada})
                else:
                    url = '/goldenstreet/carta/' + str(cartaModificada.id) + '/mostrar/'
                    return HttpResponseRedirect(url)
        else:
            return render_to_response('fallo.' + formato, {'code': 200, 'descripcion': 'No tienes suficientes permisos para realizar esta accion'})

            
def carta_borrar(request, cartaId):
    correcto, formato = comprobar_formato(request)
    if not correcto:
        return render_to_response('fallo.html', {'code': 400, 'descripcion': formato})
    if not request.user.is_authenticated():
        if formato == 'html':
            url = '/goldenstreet/login/'
            return HttpResponseRedirect(url)
        else:
            return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': 'formato incorrecto'})
    else:
        administrador = organizador_bd.comprobarTipoSession(request)
        if administrador == True:
            correcto, carta = carta_bd.coger(cartaId)
            if not correcto:
                return render_to_response('fallo.' + formato, {'code': 400, 'descripcion': carta})
        
            correcto, msgerror = carta_bd.borrar(carta)
            if not correcto:
                return render_to_response('fallo.' + formato, {'code': 500, 'descripcion': msgerror})
            else:
                url = '/goldenstreet/carta/listar/'
                return HttpResponseRedirect(url)
        else:
            return render_to_response('fallo.' + formato, {'code': 200, 'descripcion': 'No tienes suficientes permisos para realizar esta accion'})
